package com.baijiayun.groupclassui.base;

import android.content.Context;
import androidx.annotation.LayoutRes;
import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import java.util.List;

/**
 * RecyclerView Adapter 抽象基类
 * Created by Dujuncan on 2017/1/3.
 */
public abstract class BaseRecyclerAdapter<T> extends RecyclerView.Adapter<BaseViewHolder> {
    protected Context mContext;
    private RecyclerView recyclerView;
    private SparseArray<OnItemClickListener> mOnItemClickListeners;//缓存所有点击事件
    private SparseArray<OnItemLongClickListener> mOnItemLongClickListeners;//缓存所有长按事件
    private OnItemClickListener mOnItemClickListener;//item点击事件
    private OnItemLongClickListener mOnItemLongClickListener;
    private List<T> mList;
    private int mResId;

    public BaseRecyclerAdapter(Context context, List<T> list, @LayoutRes int resId) {
        mContext = context;
        mList = list;
        mResId = resId;
        mOnItemClickListeners = new SparseArray<>();
        mOnItemLongClickListeners = new SparseArray<>();
    }

    public BaseRecyclerAdapter(Context context, @LayoutRes int resId) {
        this(context, null, resId);
    }

    @Override
    public int getItemCount() {
        return mList != null ? mList.size() : 0;
    }

    @NonNull
    @Override
    public BaseViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(mContext).inflate(mResId, parent, false);
        return new BaseViewHolder(mContext, view);
    }

    @Override
    public void onBindViewHolder(@NonNull BaseViewHolder holder, int position) {
        convert(holder, mList.get(position));
    }

    protected abstract void convert(BaseViewHolder holder, T t);

    public void bindToRecyclerView(RecyclerView recyclerView) {
        this.recyclerView = recyclerView;
        recyclerView.setAdapter(this);
    }

    /**
     * 是否监听item及childView
     *
     * @return true为监听，false为禁止监听
     */
    protected abstract boolean isEnableListener();

    public void setData(List<T> list) {
        if (mList != null) mList.clear();
        mList = list;
        notifyDataSetChanged();
    }

    public List<T> getData() {
        return mList;
    }

    public void addData(T t) {
        if (mList == null || t == null) return;
        mList.add(t);
        notifyItemInserted(mList.size() - 1);
    }

    public void addData(List<T> list) {
        if (mList == null || list == null) return;
        mList.addAll(list);
        notifyItemRangeChanged(mList.size() - list.size(), list.size());
    }

    public void removeData(int position) {
        if (null == mList) return;
        if (mList.size() == 0 || position >= mList.size()) return;
        mList.remove(position);
        notifyItemRemoved(position);
    }

    public void removeData(T t) {
        if (null == mList) return;
        int position = mList.indexOf(t);

        if (position == -1) return;
        mList.remove(position);
        //由于removed是局部刷新，不会更新数据，会造成数据混乱，暂时用全局刷新。
        notifyDataSetChanged();
    }

    public void removeData(List<T> list) {
        if (null == mList) return;
        if (!mList.containsAll(list)) return;
        mList.removeAll(list);
        notifyDataSetChanged();
    }

    public interface OnChildClickListener {
        void onClick(View view, int position);
    }

    public interface OnItemClickListener {
        void onItemCLick(View view, int position);
    }

    public interface OnItemLongClickListener {
        void onItemLongClick(View view, int position);
    }

    public interface OnItemTouchListener {
        void onItmTouch(int rawX, int rawY);
    }

    public void setOnItemClickListener(OnItemClickListener listener) {
        mOnItemClickListener = listener;
    }

    public void setOnItemLongClickListener(OnItemLongClickListener listener) {
        mOnItemLongClickListener = listener;
    }
}
