package com.baijiayun.groupclassui.container

import android.content.Context
import android.graphics.Color
import android.util.AttributeSet
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.constraintlayout.widget.ConstraintSet
import com.baijiayun.bjyutils.drawable.DrawableBuilder
import com.baijiayun.bjyutils.kt.dp
import com.baijiayun.bjyutils.reactivex.RxUtils
import com.baijiayun.groupclassui.R
import com.baijiayun.groupclassui.global.EventKey
import com.baijiayun.groupclassui.layer.BaseLayerOfProfessionalClass
import com.baijiayun.groupclassui.layer.StudyRoomLayer
import com.baijiayun.groupclassui.layer.StudyRoomPlaceholderLayer
import com.baijiayun.groupclassui.layer.VideoDragLayer
import com.baijiayun.livebase.utils.ThemeDataUtil
import com.baijiayun.livebase.context.LPConstants
import com.baijiayun.livecore.models.LPStudyUserStatus
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import kotlinx.android.synthetic.main.bjysc_layout_video_empty.view.*
import java.util.Collections.sort
import java.util.concurrent.TimeUnit

/**
 * Created by pengwenjian on 2020/12/4
 * Describe: 自习室视频容器
 */
class StudyRoomContainer : BaseLayerOfProfessionalClass, VideoContainerListener {
    private val TAG = "StudyRoomContainer"
    private var studyRoomMode = LPConstants.StudyRoomMode.SelfStudy
    private var studyRoomLayer: StudyRoomLayer? = null
    private var placeholderContainer: ViewGroup? = null
    private var videoDragLayer: VideoDragLayer? = null
    private var disposableOfFullSeat: Disposable? = null
    private var disposableOfTimer: Disposable? = null
    private var imgBtn: ImageView? = null
    private var text: TextView? = null
    private var isFullSeat = false

    constructor(ctx: Context) : super(ctx)
    constructor(ctx: Context, attributeSet: AttributeSet?) : super(ctx, attributeSet)
    constructor(ctx: Context, attributeSet: AttributeSet?, defStyleAttr: Int) : super(ctx, attributeSet, defStyleAttr)

    override fun onRoomInActive() {
        super.onRoomInActive()
        release()
    }

    override fun onRoomActive() {
        super.onRoomActive()
        changeContainerHeight(studyRoomMode = studyRoomMode)
    }

    private fun changeContainerHeight(studyRoomMode: LPConstants.StudyRoomMode) {
        val layoutParams = layoutParams as ConstraintLayout.LayoutParams
        when {
            studyRoomMode.isGalleryLayout -> {
                layoutParams.bottomToTop = -1
                layoutParams.bottomToBottom = ConstraintSet.PARENT_ID
                setBackgroundColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_blackboard_color))
            }
            else -> {
                layoutParams.bottomToTop = R.id.activity_group_class_sync_container
                layoutParams.bottomToBottom = -1
                setBackgroundColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_room_bg_color))
            }
        }
    }

    private fun subscribe() {
        disposableOfFullSeat = router.getSubjectByKey<Boolean>(EventKey.NoteSelfActive)
            .subscribe {
                isFullSeat = it
                imgBtn?.visibility = if (!studyRoomMode.isGalleryLayout || isFullSeat) View.GONE else View.VISIBLE
            }
        compositeDisposable.add(router.liveRoom.studyRoomVM
            .observableOfTimeRank
            .observeOn(AndroidSchedulers.mainThread())
            .map { sortList(it) }
            .subscribe {
                router.getSubjectByKey<List<LPStudyUserStatus>>(EventKey.StudyRankList).onNext(it)
            })
    }

    /**
     * 排序如1，1，3，3，5，6，6，8
     */
    private fun sortList(list: List<LPStudyUserStatus>): List<LPStudyUserStatus> {
        if (list.isNullOrEmpty()) {
            return list
        }
        sort(list) { o1, o2 -> (o2.duration - o1.duration).toInt() }
        var temp = list[0]
        for ((index, item) in list.withIndex()) {
            item.rank = index + 1
            if (item.duration == temp.duration) {
                item.rank = temp.rank
            } else {
                temp = item
            }
        }
        return list
    }

    override fun initVideo() {
        isFullSeat = false
        studyRoomLayer = StudyRoomLayer(context)
        studyRoomLayer?.lisenter = this
        videoDragLayer = VideoDragLayer(context)
        studyRoomLayer?.setVideoDragLayer(videoDragLayer)
        post {
            videoDragLayer?.setSeatContainerWidth(measuredHeight)
        }
        addPlaceholderViews()
        addView(studyRoomLayer, ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT))
        addOuterLayer()
        subscribe()
    }

    /**
     * 外层容器，抢位置按钮和提示
     */
    private fun addOuterLayer() {
        val imgParams = LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        imgParams.gravity = Gravity.CENTER
        imgBtn = ImageView(context)
        imgBtn!!.run {
            setImageResource(R.drawable.sel_grab_seat)
            addView(this, imgParams)
            setOnClickListener {
                router.liveRoom.onlineUserVM.requestAddActiveUser(router.liveRoom.currentUser)
            }
        }
        val textParams = LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        text = TextView(context)
        textParams.gravity = Gravity.TOP or Gravity.CENTER_HORIZONTAL
        textParams.topMargin = 8.dp
        textParams.width = 300.dp
        val topPadding = 10.dp
        text?.run {
            setTextColor(Color.WHITE)
            setPadding(0, topPadding, 0, topPadding)
            gravity = Gravity.CENTER
            addView(this, textParams)
            visibility = View.GONE
            background = DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                .cornerRadius(4.dp)
                .build()
        }
    }

    /**
     * 按钮加4个空位占位图
     */
    private fun addPlaceholderViews() {
        placeholderContainer = StudyRoomPlaceholderLayer(context)
        val maxSpeaks = router.liveRoom.maxActiveUsers
        for (i in 0 until maxSpeaks) {
            val emptyView = View.inflate(context, R.layout.bjysc_layout_video_empty, null)
            emptyView.window_video_container_to_seat_placeholder.setImageResource(R.drawable.base_ic_video_default_head)
            emptyView.window_video_container_seat_name.text = context.getString(R.string.bjysc_study_room_video_empty)
            placeholderContainer?.addView(emptyView)
        }
        addView(placeholderContainer, 0, ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT))
    }

    override fun getVideoDragLayer() = videoDragLayer

    override fun switchStudyRoomMode(roomMode: LPConstants.StudyRoomMode) {
        studyRoomMode = roomMode
        val hidePlaceholder = !roomMode.isGalleryLayout
        imgBtn?.visibility = if (hidePlaceholder || isFullSeat) View.GONE else View.VISIBLE
        placeholderContainer?.visibility = if (hidePlaceholder) View.GONE else View.VISIBLE
        studyRoomLayer?.switchStudyRoomMode(roomMode)
        changeContainerHeight(roomMode)
        RxUtils.dispose(disposableOfTimer)
        when (roomMode) {
            LPConstants.StudyRoomMode.SelfStudy -> {
                text?.run {
                    visibility = View.VISIBLE
                    text = context.getString(R.string.bjysc_study_room_switch_self_study_tip)
                    startTimer()
                }
            }
            LPConstants.StudyRoomMode.Discuss -> {
                text?.run {
                    visibility = View.VISIBLE
                    text = context.getString(R.string.bjysc_study_room_switch_tutor_tip)
                    startTimer()
                }
            }
            else -> {
                text?.visibility = View.GONE
            }
        }
    }

    private fun startTimer() {
        disposableOfTimer = Observable.timer(15L, TimeUnit.SECONDS)
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe {
                text?.visibility = View.GONE
            }
    }

    override fun onDestroy() {
        super.onDestroy()
        release()
    }

    override fun release() {
        RxUtils.dispose(disposableOfFullSeat)
        RxUtils.dispose(disposableOfTimer)
        removeAllViews()
        videoDragLayer?.onDestroy()
        studyRoomLayer?.onDestroy()
    }

    override fun switchLayoutMode(isBoardLayout: Boolean) {
        // empty
    }
}