package com.baijiayun.groupclassui.dialog

import android.content.res.ColorStateList
import android.graphics.Bitmap
import android.os.Bundle
import android.view.Gravity
import android.view.View
import android.view.WindowManager
import com.baijiayun.bjyutils.drawable.DrawableBuilder
import com.baijiayun.bjyutils.kt.dp
import com.baijiayun.bjyutils.reactivex.RxUtils
import com.baijiayun.groupclassui.R
import com.baijiayun.groupclassui.base.BaseGroupDialogFragment
import com.baijiayun.livebase.utils.ThemeDataUtil
import com.baijiayun.liveuibase.zxing.ZxingUtils
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.bjy_group_qrcode_dialog.*

/**
 * Created by pengwenjian on 2020/11/17
 * Describe:
 */
class QRcodeDialog : BaseGroupDialogFragment() {
    private var qrCodeDispose: Disposable? = null
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        hideTitleBar()
        val url = arguments?.getString("url")
        if (url != null) {
            val width = view.resources.getDimensionPixelSize(R.dimen.bjy_base_ascamera_qrcode_size)
            qrCodeDispose = Observable.create<Bitmap> {
                ZxingUtils.generateQRCodeBitmap(url, width, width)?.run {
                    it.onNext(this)
                }
            }
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    ivQRcode.setImageBitmap(it)
                }
        }
        ivClose.setOnClickListener {
            dismiss()
        }
        ivClose.imageTintList = ColorStateList.valueOf(
            ThemeDataUtil.getColorFromThemeConfigByAttrId(
                context,
                R.attr.base_theme_bottom_menu_text_color
            )
        )
    }

    override fun onStart() {
        super.onStart()
        dialog?.setCanceledOnTouchOutside(false)
        dialog?.window?.setBackgroundDrawable(
            DrawableBuilder()
                .rectangle()
                .solidColor(
                    ThemeDataUtil.getColorFromThemeConfigByAttrId(
                        context!!,
                        R.attr.base_theme_window_bg_color
                    )
                )
                .cornerRadius(4.dp)
                .build()
        )
    }

    override fun init(savedInstanceState: Bundle?, arguments: Bundle?) {
        // do nothing
    }

    override fun getLayoutId() = R.layout.bjy_group_qrcode_dialog

    override fun setWindowParams(windowParams: WindowManager.LayoutParams) {
        super.setWindowParams(windowParams)
        windowParams.width =
            resources.getDimensionPixelSize(R.dimen.bjy_base_zxing_qrcode_container_width)
        windowParams.height =
            resources.getDimensionPixelSize(R.dimen.bjy_base_zxing_qrcode_container_height)
        windowParams.gravity = Gravity.CENTER
    }

    override fun onDestroyView() {
        super.onDestroyView()
        RxUtils.dispose(qrCodeDispose)
    }
}