package com.baijiayun.groupclassui.layer;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Shader;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.ColorDrawable;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.log.LPLogger;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.glide.Glide;
import com.baijiayun.glide.request.target.SimpleTarget;
import com.baijiayun.glide.request.transition.Transition;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.container.ShapePaint;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.window.blackboard.BlackImageWindow;
import com.baijiayun.groupclassui.window.blackboard.BlackboardListener;
import com.baijiayun.livecore.models.LPGroupItem;
import com.baijiayun.liveuibase.utils.ShadowUtil;

import java.util.List;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.context.LPConstants;

/**
 * Created by Shubo on 2018/9/11.
 */
public class BlackboardLayer extends BaseLayerOfProfessionalClass implements BlackboardListener {

    private BlackImageWindow blackImageWindow;
    private ImageView bgLayout;
    private ShapePaint shapePaint;
    private int whiteboardHeight = 0;
    private SimpleTarget<Bitmap> simpleTarget;
    private Runnable runnable;
    private Disposable disposableOfGroupUpdate;
    private RelativeLayout groupLabel;
    private boolean showGroupLabel;

    public BlackboardLayer(@NonNull Context context) {
        super(context);
    }

    public void initView(){
        removeAllViews();
        if(blackImageWindow != null){
            blackImageWindow.onDestroy();
        }
        blackImageWindow = new BlackImageWindow(getContext());
        blackImageWindow.initListener();
        bgLayout = new ImageView(getContext());
        bgLayout.setScaleType(ImageView.ScaleType.CENTER_CROP);
        addView(bgLayout, new FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.MATCH_PARENT));
        addView(blackImageWindow.getView(), new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
        initWhiteboardViewHeight();
        if(shapePaint != null){
            blackImageWindow.setShapePaint(shapePaint);
        }

        groupLabel = new RelativeLayout(getContext());
        addView(groupLabel);
        disposableOfGroupUpdate = router.getLiveRoom().getOnlineUserVM().getObservableOfOnGroupItem()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpGroupItems -> {
                    if (router.getLiveRoom().getCurrentUser().getGroup() != 0 && router.getLiveRoom().getOnlineUserVM().isMediaGroup()) {
                        for (LPGroupItem groupItem : lpGroupItems) {
                            if (groupItem.id == router.getLiveRoom().getCurrentUser().getGroup() && groupItem.groupItemModel != null) {
                                addText(getContext().getString(R.string.bjy_group_locate, groupItem.groupItemModel.name));
                                break;
                            }
                        }
                    } else {
                        showGroupToast(lpGroupItems);
                        groupLabel.removeAllViews();
                        groupLabel.setVisibility(View.GONE);
                        showGroupLabel = false;
                    }
                });
    }

    private void showGroupToast(List<LPGroupItem> groupItemList) {
        int count = 0;
        for (LPGroupItem groupItem : groupItemList) {
            count += groupItem.count;
        }
        // 学生被解散分组或被移出时提示
        if (router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student) {
            if (count == 0 || groupItemList.size() == 0) {
                if (showGroupLabel) {
                    router.getSubjectByKey(EventKey.ReminderMessage).onNext(getContext().getString(R.string.bjy_group_group_end));
                }
            } else {
                if (showGroupLabel) {
                    router.getSubjectByKey(EventKey.ReminderMessage).onNext(getContext().getString(R.string.bjy_group_group_remove));
                }
            }
        }
    }

    @SuppressLint("CheckResult")
    private void addText(String message) {
        if (!showGroupLabel) {
            router.getSubjectByKey(EventKey.ReminderMessage).onNext(getContext().getString(R.string.bjy_group_set_group));
        }
        showGroupLabel = true;
        groupLabel.removeAllViews();
        groupLabel.setVisibility(View.VISIBLE);
        groupLabel.setBackground(new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(getContext(), R.attr.base_theme_brand_container_color))
                .bottomRightRadius(getContext().getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .build());

        TextView textView = new TextView(getContext());
        textView.setText(message);
        textView.setTextSize(12);
        textView.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(getContext(), R.attr.base_theme_window_assistant_text_color));
        RelativeLayout.LayoutParams tp = new RelativeLayout.LayoutParams(
                ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.MATCH_PARENT);
        tp.addRule(RelativeLayout.CENTER_VERTICAL);
        tp.setMarginStart(UtilsKt.getDp(10));
        tp.setMarginEnd(UtilsKt.getDp(10));
        textView.setSingleLine(true);
        textView.setEllipsize(TextUtils.TruncateAt.END);
        textView.setGravity(Gravity.START | Gravity.CENTER_VERTICAL);
        textView.setTextAlignment(TEXT_ALIGNMENT_CENTER);
        textView.measure(0, 0);
        int measuredWidth = textView.getMeasuredWidth();
        FrameLayout.LayoutParams lp = new FrameLayout.LayoutParams(
                measuredWidth + UtilsKt.getDp(20),
                getContext().getResources().getDimensionPixelSize(R.dimen.bjysc_remind_label_height));
        groupLabel.setLayoutParams(lp);
        groupLabel.addView(textView, tp);
    }

    @Override
    public boolean onRoomStatusChange(boolean isRoomActive) {
        boolean isLayerInit = super.onRoomStatusChange(isRoomActive);
        if(isLayerInit && !isRoomActive){
            unSubscribe();
        }
        return true;
    }

    private void unSubscribe(){
        LPLogger.d("wtf", "blackboardLayer unSubscribe");
        removeCallbacks(runnable);
        if (blackImageWindow != null)
            blackImageWindow.onDestroy();
        blackImageWindow = null;
        Glide.with(getContext().getApplicationContext()).clear(simpleTarget);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        RxUtils.dispose(disposableOfGroupUpdate);
        unSubscribe();
    }

    private void initWhiteboardViewHeight(){
        if (router.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE) {
            whiteboardHeight = getMeasuredHeight() * router.getLiveRoom().getPartnerConfig().live1v1BlackboardPages;
        } else {
            whiteboardHeight = getMeasuredHeight() * router.getLiveRoom().getPartnerConfig().liveBlackboardPages;
        }
        blackImageWindow.initWhiteBoardHeight(whiteboardHeight);
        bgLayout.setLayoutParams(new FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, getMeasuredHeight()));
        runnable = this::initWhiteboardBackground;
        post(runnable);
    }

    private void initWhiteboardBackground() {
        if (router == null) {
            return;
        }
        String whiteboardBackgroundUrl = router.getLiveRoom().getWhiteboardBackgroundUrl();
        if (router.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE) {
            whiteboardHeight = getMeasuredHeight() * router.getLiveRoom().getPartnerConfig().live1v1BlackboardPages;
        } else {
            whiteboardHeight = getMeasuredHeight() * router.getLiveRoom().getPartnerConfig().liveBlackboardPages;
        }
        bgLayout.setLayoutParams(new FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, getMeasuredHeight()));
        blackImageWindow.initWhiteBoardHeight(whiteboardHeight);

        boolean isDefaultWhiteBoard = router.getLiveRoom().isDefaultWhiteBoard();
        if(!isDefaultWhiteBoard && !TextUtils.isEmpty(whiteboardBackgroundUrl)){
            simpleTarget = new SimpleTarget<Bitmap>() {
                @Override
                public void onResourceReady(@NonNull Bitmap resource, @Nullable Transition<? super Bitmap> transition) {
                    BitmapDrawable bitmapDrawable = new BitmapDrawable(getResources(), resource);
                    bitmapDrawable.setTileModeXY(Shader.TileMode.REPEAT, Shader.TileMode.REPEAT);
                    bgLayout.setImageDrawable(bitmapDrawable);
                }
            };
            Glide.with(getContext())
                    .asBitmap()
                    .load(whiteboardBackgroundUrl)
                    .into(simpleTarget);
        } else{
            ColorDrawable drawable = new ColorDrawable(ThemeDataUtil.getColorFromThemeConfigByAttrId(getContext(), R.attr.base_theme_blackboard_color));
            bgLayout.setImageDrawable(drawable);
        }
    }

    @Override
    public void scrollTo(float ratio) {
        blackImageWindow.scrollTo(ratio);
    }

    @Override
    public BlackImageWindow getBlackImageWindow() {
        return blackImageWindow;
    }

    @Override
    public void setShapePaint(ShapePaint shapePaint) {
        this.shapePaint = shapePaint;
    }
}
