package com.baijiayun.groupclassui.layer;

import static com.baijiayun.bjyutils.kt.UtilsKt.removeViewFromParent;

import android.content.Context;
import android.view.Gravity;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;

import androidx.annotation.NonNull;

import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.util.InteractiveUtils;
import com.baijiayun.groupclassui.widget.DragRelativeLayout;
import com.baijiayun.groupclassui.window.BaseTitledWindow;
import com.baijiayun.groupclassui.window.bottommenu.SpeakApplyStatus;
import com.baijiayun.groupclassui.window.ppt.PPTTabLayer;
import com.baijiayun.groupclassui.window.ppt.PPTWindow;
import com.baijiayun.groupclassui.window.toolbar.ToolbarWindow;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.listener.OnSpeakApplyCountDownListener;
import com.baijiayun.livecore.ppt.whiteboard.LaserShapeLayer;
import com.baijiayun.liveuibase.base.BaseWindow;
import com.baijiayun.liveuibase.toolbox.browser.BrowserWindow;
import com.baijiayun.liveuibase.toolbox.browser.Tuple;
import com.baijiayun.liveuibase.utils.BaseUtilsKt;
import com.baijiayun.liveuibase.widgets.common.ProgressCircleView;

import java.util.ArrayList;
import java.util.List;

import io.reactivex.android.schedulers.AndroidSchedulers;

/**
 * Created by Shubo on 2018/10/11.
 */
public class FullScreenLayer extends BaseLayerOfProfessionalClass {

    private BaseTitledWindow fullScreenWindow;
    private List<PPTWindow> pptWindows;
    private BaseWindow fullBrowserWindow;
    private ToolbarWindow toolbarWindow;
    private DragRelativeLayout.OnUpdateLayoutParamsListener listener;
    private FrameLayout handsUpContainer;//全屏下举手布局

    private boolean isNeedRemoveSwitchWindow = true;

    private SpeakApplyStatus speakApplyStatus = SpeakApplyStatus.None;
    private LaserShapeLayer laserShapeLayer;
    private BaseLayerOfProfessionalClass pptTabLayer;

    public FullScreenLayer(@NonNull Context context) {
        super(context);
    }

    public ToolbarWindow getToolBarWindow() {
        return toolbarWindow;
    }


    @Override
    public boolean onRoomStatusChange(boolean isRoomActive) {
        boolean isLayerInit = super.onRoomStatusChange(isRoomActive);
        if (isLayerInit) {
            if (isRoomActive) {
                subscribe();
            } else {
                removeAllViews();
            }
        }
        return true;
    }

    private void subscribe() {
        pptWindows = new ArrayList<>();
        if (toolbarWindow == null) {
            toolbarWindow = new ToolbarWindow(getContext(), true);
            addOnLayoutChangeListener(onLayoutChangeListener);
            toolbarWindow.setDragParam(getWidth(), getHeight());
        }
        compositeDisposable.add(router.getSubjectByKey(EventKey.SetFullScreenWindowOn)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(c -> {
                    if (c instanceof BrowserWindow) {
                        // 网页全屏
                        fullBrowserWindow = (BrowserWindow) c;
                        removeViewFromParent(fullBrowserWindow.getView());

                        LayoutParams layoutParams = new LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                                ViewGroup.LayoutParams.MATCH_PARENT);
                        FullScreenLayer.this.addWindow(fullBrowserWindow, layoutParams);

                        if (!BaseUtilsKt.isAdmin(router.getLiveRoom())) {
                            showHansUp();
                        }
                    } else if (c instanceof BaseTitledWindow) {
                        // ppt 全屏
                        if (fullScreenWindow != null && !(fullScreenWindow instanceof PPTWindow)) {
                            removeViewFromParent(fullScreenWindow.getView());
                        }
                        fullScreenWindow = (BaseTitledWindow) c;
                        removeViewFromParent(fullScreenWindow.getView());

                        LayoutParams layoutParams = new LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                                ViewGroup.LayoutParams.MATCH_PARENT);
                        FullScreenLayer.this.addWindow(fullScreenWindow, layoutParams);
                        if (c instanceof PPTWindow && BaseUtilsKt.isAdmin(router.getLiveRoom())) {
                            PPTTabLayer pptTabLayer = router.getValueByKey(EventKey.PPTTabLayer, PPTTabLayer.class);
                            if (pptTabLayer != null) {
                                removeViewFromParent(pptTabLayer);
                                int fullScreenWindowIndex = FullScreenLayer.this.indexOfChild(fullScreenWindow.getView()) + 1;
                                if (fullScreenWindowIndex < getChildCount()) {
                                    FullScreenLayer.this.addView(pptTabLayer, fullScreenWindowIndex);
                                } else {
                                    FullScreenLayer.this.addView(pptTabLayer);
                                }
                            }
                        }
                        if (!BaseUtilsKt.isAdmin(router.getLiveRoom())) {
                            showHansUp();
                        }
                        if (c instanceof PPTWindow) {
                            pptWindows.remove(c);
                            pptWindows.add((PPTWindow) fullScreenWindow);
                            if (laserShapeLayer != null) {
                                laserShapeLayer.bringToFront();
                            }

                            if (InteractiveUtils.hasDrawingAuth(router.getLiveRoom(), router.getLiveRoom().getCurrentUser())
                                    || InteractiveUtils.hasPPTAuth(router.getLiveRoom(), router.getLiveRoom().getCurrentUser())
                                    || InteractiveUtils.hasScreenShareAuth(router.getLiveRoom(), router.getLiveRoom().getCurrentUser())) {
                                initListener();
                                if (toolbarWindow.getView().getParent() == null) {
                                    FullScreenLayer.this.addView(toolbarWindow.getView());
                                } else {
                                    toolbarWindow.getView().bringToFront();
                                }
                            }
                        }
                    } else {
                        if (fullScreenWindow != null) {
                            if (c instanceof Boolean) {
                                isNeedRemoveSwitchWindow = (Boolean) c;
                            }
                            FullScreenLayer.this.removeWindow(fullScreenWindow);
                            if (fullScreenWindow instanceof PPTWindow) {
                                ((PPTWindow) fullScreenWindow).resetInFull(false);
                            }
                            fullScreenWindow = null;
                        }
                    }
                    if (laserShapeLayer != null) {
                        laserShapeLayer.bringToFront();
                    }
                }));


        compositeDisposable.add(router.getSubjectByKey(EventKey.PPTWindowBringToFront)
                .ofType(String.class)
                .mergeWith(router.getSubjectByKey(EventKey.PPTWindowBringToFrontLocal).ofType(String.class))
                .subscribe(s -> {
                    if (toolbarWindow != null && toolbarWindow.getView().getParent() instanceof FullScreenLayer) {
                        toolbarWindow.getView().bringToFront();
                    }
                }));


        compositeDisposable.add(router.getSubjectByKey(EventKey.SetFullScreenWindowOff)
                .subscribe(c -> {
                    if (c == fullBrowserWindow) {
                        fullBrowserWindow = null;
                    }
                    if (c == fullScreenWindow) {
                        fullScreenWindow = null;
                    }
                    // if true remove all layer views !!
                    if (isNeedRemoveSwitchWindow) {
                        if (fullBrowserWindow != null) {
                            LayoutParams layoutParams = new LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                                    ViewGroup.LayoutParams.MATCH_PARENT);
                            FullScreenLayer.this.addWindow(fullBrowserWindow, layoutParams);
                        }
                    }
                    isNeedRemoveSwitchWindow = true;
                    pptWindows.clear();
                    FullScreenLayer.this.removeAllViews();
                }));

        compositeDisposable.add(router.getSubjectByKey(EventKey.RemoveFullScreenWindow)
                .subscribe(o -> {
                    FullScreenLayer.this.removeAllViews();
                }));

        compositeDisposable.add(router.getLiveRoom().getMediaVM().getObservableOfPlayerViewUpdate()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpPlayerViewUpdateModel -> {
                    if (lpPlayerViewUpdateModel.action == LPConstants.LPPlayerAction.REMOVE) {
                        hideHansUp();
                    }
                }));

        compositeDisposable.add(router.getSubjectByKey(EventKey.LaserLayerFullScreen)
                .ofType(LaserShapeLayer.class)
                .subscribe(laserShapeLayer -> {
                    removeViewFromParent(laserShapeLayer);
                    int fullScreenWindowIndex = 0;
                    if (fullScreenWindow != null && fullScreenWindow.getParentViewGroup() == FullScreenLayer.this) {
                        fullScreenWindowIndex = FullScreenLayer.this.indexOfChild(fullScreenWindow.getView()) + 1;
                    }
                    if (fullScreenWindow != null && fullScreenWindow instanceof PPTWindow && ((PPTWindow) fullScreenWindow).getPPTView() != null) {
                        laserShapeLayer.setDocInfo(((PPTWindow) fullScreenWindow).getRealDocId(), ((PPTWindow) fullScreenWindow).getPPTView().getCurrentPageIndex());
                    }
                    FullScreenLayer.this.addView(laserShapeLayer, fullScreenWindowIndex, new LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
                    this.laserShapeLayer = laserShapeLayer;
                }));

        compositeDisposable.add(router.getSubjectByKey(EventKey.ClosePPTWindow)
                .ofType(Tuple.class)
                .subscribe(s -> {
                    if (fullScreenWindow == null) {
                        return;
                    }
                    for (PPTWindow pptWindow : pptWindows) {
                        if (s.docId.equals(pptWindow.getId())) {
                            removePPTWindow(pptWindow);
                            break;
                        }
                    }
                }));

        compositeDisposable.add(router.getLiveRoom().getDocListVM().getObservableOfDocViewUpdate()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpDocViewUpdateModel -> {
                    if ("remove".equals(lpDocViewUpdateModel.action)) {
                        for (PPTWindow pptWindow : pptWindows) {
                            if (lpDocViewUpdateModel.docId.equals(pptWindow.getId())) {
                                removePPTWindow(pptWindow);
                                break;
                            }
                        }
                    }
                }));

        compositeDisposable.add(router.getSubjectByKey(EventKey.OpenBrowserWindow)
                .ofType(Boolean.class)
                .subscribe(aBoolean -> {
                    if (fullBrowserWindow == null) {
                        return;
                    }
                    if (!aBoolean) {
                        FullScreenLayer.this.removeWindow(fullBrowserWindow);
                        fullBrowserWindow = null;
                    }
                }));

        compositeDisposable.add(router.getLiveRoom().getToolBoxVM().getObservableOfWebPageInfo()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(webPageInfoModel -> {
                    if (fullBrowserWindow == null) {
                        return;
                    }
                    if (!webPageInfoModel.isOpenStatus()) {
                        if (!BaseUtilsKt.isAdmin(router.getLiveRoom())) {
                            FullScreenLayer.this.removeWindow(fullBrowserWindow);
                            fullBrowserWindow.onDestroy();
                            fullBrowserWindow = null;
                        }
                        hideHansUp();
                    }
                }));
    }

    private void initListener() {
        toolbarWindow.setOnUpdateLayoutParamsListener(new DragRelativeLayout.OnUpdateLayoutParamsListener() {
            @Override
            public void onUpdateLayoutParams(int left, int top, int right, int bottom) {
                // do nothing
            }

            @Override
            public void onActionDown() {
                if (listener != null) {
                    listener.onActionDown();
                }
            }
        });
    }

    private void removePPTWindow(PPTWindow pptWindow) {
        FullScreenLayer.this.removeWindow(pptWindow);
        if (fullScreenWindow instanceof PPTWindow) {
            if (pptWindow == fullScreenWindow) {
                FullScreenLayer.this.removeAllViews();
                fullScreenWindow = null;
            }
        }
    }

    private void showHansUp() {
        hideHansUp();
        ImageView iv = new ImageView(getContext());
        iv.setImageResource(R.drawable.ic_interactive_hand_up);
        iv.setSelected(false);
        iv.setEnabled(speakApplyStatus == SpeakApplyStatus.None);
        LayoutParams params = new LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        iv.setLayoutParams(params);

        ProgressCircleView circleView = new ProgressCircleView(getContext());
        circleView.setVisibility(INVISIBLE);

        handsUpContainer = new FrameLayout(getContext());
        handsUpContainer.addView(iv, params);
        handsUpContainer.addView(circleView, params);

        LayoutParams layoutParams = new LayoutParams(UtilsKt.getDp(40), UtilsKt.getDp(40));
        layoutParams.gravity = Gravity.BOTTOM | Gravity.END;
        layoutParams.rightMargin = UtilsKt.getDp(15);
        layoutParams.bottomMargin = UtilsKt.getDp(15);
        addView(handsUpContainer, layoutParams);

        iv.setOnClickListener((view) -> {
            if (!router.getLiveRoom().isClassStarted()) {
                router.getSubjectByKey(EventKey.ReminderMessage).onNext(getContext().getString(R.string.bjysc_closed_class_not_start_error));
                return;
            }
            if (speakApplyStatus == SpeakApplyStatus.None) {
                if (router.getLiveRoom().getForbidRaiseHandStatus()) {
                    router.getSubjectByKey(EventKey.ReminderMessage).onNext(getContext().getString(R.string.bjysc_forbid_send_message));
                    return;
                }

                router.getLiveRoom().getSpeakQueueVM().requestSpeakApply(new OnSpeakApplyCountDownListener() {
                    @Override
                    public void onTimeOut() {
                        speakApplyStatus = SpeakApplyStatus.None;
                        router.getLiveRoom().getSpeakQueueVM().cancelSpeakApply();
                        router.getSubjectByKey(EventKey.ReminderMessage).onNext(getContext().getString(R.string.bjysc_speak_apply_disagree));
                    }

                    @Override
                    public void onTimeCountDown(int counter, int timeOut) {
                        circleView.setVisibility(VISIBLE);
                        circleView.setRatio((timeOut - counter) / (float) timeOut);
                    }
                });
                speakApplyStatus = SpeakApplyStatus.Applying;
            } else if (speakApplyStatus == SpeakApplyStatus.Applying) {
                // 取消发言请求
                speakApplyStatus = SpeakApplyStatus.None;
                router.getLiveRoom().getSpeakQueueVM().cancelSpeakApply();
                iv.setSelected(false);
                iv.setEnabled(true);
                circleView.setVisibility(INVISIBLE);
            } else if (speakApplyStatus == SpeakApplyStatus.Speaking) {
                // 取消发言
            }
        });

        compositeDisposable.add(router.getLiveRoom().getSpeakQueueVM().getObservableOfSpeakResponse()
                .filter(iMediaControlModel -> iMediaControlModel.getUser().getUserId().equals(router
                        .getLiveRoom().getCurrentUser().getUserId()) && !BaseUtilsKt.isAdmin(router.getLiveRoom()))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaControlModel -> {
                    if (iMediaControlModel.isApplyAgreed()) {
                        // 进入发言模式
                        iv.setEnabled(false);
                        circleView.setVisibility(INVISIBLE);
                        speakApplyStatus = SpeakApplyStatus.Speaking;
                    } else {
                        //拒绝发言请求
                        speakApplyStatus = SpeakApplyStatus.None;
                        if (!iMediaControlModel.getSenderUserId().equals(router.getLiveRoom().getCurrentUser().getUserId())) {
                            // 不是自己结束发言的
                            iv.setEnabled(true);
                            iv.setSelected(false);
                            circleView.setVisibility(INVISIBLE);
                        }
                    }
                }));
    }

    private void hideHansUp() {
        if (handsUpContainer != null) {
            removeView(handsUpContainer);
        }
        handsUpContainer = null;
    }

    public void setOnUpdateLayoutParamsListener(DragRelativeLayout.OnUpdateLayoutParamsListener listener) {
        this.listener = listener;
    }

    private OnLayoutChangeListener onLayoutChangeListener = (v, left, top, right, bottom, oldLeft, oldTop, oldRight, oldBottom) -> {
        if (toolbarWindow != null) {
            toolbarWindow.setDragParam(right - left, bottom - top);
        }
    };

    @Override
    public void onDestroy() {
        super.onDestroy();
        fullScreenWindow = null;
    }
}
