package com.baijiayun.groupclassui.studyreport;

import android.app.Dialog;
import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.baijiayun.groupclassui.R;
import com.baijiayun.livecore.models.LPStudyReportModel;
import com.baijiayun.livecore.utils.LPRecyclerItemClickSupport;

import java.util.List;

/**
 * @author wubanglin
 */
public class StudyReportListDialog extends Dialog {
    private Context context;
    private View view;
    private ReportItemClickListener listener;

    private ImageView closeDialogIv;
    private RecyclerView reportStudentListRv;
    private ReportStudnetListAdapter adapter;


    private int mWidth = RelativeLayout.LayoutParams.FILL_PARENT;
    private int mHeight = RelativeLayout.LayoutParams.WRAP_CONTENT;

    public StudyReportListDialog(@NonNull Context context,ReportItemClickListener listener) {
        super(context);
        this.context = context;
        adapter = new ReportStudnetListAdapter();
        this.listener = listener;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setBackgroundDrawableResource(android.R.color.transparent);
        getWindow().setGravity(Gravity.BOTTOM);
        getWindow().setWindowAnimations(R.style.BJYGroupShareAnimation);

        LayoutInflater inflater = ((StudyReportActivity) context).getLayoutInflater();
        view = inflater.inflate(R.layout.dialog_share_user_list_group_class, null);
        setContentView(view);
        getWindow().setLayout(mWidth, mHeight);

        initView();
    }

    private void initView() {
        closeDialogIv = findViewById(R.id.iv_gc_user_list_cancle);
        reportStudentListRv = findViewById(R.id.rl_gc_user_list);

        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(context);
        reportStudentListRv.setLayoutManager(linearLayoutManager);
        reportStudentListRv.setAdapter(adapter);

        LPRecyclerItemClickSupport.addTo(reportStudentListRv).setOnItemClickListener((recyclerView, position, v) -> {
            if (adapter == null || adapter.getItemCount() <= 0 || listener == null) {
                dismiss();
                return;
            }
            LPStudyReportModel studyReportModel = this.adapter.list.get(position);
            if (!studyReportModel.userNumber.equals(adapter.currentStudyReport.userNumber)) {
                this.adapter.currentStudyReport = studyReportModel;
                listener.onLoad(studyReportModel);
            }
            dismiss();
        });

        closeDialogIv.setOnClickListener(v->{
            dismiss();
        });
    }

    public void setLayout(int width, int height) {
        mWidth = width;
        mHeight = height;
        getWindow().setLayout(width, height);
    }

    public StudyReportListDialog setCurrReportItem(LPStudyReportModel studyReportModel) {
        this.adapter.currentStudyReport = studyReportModel;
        return this;
    }

    public StudyReportListDialog setAllReport(List<LPStudyReportModel> list) {
        if (list.isEmpty()) {
            this.adapter.list = null;
        } else {
            this.adapter.list = list;
        }
        return this;
    }

    public void show(){
        super.show();
        adapter.notifyDataSetChanged();
    }

    public class ReportStudnetListAdapter extends RecyclerView.Adapter<ReportStudentViewHolder> {
        public LPStudyReportModel currentStudyReport;
        public List<LPStudyReportModel> list;

        @NonNull
        @Override
        public ReportStudentViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_gc_share_user_list, parent, false);
            return new ReportStudentViewHolder(view);
        }

        @Override
        public void onBindViewHolder(@NonNull ReportStudentViewHolder viewHolder, int position) {
            LPStudyReportModel model = list.get(position);
            if (model.userNumber.equals(currentStudyReport.userNumber)) {
                viewHolder.reportStudentRl.setBackgroundColor(Color.parseColor("#FFF6FBFF"));
                viewHolder.reportStateIv.setVisibility(View.VISIBLE);
            } else {
                viewHolder.reportStudentRl.setBackgroundColor(Color.WHITE);
                viewHolder.reportStateIv.setVisibility(View.INVISIBLE);
            }
            viewHolder.reportStudentNameTv.setText(model.userName);
        }


        @Override
        public int getItemCount() {
            if (list.isEmpty()) {
                return 0;
            } else {
                return list.size();
            }
        }
    }

    public static class ReportStudentViewHolder extends RecyclerView.ViewHolder {
        private RelativeLayout reportStudentRl;
        private ImageView reportStateIv;
        private TextView reportStudentNameTv;

        public ReportStudentViewHolder(@NonNull View itemView) {
            super(itemView);
            reportStudentRl = itemView.findViewById(R.id.rl_gc_item_user_list);
            reportStateIv = itemView.findViewById(R.id.iv_gc_item_user_list_state);
            reportStudentNameTv = itemView.findViewById(R.id.tv_gc_item_user_list_title);
        }
    }

    public interface ReportItemClickListener {
        void onLoad(LPStudyReportModel studyReportModel);
    }
}
