package com.baijiayun.groupclassui.user;

import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;

import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.baijiahulian.common.cropperv2.uikit.CircleColorView;
import com.baijiayun.glide.Glide;
import com.baijiayun.glide.load.resource.bitmap.CircleCrop;
import com.baijiayun.glide.request.RequestOptions;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventCode;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.groupclassui.global.RouterListener;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livecore.models.LPGroupItem;

/**
 * Created by yongjiaming on 2018/10/11
 * 举手用户列表
 */
public class HandsUpUserListFragment extends Fragment implements IUserInteractionListener {

    private RecyclerView recyclerView;
    private OnlineUserPresenter userPresenter;
    private HandsUpUserListRecyclerViewAdapter handsUpAdapter;
    //举手列表暂时去除禁止举手和允许举手
    private LinearLayout showForbidHandsUp;
    private TextView btnForbidHandsUp;

    private boolean isForbidHandUp = false;
    private IRouter iRouter;

    public HandsUpUserListFragment() {
        // Required empty public constructor
    }

    public static HandsUpUserListFragment newInstance() {
        return new HandsUpUserListFragment();
    }

    private void setPresenter(OnlineUserPresenter onlineUserPresenter) {
        this.userPresenter = onlineUserPresenter;
        userPresenter.setRouter(iRouter);
        userPresenter.subscribe();
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof RouterListener) {
            iRouter = ((RouterListener) context).getRouter();
        }
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.bjysc_fragment_handup, container, false);

        Context context = view.getContext();
        recyclerView = view.findViewById(R.id.bjysc_hands_up_list);
        recyclerView.setLayoutManager(new LinearLayoutManager(context));
        setPresenter(new OnlineUserPresenter(this));
        handsUpAdapter = new HandsUpUserListRecyclerViewAdapter();
        recyclerView.setAdapter(handsUpAdapter);

        showForbidHandsUp = view.findViewById(R.id.bjysc_show_forbid_handsUp);
        btnForbidHandsUp = view.findViewById(R.id.bjysc_btn_handUp);

        btnForbidHandsUp.setOnClickListener(v -> {
            userPresenter.requestForbidRaiseHand(!isForbidHandUp);
        });
        if (userPresenter != null) {
            userPresenter.getForbidRaiseHandUp();
        }
        return view;
    }

    void agreeSpeakApply(IUserModel requestUser) {
        userPresenter.agreeSpeakApply(requestUser);
    }

    void disagreeSpeakApply(String userId) {
        userPresenter.disagreeSpeakApply(userId);
    }

    class HandsUpUserListRecyclerViewAdapter extends RecyclerView.Adapter<HandsUpUserListRecyclerViewAdapter.ViewHolder> {

        @NonNull
        @Override
        public ViewHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int i) {
            View view = LayoutInflater.from(viewGroup.getContext())
                    .inflate(R.layout.item_hands_up_user, viewGroup, false);
            return new ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(@NonNull ViewHolder viewHolder, int i) {
            IUserModel userModel = userPresenter.getHandsUpUser(i);
            if (userModel == null) {
                return;
            }
            viewHolder.userNameTv.setText(getEncodePhoneNumber(userModel.getName()));
            if (!TextUtils.isEmpty(userModel.getAvatar()) && getContext() != null) {
                Glide.with(getContext()).load(userModel.getAvatar())
                        .apply(new RequestOptions().transform(new CircleCrop()))
                        .into(viewHolder.userAvatarIv);
            }
            viewHolder.handsUpIv.setOnClickListener(v -> {
                agreeSpeakApply(userModel);
            });

            viewHolder.disagreeHandsUpIv.setOnClickListener(v -> disagreeSpeakApply(userModel.getUserId()));
            if (userModel.getGroup() == 0) {
                viewHolder.groupContainer.setVisibility(View.GONE);
            } else {
                viewHolder.groupContainer.setVisibility(View.VISIBLE);
                final LPGroupItem groupItem = userPresenter.getGroupItem(userModel.getGroup());
                if (groupItem != null && groupItem.groupItemModel != null) {
                    viewHolder.groupNameTv.setText(groupItem.groupItemModel.name);
                    viewHolder.groupColorView.setColor(Color.parseColor(groupItem.groupItemModel.color));
                }
            }
        }

        @Override
        public int getItemCount() {
            return userPresenter.getHandsUpUserCount();
        }


        class ViewHolder extends RecyclerView.ViewHolder {

            ImageView userAvatarIv;
            TextView userNameTv;
            ImageView handsUpIv;
            ImageView disagreeHandsUpIv;
            View groupContainer;
            TextView groupNameTv;
            CircleColorView groupColorView;

            ViewHolder(@NonNull View itemView) {
                super(itemView);
                userAvatarIv = itemView.findViewById(R.id.user_avatar_iv);
                userNameTv = itemView.findViewById(R.id.tv_user_name);
                handsUpIv = itemView.findViewById(R.id.agree_hand_up_iv);
                disagreeHandsUpIv = itemView.findViewById(R.id.disagree_hand_up_iv);
                groupContainer = itemView.findViewById(R.id.ll_group_container);
                groupNameTv = itemView.findViewById(R.id.tv_group_name);
                groupColorView = itemView.findViewById(R.id.view_group_color);
            }
        }
    }

    public void notifyDataChange() {
        handsUpAdapter.notifyDataSetChanged();
    }

    @Override
    public void notifyOnlineUserDataSetChange() {
        // do nothing
    }

    @Override
    public void notifyOnlineUserCountChange() {
        // do nothing
    }

    @Override
    public void notifyHandsUpUserDataSetChange() {
        notifyDataChange();
    }

    @Override
    public void notifyActiveUserDataChange() {
        // do nothing
    }

    @Override
    public void notifySpeakApplyResult() {
        // do nothing
    }

    @Override
    public void notifyForbidStatus() {
        // do nothing
    }

    @Override
    public void notifyUserDataChange() {
        // do nothing
    }

    @Override
    public void notifyForbidHandUp(boolean isForbid) {
        if (getActivity() == null || getContext() == null) {
            return;
        }
        Drawable drawable;
        if (isForbid) {
            drawable = ContextCompat.getDrawable(getContext(), R.drawable.ic_not_forbid_hand_up);
        } else {
            drawable = ContextCompat.getDrawable(getContext(), R.drawable.ic_forbid_hand_up);
        }
        getActivity().runOnUiThread(() -> {
            isForbidHandUp = isForbid;
            if (isForbid) {
                recyclerView.setVisibility(View.GONE);
                showForbidHandsUp.setVisibility(View.GONE);
                btnForbidHandsUp.setText(getString(R.string.bjysc_un_forbid_hands_up));
                btnForbidHandsUp.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(getContext(), R.attr.base_theme_live_product_color));

            } else {
                showForbidHandsUp.setVisibility(View.GONE);
                recyclerView.setVisibility(View.VISIBLE);
                btnForbidHandsUp.setText(getString(R.string.bjysc_forbid_hands_up));
                btnForbidHandsUp.setTextColor(getResources().getColor(R.color.bjysc_red));
            }
            if (drawable != null) {
                drawable.setBounds(0, 0, drawable.getMinimumWidth(), drawable.getMinimumHeight());
                btnForbidHandsUp.setCompoundDrawables(drawable,
                        null, null, null);
            }
        });

    }

    @Override
    public void notifyCurrentSeatFull() {
        if (iRouter != null) {
            iRouter.getPublishSubjectByKey(EventCode.NotifyCurrentSeatFull).onNext(true);
        }
    }

    @Override
    public void showBlockedUserFreeAllUI() {
        // do nothing
    }

    @Override
    public void notifyStudyRoomSwitch() {
        // do nothing
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        if (userPresenter != null) {
            userPresenter.destroy();
        }
    }
}
