package com.baijiayun.groupclassui.user;

import android.app.Activity;
import android.content.Context;
import android.content.res.ColorStateList;
import android.graphics.drawable.Drawable;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.viewpager.widget.ViewPager;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.drawable.StateListDrawableBuilder;
import com.baijiayun.bjyutils.enums.ResStateType;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.bjyutils.screen.DisplayUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventCode;
import com.baijiayun.groupclassui.global.GroupClassActivity;
import com.baijiayun.groupclassui.util.ReplaceWindowGenerateUtil;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.google.android.material.tabs.TabLayout;

import java.util.Objects;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;

/**
 * Created by yongjiaming on 2018/10/11
 */
public class OnlineUserWindow extends InteractiveBaseWindow implements IUserInteractionListener{

    private TextView handsUpTabTv, onlineUserTabTv;
    private Context context;
    private TabLayout.Tab onlineUserTab,handsUpTab;
    private View handsupTip;
    private ViewPager viewPager;
    private boolean hasCheck;//消息是否已读
    private boolean startHandsUpList;
    private OnlineUserPresenter onlineUserPresenter;
    private AllOnlineUserListFragment onlineUserListFragment;
    private ReplaceUserWithStatus userWhenFullActiveAdd;
    private Disposable disposableOfSeatFull;
    private Disposable disposableOfSeatFullUserAdd;

    public OnlineUserWindow(Context context) {
        super(context);
    }

    @Override
    protected View onCreateView(Context context) {
        final View view = View.inflate(context, R.layout.bjy_group_dialog_online_user, null);
        view.setBackground(ThemeDataUtil.getCommonWindowBg(context));
        this.context = context;
        //之前这里是用presenter回调的，现在改用router
        RxUtils.dispose(disposableOfSeatFull);
        RxUtils.dispose(disposableOfSeatFullUserAdd);
        disposableOfSeatFull = iRouter.getPublishSubjectByKey(EventCode.NotifyCurrentSeatFull)
                .ofType(Boolean.class)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> notifyCurrentSeatFull());

        disposableOfSeatFullUserAdd = iRouter.getPublishSubjectByKey(EventCode.UserWhenFullActiveAdd)
                .ofType(ReplaceUserWithStatus.class)
                .subscribe(userWhenFullActiveAdd -> {
                    this.userWhenFullActiveAdd = userWhenFullActiveAdd;
                });
        return view;
    }

    private void initView(View view) {
        setAllowTouch(false);
        HandsUpUserListFragment handsUpUserListFragment = HandsUpUserListFragment.newInstance();
        onlineUserListFragment = AllOnlineUserListFragment.newInstance();

        viewPager = view.findViewById(R.id.online_user_viewpager);
        TabLayout tabLayout = view.findViewById(R.id.user_tab);

        onlineUserPresenter = new OnlineUserPresenter(this);
        onlineUserPresenter.setRouter(iRouter);

        if (!(context instanceof FragmentActivity)) {
            return;
        }
        FragmentActivity activity = (FragmentActivity) context;
        viewPager.setAdapter(new FragmentPagerAdapter(activity.getSupportFragmentManager()) {
            @Override
            public Fragment getItem(int i) {
                return i == 0 ? onlineUserListFragment : handsUpUserListFragment;
            }

            @Override
            public int getCount() {
                return 2;
            }
        });
        viewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageScrolled(int i, float v, int i1) {
                // do nothing
            }

            @Override
            public void onPageSelected(int i) {
                if (i == 1) {
                    hasCheck = true;
                }
                if (onlineUserPresenter.getHandsUpUserCount() > 0 && !hasCheck) {
                    handsupTip.setVisibility(View.VISIBLE);
                } else {
                    handsupTip.setVisibility(View.GONE);
                }
            }

            @Override
            public void onPageScrollStateChanged(int i) {
                // do nothing
            }
        });
        tabLayout.setupWithViewPager(viewPager);
        onlineUserTab = tabLayout.getTabAt(0);
        handsUpTab = tabLayout.getTabAt(1);
        final Drawable drawable = new DrawableBuilder()
                .rectangle()
                .cornerRadii(UtilsKt.getDp(2), UtilsKt.getDp(2), 0, 0)
                .strokeWidth((int) UtilsKt.getDp(0.5f))
                .strokeColor(ContextCompat.getColor(context,R.color.bjysc_bg_stroke))
                .build();
        final Drawable selectedDrawable = new DrawableBuilder()
                .rectangle()
                .cornerRadii(UtilsKt.getDp(2), UtilsKt.getDp(2), 0, 0)
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                .build();
        final ColorStateList tabTextColor =
                DisplayUtils.getColorStateList(ThemeDataUtil.getColorFromThemeConfigByAttrId(
                context, R.attr.base_theme_dialog_positive_text_color),
                ThemeDataUtil.getColorFromThemeConfigByAttrId(
                context, R.attr.base_theme_window_main_text_color),
                ResStateType.selected);

        if (handsUpTab != null) {
            handsUpTab.setCustomView(R.layout.user_dialog_tab_left_item);
            handsUpTabTv = Objects.requireNonNull(handsUpTab.getCustomView()).findViewById(R.id.left_tab_tv);
            handsupTip = Objects.requireNonNull(handsUpTab.getCustomView()).findViewById(R.id.view_tip);
            handsUpTabTv.setText(context.getString(R.string.hands_up_user, onlineUserPresenter.getHandsUpUserCount()));
            handsUpTabTv.setTextColor(tabTextColor);
            handsUpTabTv.setBackground(new StateListDrawableBuilder()
                    .normal(drawable)
                    .selected(selectedDrawable)
                    .build());
        }
        if (onlineUserTab != null) {
            onlineUserTab.setCustomView(R.layout.user_dialog_tab_right_item);
            onlineUserTabTv = Objects.requireNonNull(onlineUserTab.getCustomView()).findViewById(R.id.right_tab_tv);
            onlineUserTabTv.setText(context.getString(R.string.all_online_user, onlineUserPresenter.getAllUserCount()));
            onlineUserTabTv.setTextColor(tabTextColor);
            onlineUserTabTv.setBackground(new StateListDrawableBuilder()
                    .normal(drawable)
                    .selected(selectedDrawable)
                    .build());
        }
        ImageView closeIv = view.findViewById(R.id.online_user_close_iv);
        closeIv.setOnClickListener(v -> {
            onlineUserPresenter.closeWindow();
        });
        onlineUserPresenter.subscribe();
        if (startHandsUpList && handsUpTab != null) {
            handsUpTab.select();
        }
    }

    public void startHandsUpList() {
        if (handsUpTab != null) {
            handsUpTab.select();
        } else {
            this.startHandsUpList = true;
        }
    }

    public void destroy() {
        if (onlineUserPresenter != null) {
            onlineUserPresenter.destroy();
        }
        if (viewPager != null) {
            viewPager.clearOnPageChangeListeners();
        }
        RxUtils.dispose(disposableOfSeatFull);
        RxUtils.dispose(disposableOfSeatFullUserAdd);
        removeAllFragment();
    }

    @Override
    public void notifyOnlineUserDataSetChange() {
        // do nothing
    }

    @Override
    public void notifyOnlineUserCountChange() {
        onlineUserTabTv.setText(context.getString(R.string.all_online_user, onlineUserPresenter.getAllUserCount()));
    }

    @Override
    public void notifyHandsUpUserDataSetChange() {
        //在举手页面已读
        hasCheck = handsUpTab.isSelected();
        if (onlineUserPresenter.getHandsUpUserCount() > 0 && !hasCheck) {
            handsupTip.setVisibility(View.VISIBLE);
        } else {
            handsupTip.setVisibility(View.GONE);
        }
        handsUpTabTv.setText(context.getString(R.string.hands_up_user, onlineUserPresenter.getHandsUpUserCount()));
    }

    @Override
    public void notifyActiveUserDataChange() {
        //  do nothing
    }

    private void removeAllFragment() {
        if (context == null) return;
        for (Fragment fragment : ((AppCompatActivity)context).getSupportFragmentManager().getFragments()) {
            if (fragment != null)
                ((AppCompatActivity)context).getSupportFragmentManager().beginTransaction().remove(fragment).commitAllowingStateLoss();
        }
    }

    @Override
    public void notifySpeakApplyResult() {
        // do nothing
    }

    @Override
    public void notifyForbidStatus() {
        // do nothing
    }

    @Override
    public void notifyUserDataChange() {
        // do nothing
    }

    @Override
    public void notifyForbidHandUp(boolean isForbid) {
        // do nothing
    }

    @Override
    public void notifyCurrentSeatFull() {
        showFullDialog();
    }

    @Override
    public void showBlockedUserFreeAllUI() {
        // do nothing
    }

    @Override
    public void notifyStudyRoomSwitch() {
        // do nothing
    }

    private void showFullDialog() {
        if (context instanceof Activity) {
            Activity activity = (Activity) context;
            if (activity.isDestroyed() || activity.isFinishing()) {
                return;
            }
            ReplaceWindowGenerateUtil.generateWindow((GroupClassActivity)activity, iRouter, userWhenFullActiveAdd);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (onlineUserPresenter != null) {
            onlineUserPresenter.destroy();
        }
    }

    @Override
    public boolean onRoomStatusChange(boolean isActive) {
        boolean isWindowInit = super.onRoomStatusChange(isActive);
        if(isWindowInit){
            if (!isActive) {
                destroy();
            } else {
                initView(view);
            }
        }
        return true;
    }
}
