package com.baijiayun.groupclassui.window;

import android.content.Context;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;

import com.baijiayun.bjyutils.kt.UtilsKt;

/**
 * 固定缩放宽高比的window
 * <p>
 * Created by Shubo on 2019/3/26.
 */
public abstract class BaseFixedRatioTitledWindow extends BaseTitledWindow {
    // 计算 y : x 的比例值
    private float y2x_ratio;

    protected BaseFixedRatioTitledWindow(Context context) {
        super(context);
    }


    public void setWindowLayoutParams(ViewGroup.LayoutParams layoutParams) {
        minSize = UtilsKt.getDp(layoutParams.width > layoutParams.height ? DEFAULT_MIN_SIZE : 150);
        otherSizeRatio = layoutParams.width > layoutParams.height
                ? (float) layoutParams.width / (float) layoutParams.height
                : - ((float) layoutParams.height / (float) layoutParams.width);
        y2x_ratio = otherSizeRatio < 0 ? - otherSizeRatio : (1 / otherSizeRatio);
    }

    @Override
    protected void handleScaleTouchEvent(View v, MotionEvent event) {
        int x = (int) event.getX();
        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                downX = x;
                ViewGroup.LayoutParams layoutParams = dragScaleRelativeLayout.getLayoutParams();
                downWidth = layoutParams.width;
                downHeight = layoutParams.height;
                if (v.getParent() != null && v.getParent() instanceof ViewGroup) {
                    ViewGroup viewGroup = (ViewGroup) v.getParent();
                    measuredParentHeight = viewGroup.getMeasuredHeight();
                    measuredParentWidth = viewGroup.getMeasuredWidth();
                }
                break;
            case MotionEvent.ACTION_MOVE:
                int offX = x - downX;
                int offY = Math.round((x - downX) * y2x_ratio);

                // 缩放不超出边界
                if (v.getLeft() + downWidth + offX >= measuredParentWidth ||
                        v.getTop() + downHeight + offY >= measuredParentHeight) {
                    offX = Math.min(measuredParentWidth - v.getLeft() - downWidth,
                            Math.round((measuredParentHeight - v.getTop() - downHeight) / y2x_ratio));
                    offY = Math.round(offX * y2x_ratio);
                }

                // 缩放不小于最小值
                int height;
                int width;
                if (otherSizeRatio > 0) {
                    height = Math.max(downHeight + offY, minSize);
                    width = Math.max(downWidth + offX, ((int) (minSize * otherSizeRatio)));
                } else {
                    height = Math.max(downHeight + offY, ((int) (minSize * - otherSizeRatio)));
                    width = Math.max(downWidth + offX, minSize);
                }
                scaleTo(width, height);
                if (onWindowRepositionListener != null)
                    onWindowRepositionListener.OnWindowScale(width, height);
                break;
            case MotionEvent.ACTION_UP:
                scaling = false;
                break;
            default:
                break;
        }
    }
}
