package com.baijiayun.groupclassui.window.bottommenu;

import android.content.Context;
import android.text.TextUtils;

import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.groupclassui.user.UserStatus;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.listener.OnSpeakApplyCountDownListener;
import com.baijiayun.livecore.models.roomresponse.LPResCloudRecordModel;
import com.baijiayun.liveuibase.utils.BaseUtilsKt;

import java.util.concurrent.TimeUnit;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;
import io.reactivex.disposables.Disposable;

/**
 * Created by Shubo on 2018/9/21.
 */
public class BottomMenuPresenter implements BottomMenuContract.Presenter {

    private IRouter iRouter;
    private BottomMenuContract.View view;
    private Context context;
    private Disposable disposableOfChat, disposableOfFloatChat, disposableOfUserList, disposableOfCloudRecord, disposableOfMuteAll,
            disposableOfForbidHandsUp, disposableOfMsgReceived, disposableOfRoomLayoutSwitch, disposableOfUserRemove,
            disposableOfSpeakResponse, disposableOfMicOn, disposableOfCameraOn, disposableOfCloudRecordAllowed, disposableOfUserAdd,
            disposableOfCloudRecordProcess, disposableOfSpeakApplyDeny, disposableOfSpeakApply, disposableOfSpeakResponseNoFilter, disposableOfStatusTutor, disposableOfBonusPointsAdd;
    private CompositeDisposable compositeDisposable = new CompositeDisposable();
    // 当前布局类型，默认为板书布局
    private LPConstants.RoomLayoutMode currLayoutCategory = LPConstants.RoomLayoutMode.BOARD;
    private LPResCloudRecordModel cloudRecordStatus = new LPResCloudRecordModel(LPConstants.CloudRecordStatus.Stopped.getStatus());
    private SpeakApplyStatus speakApplyStatus = SpeakApplyStatus.None;
    private boolean isFirstCallRecordStatus = true;
    // 当前举手人数
    private int hansUpCount;
    private boolean isMediaCoursewarePlaying;

    BottomMenuPresenter(BottomMenuContract.View view, Context context) {
        this.view = view;
        this.context = context;
    }

    @Override
    public void subscribe() {
        disposableOfChat = iRouter.getSubjectByKey(EventKey.DisplayChat)
                .ofType(Boolean.class)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> view.showChatWindowShowingStatus(aBoolean));

        disposableOfFloatChat = iRouter.getSubjectByKey(EventKey.DisplayFloatChat)
                .ofType(Boolean.class)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> view.showFloatChatWindowShowingStatus(aBoolean));

        disposableOfUserList = iRouter.getSubjectByKey(EventKey.DisplayUserList)
                .ofType(UserStatus.class)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(userStatus -> {
                    view.showUserListWindowShowingStatus(userStatus != UserStatus.Close);
                    if (userStatus != UserStatus.Close) {
                        view.hideHandsUpCount();
                    }
                });

        if (BaseUtilsKt.isAdmin(iRouter.getLiveRoom())) {
            disposableOfCloudRecord = iRouter.getLiveRoom().getObservableOfCloudRecordStatus()
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(recordStatus -> {
                        cloudRecordStatus = recordStatus;

                        if (isFirstCallRecordStatus && cloudRecordStatus.status == LPConstants.CloudRecordStatus.Recording.getStatus()) {
                            isFirstCallRecordStatus = false;
                            view.showCloudRecordHint();
                        }

                        view.showRecordingStatus(cloudRecordStatus);
                    });

            cloudRecordStatus = iRouter.getLiveRoom().getCloudRecordStatus();
            if (isFirstCallRecordStatus && cloudRecordStatus.status == LPConstants.CloudRecordStatus.Recording.getStatus()) {
                isFirstCallRecordStatus = false;
                if (iRouter.getLiveRoom().canCloudRecord()) {
                    view.showCloudRecordHint();
                }
            }
            view.showRecordingStatus(cloudRecordStatus);
            view.showCloudRecordButton(showCloudRecord());
        }

        disposableOfForbidHandsUp = iRouter.getLiveRoom().getObservableOfForbidRaiseHand()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> {
                    iRouter.getSubjectByKey(EventKey.ForbidHandsUp).onNext(aBoolean);
                    if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student
                            && speakApplyStatus == SpeakApplyStatus.Speaking) {
                        // 学生在发言中，则只显示不可用状态
                        view.showSpeakApplyDisable();
                    } else {
                        view.showForbidHandsUpStatus(aBoolean);
                    }
                });
        view.showForbidHandsUpStatus(iRouter.getLiveRoom().getForbidRaiseHandStatus());

        disposableOfRoomLayoutSwitch = iRouter.getLiveRoom().getObservableOfRoomLayoutSwitch()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(integer -> {
                    currLayoutCategory = integer;
                    view.showGalleryModel(integer == LPConstants.RoomLayoutMode.GALLERY);
                });

        currLayoutCategory = iRouter.getLiveRoom().getCurrentRoomLayout();

        view.showGalleryModel(currLayoutCategory == LPConstants.RoomLayoutMode.GALLERY);

        disposableOfSpeakResponse = iRouter.getLiveRoom().getSpeakQueueVM().getObservableOfSpeakResponse()
                .filter(iMediaControlModel -> iMediaControlModel.getUser().getUserId().equals(iRouter
                        .getLiveRoom().getCurrentUser().getUserId()) && !BaseUtilsKt.isAdmin(iRouter.getLiveRoom()))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaControlModel -> {
                    if (iMediaControlModel.isApplyAgreed()) {
                        // 进入发言模式
                        iRouter.getSubjectByKey(EventKey.ReminderMessage).
                                onNext(context.getString(R.string.bjysc_speak_apply_agree));
                        view.showSpeakApplyNormal();
                    } else {
                        speakApplyStatus = SpeakApplyStatus.None;
                        if (!iMediaControlModel.getSenderUserId().equals(
                                iRouter.getLiveRoom().getCurrentUser().getUserId())) {
                            // 不是自己结束发言的
                            iRouter.getSubjectByKey(EventKey.ReminderMessage).
                                    onNext(context.getString(R.string.bjysc_speak_apply_disagree));
                            view.showSpeakApplyNormal();
                        }
                    }
                });

        disposableOfSpeakApply = iRouter.getLiveRoom().getSpeakQueueVM().getObservableOfSpeakApply()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModel -> {
                    //举手监听
                    hansUpCount++;
                    Object userStatus = iRouter.getSubjectByKey(EventKey.DisplayUserList).getValue();
                    if (null == userStatus || userStatus == UserStatus.Close) {
                        view.showHandsUpRemind(hansUpCount, iMediaModel.getUser().getName());
                    }
                });

        disposableOfSpeakResponseNoFilter = iRouter.getLiveRoom().getSpeakQueueVM().getObservableOfSpeakResponse()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaControlModel -> {
                    if (hansUpCount >= 1) {
                        hansUpCount--;
                    }
                    Object userStatus = iRouter.getSubjectByKey(EventKey.DisplayUserList).getValue();
                    if (null != userStatus && userStatus != UserStatus.Close) {
                        return;
                    }
                    if (hansUpCount > 0) {
                        view.showHandsUpRemind(hansUpCount, iMediaControlModel.getUser().getName());
                    } else {
                        view.hideHandsUpCount();
                    }
                });

        disposableOfSpeakApplyDeny = iRouter.getLiveRoom().getSpeakQueueVM().getObservableOfSpeakApplyDeny()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModel -> {
                    // 服务器拒绝发言
                    speakApplyStatus = SpeakApplyStatus.None;
                    iRouter.getLiveRoom().getSpeakQueueVM().cancelSpeakApply();
                    iRouter.getSubjectByKey(EventKey.ReminderMessage).
                            onNext(context.getString(R.string.bjysc_closed_by_server));
                    view.showSpeakApplyNormal();
                });

        disposableOfMicOn = iRouter.getLiveRoom().getRecorder().getObservableOfMicOn()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(isMicOn -> {
                    view.showMicStatus(isMicOn);
                    if (iRouter.getLiveRoom().getCurrentUser().getType() != LPConstants.LPUserType.Student) {
                        return;
                    }
                    if (isMicOn) {
                        speakApplyStatus = SpeakApplyStatus.Speaking;
                    } else {
                        speakApplyStatus = SpeakApplyStatus.None;
                    }
                    if (iRouter.getLiveRoom().getForbidRaiseHandStatus() || isMicOn) {
                        view.showSpeakApplyDisable();
                    } else {
                        view.showSpeakApplyNormal();
                    }
                });

        disposableOfCameraOn = iRouter.getLiveRoom().getRecorder().getObservableOfCameraOn()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(isCameraOn -> view.showCameraStatus(isCameraOn));

        disposableOfUserRemove = iRouter.getLiveRoom().getOnlineUserVM().getPublishSubjectOfActiveUserRemove()
                .observeOn(AndroidSchedulers.mainThread())
                .filter(lpResRoomUserInModel -> iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student)
                .subscribe(lpResRoomUserInModel -> {
                    //自习模式下台，允许举手
                    if (iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout()) {
                        if (TextUtils.equals(lpResRoomUserInModel.user.userId, iRouter.getLiveRoom().getCurrentUser().getUserId())
                                && !iRouter.getLiveRoom().getForbidRaiseHandStatus()) {
                            view.showSpeakApplyNormal();
                        }
                    }
                });

        disposableOfUserAdd = iRouter.getLiveRoom().getOnlineUserVM().getPublishSubjectOfActiveUserAdd()
                .observeOn(AndroidSchedulers.mainThread())
                .filter(lpResRoomUserInModel -> iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student)
                .subscribe(lpResRoomUserInModel -> {
                    //自习模式上台，禁止举手
                    if (iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout()) {
                        if (TextUtils.equals(lpResRoomUserInModel.user.userId, iRouter.getLiveRoom().getCurrentUser().getUserId())
                                || iRouter.getLiveRoom().getForbidRaiseHandStatus()) {
                            view.showSpeakApplyDisable();
                        }
                    }
                });

        disposableOfMsgReceived = iRouter.getSubjectByKey(EventKey.ChatMessageReceived)
                .ofType(Boolean.class)
                .subscribe(aBoolean -> view.showChatWindowRedPoint(aBoolean));

        disposableOfMuteAll = iRouter.getLiveRoom().getObservableOfQuickMuteAllStudentMic()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> view.showMuteAllStateChange(aBoolean));

        disposableOfStatusTutor = iRouter.getSubjectByKey(EventKey.StudyStatusTutor)
                .filter(o -> BaseUtilsKt.isAdmin(iRouter.getLiveRoom()))
                .ofType(String.class)
                .subscribe(s -> view.showUserListStatus(TextUtils.isEmpty(s)));
        disposableOfBonusPointsAdd = iRouter.getLiveRoom().getToolBoxVM()
                .getObservableOfBonusPointsAdd()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(integer -> view.showBonusPointsAdd(integer));

        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.openMediaPlayerWindow)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(window -> isMediaCoursewarePlaying = true));

        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.closeMediaPlayerWindow)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(o -> isMediaCoursewarePlaying = false));
    }

    @Override
    public void unSubscribe() {
        // do nothing
    }

    @Override
    public void navigateToGalleryMode() {
        iRouter.getLiveRoom().requestRoomLayoutSwitch(currLayoutCategory == LPConstants.RoomLayoutMode.GALLERY ? LPConstants.RoomLayoutMode.BOARD : LPConstants.RoomLayoutMode.GALLERY);
    }

    @Override
    public void changeRecordStatus() {
        if (cloudRecordStatus.status == LPConstants.CloudRecordStatus.Stopped.getStatus()) {
            disposableOfCloudRecordAllowed = iRouter.getLiveRoom().requestPlaybackProcessStatus()
                    .throttleFirst(1, TimeUnit.SECONDS)
                    .subscribeOn(AndroidSchedulers.mainThread())
                    .subscribe(lpPlaybackProcessStatusModel -> {
                        //0:未开启云端录制 1:开启云端录制（未转码） 2:已触发云端录制转码
                        if (lpPlaybackProcessStatusModel.status == 0) {
                            iRouter.getLiveRoom().requestCloudRecord(LPConstants.CloudRecordStatus.Recording);
                        } else if (lpPlaybackProcessStatusModel.status == 1) {
                            if (lpPlaybackProcessStatusModel.isLongTermClass()) {
                                view.showCloudRecordRestartDialog();
                            } else {
                                iRouter.getLiveRoom().requestCloudRecord(LPConstants.CloudRecordStatus.Recording);
                            }
                        } else if (lpPlaybackProcessStatusModel.status == 2) {
                            if (lpPlaybackProcessStatusModel.isLongTermClass()) {
                                iRouter.getLiveRoom().requestCloudRecord(LPConstants.CloudRecordStatus.Recording);
                            } else {
                                view.showShortTermClassAlreadyHasPlayback();
                            }
                        }
                    });
        } else {
            stopCloudRecord();
        }
    }

    @Override
    public void changeForbidHandsUpStatus() {
        Object object = iRouter.getSubjectByKey(EventKey.ForbidHandsUp).getValue();
        iRouter.getLiveRoom().requestForbidRaiseHand(null == object || object.equals(false));
    }

    @Override
    public void navigateToUserList() {
        Object object = iRouter.getSubjectByKey(EventKey.DisplayUserList).getValue();
        if (null == object || object == UserStatus.Close) {
            iRouter.getSubjectByKey(EventKey.DisplayUserList).onNext(UserStatus.UserList);
        } else {
            iRouter.getSubjectByKey(EventKey.DisplayUserList).onNext(UserStatus.Close);
        }
    }

    @Override
    public void navigateToChat() {
        Object object = iRouter.getSubjectByKey(EventKey.DisplayChat).getValue();
        iRouter.getSubjectByKey(EventKey.DisplayChat).onNext(null == object || object.equals(false));
    }

    @Override
    public void navigateToHomework() {
        iRouter.getSubjectByKey(EventKey.CoursewareManageEnable).onNext(true);
    }

    @Override
    public void navigateToEyeCare(boolean enableEyeCare) {
        iRouter.getSubjectByKey(EventKey.DisplayEyeCare).onNext(enableEyeCare);
    }

    @Override
    public void speakApply() {
        if (!iRouter.getLiveRoom().isClassStarted()) {
            iRouter.getSubjectByKey(EventKey.ReminderMessage).
                    onNext(context.getString(R.string.bjysc_closed_class_not_start_error));
            view.showSpeakApplyNormal();
            return;
        }

        if (speakApplyStatus == SpeakApplyStatus.None) {
            iRouter.getLiveRoom().getSpeakQueueVM().requestSpeakApply(new OnSpeakApplyCountDownListener() {
                @Override
                public void onTimeOut() {
                    speakApplyStatus = SpeakApplyStatus.None;
                    iRouter.getLiveRoom().getSpeakQueueVM().cancelSpeakApply();
                    if (iRouter.getLiveRoom().getForbidRaiseHandStatus()) {
                        iRouter.getSubjectByKey(EventKey.ReminderMessage).
                                onNext(context.getString(R.string.bjysc_forbid_send_message));
                        view.showSpeakApplyDisable();
                    } else {
                        iRouter.getSubjectByKey(EventKey.ReminderMessage).
                                onNext(context.getString(R.string.bjysc_speak_apply_disagree));
                        view.showSpeakApplyNormal();
                    }
                }

                @Override
                public void onTimeCountDown(int counter, int timeOut) {
                    if (iRouter.getLiveRoom().getForbidRaiseHandStatus()) {
                        speakApplyStatus = SpeakApplyStatus.None;
                        iRouter.getLiveRoom().getSpeakQueueVM().cancelSpeakApply();
                        view.showSpeakApplyDisable();
                    } else {
                        view.showSpeakApplyCountDown(timeOut - counter, timeOut);
                    }
                }
            });
            speakApplyStatus = SpeakApplyStatus.Applying;
        } else if (speakApplyStatus == SpeakApplyStatus.Applying) {
            // 取消发言请求
            speakApplyStatus = SpeakApplyStatus.None;
            iRouter.getLiveRoom().getSpeakQueueVM().cancelSpeakApply();
            view.showSpeakApplyNormal();
        }
    }

    @Override
    public void stopCloudRecord() {
        cloudRecordStatus.status = LPConstants.CloudRecordStatus.Stopped.getStatus();
        iRouter.getLiveRoom().requestCloudRecord(LPConstants.CloudRecordStatus.Stopped);
    }

    @Override
    public void pauseCloudRecord() {
        cloudRecordStatus.status = LPConstants.CloudRecordStatus.Paused.getStatus();
        iRouter.getLiveRoom().requestCloudRecord(LPConstants.CloudRecordStatus.Paused);
    }

    @Override
    public void startNewCloudRecord() {
        disposableOfCloudRecordProcess = iRouter.getLiveRoom().requestCloudRecordStartProcessing()
                .subscribeOn(AndroidSchedulers.mainThread())
                .subscribe(c -> {
                    RxUtils.dispose(disposableOfCloudRecordProcess);
                    iRouter.getLiveRoom().requestCloudRecord(LPConstants.CloudRecordStatus.Recording);
                });
    }

    @Override
    public void continueCloudRecord() {
        iRouter.getLiveRoom().requestCloudRecord(LPConstants.CloudRecordStatus.Recording);
    }

    @Override
    public LPResCloudRecordModel getCloudRecordStatus() {
        return cloudRecordStatus;
    }

    @Override
    public void quickMuteAllStudentMic(boolean mute) {
        iRouter.getLiveRoom().quickMuteAllStudentMic(mute);
    }

    @Override
    public void exit() {
        iRouter.getSubjectByKey(EventKey.CloseDialog).onNext(true);
    }

    @Override
    public boolean showCloudRecord() {
        return iRouter.getLiveRoom().canCloudRecord()
                && !iRouter.getLiveRoom().getStudyRoomVM().isStudyRoom()
                && !iRouter.getLiveRoom().getPartnerConfig().disableRecordStatus();
    }

    @Override
    public boolean isMediaCoursewarePlaying() {
        return isMediaCoursewarePlaying;
    }

    @Override
    public void navigateToBonusPoints() {
        iRouter.getSubjectByKey(EventKey.ShowBonusPointWindow).onNext(true);
    }

    @Override
    public void setRouter(IRouter router) {
        iRouter = router;
    }

    @Override
    public void destroy() {
        RxUtils.dispose(disposableOfChat);
        RxUtils.dispose(disposableOfFloatChat);
        RxUtils.dispose(disposableOfUserList);
        RxUtils.dispose(disposableOfCloudRecord);
        RxUtils.dispose(disposableOfForbidHandsUp);
        RxUtils.dispose(disposableOfMsgReceived);
        RxUtils.dispose(disposableOfRoomLayoutSwitch);
        RxUtils.dispose(disposableOfSpeakResponse);
        RxUtils.dispose(disposableOfMicOn);
        RxUtils.dispose(disposableOfCameraOn);
        RxUtils.dispose(disposableOfCloudRecordAllowed);
        RxUtils.dispose(disposableOfSpeakApplyDeny);
        RxUtils.dispose(disposableOfSpeakApply);
        RxUtils.dispose(disposableOfSpeakResponseNoFilter);
        RxUtils.dispose(disposableOfMuteAll);
        RxUtils.dispose(disposableOfUserRemove);
        RxUtils.dispose(disposableOfUserAdd);
        RxUtils.dispose(disposableOfStatusTutor);
        RxUtils.dispose(disposableOfBonusPointsAdd);
        iRouter = null;
        view = null;
        context = null;
    }
}
