package com.baijiayun.groupclassui.window.coursewaremanage;

import android.annotation.SuppressLint;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LifecycleObserver;
import androidx.lifecycle.OnLifecycleEvent;

import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.groupclassui.global.RouterListener;
import com.baijiayun.groupclassui.window.IRoomStatusListener;
import com.baijiayun.groupclassui.window.ppt.PPTWindow;
import com.baijiayun.livebase.widgets.dialog.CommonDialog;
import com.baijiayun.livecore.context.LiveRoom;
import com.baijiayun.livecore.models.LPDocModel;
import com.baijiayun.livecore.models.LPDocViewElementModel;
import com.baijiayun.livecore.models.LPDocViewUpdateModel;
import com.baijiayun.livecore.models.LPUploadDocModel;
import com.baijiayun.livecore.models.LPUploadHomeworkUserModel;
import com.baijiayun.livecore.models.courseware.LPMediaCoursewareModel;
import com.baijiayun.livecore.models.file.cloudfile.LPCloudFileModel;
import com.baijiayun.livecore.models.roomresponse.LPAllowUploadHomeworkModel;
import com.baijiayun.liveuibase.utils.ToastUtil;
import com.baijiayun.liveuibase.widgets.courseware.BaseCourseWareContract;
import com.baijiayun.liveuibase.widgets.courseware.BaseCourseWareFragment;
import com.baijiayun.liveuibase.widgets.courseware.CourseManagePresenter;
import com.baijiayun.liveuibase.widgets.courseware.ICourseWareViewListener;
import com.baijiayun.liveuibase.widgets.courseware.models.UploadDocModel;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.atomic.AtomicBoolean;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;

/**
 * 课件管理窗口
 */
public class CourseManageWindow extends BaseCourseWareFragment implements LifecycleObserver, IRoomStatusListener {
    private Lifecycle lifecycle;
    private IRouter iRouter;
    private final CompositeDisposable compositeDisposable = new CompositeDisposable();

    @Override
    protected void initChildData() {
        isWindowInit = new AtomicBoolean(false);
        lateCallRoomStatusChange = new AtomicBoolean(false);
        if (context instanceof RouterListener) {
            iRouter = ((RouterListener) context).getRouter();
            lifecycle = ((RouterListener) context).getLifecycle();
            lifecycle.addObserver(this);
            ((RouterListener) context).addRoomStatusListener(this);
        }

        contentView.addOnAttachStateChangeListener(new View.OnAttachStateChangeListener() {
            @Override
            public void onViewAttachedToWindow(View v) {
                isWindowInit.set(true);
                if (lateCallRoomStatusChange.get()) {
                    onRoomStatusChange(true);
                    lateCallRoomStatusChange.set(false);
                }
            }

            @Override
            public void onViewDetachedFromWindow(View v) {
                // do nothing
            }
        });
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = super.onCreateView(inflater, container, savedInstanceState);
        presenter.subscribe(iRouter.getLiveRoom());
        return view;
    }

    @Override
    protected ICourseWareViewListener initCourseWareViewListener() {
        return new ICourseWareViewListener() {
            @Override
            public void onClose() {
                iRouter.getSubjectByKey(EventKey.CoursewareManageEnable).onNext(false);
            }

            @Override
            public void onPlayImg(LPUploadDocModel uploadDocModel) {
                iRouter.getSubjectByKey(EventKey.GraphBitmapEnable).onNext(uploadDocModel);
            }

            @Override
            public void onReminderMsg(String msg) {
                iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(msg);
            }

            @Override
            public void onDeleteDoc(String fid, boolean isReUpload) {
                if (!isReUpload) {
                    new CommonDialog(context, CommonDialog.ChoiceMode.Double_Red)
                            .setMainDisplayText(context.getString(R.string.base_course_manage_delete_doc_reminder))
                            .setPositive(context.getString(R.string.base_course_manage_delete_confirm), ((dialog, which) ->
                                    iRouter.getLiveRoom().getDocListVM().deleteDocument(fid)))
                            .setNegative(context.getString(R.string.base_cancel), null).show();
                } else {
                    iRouter.getLiveRoom().getDocListVM().deleteDocument(fid);
                }
            }

            @Override
            public void onDeleteMediaCourseware(String fid) {
                new CommonDialog(context, CommonDialog.ChoiceMode.Double_Red)
                        .setMainDisplayText(context.getString(R.string.base_course_manage_delete_doc_reminder))
                        .setPositive(context.getString(R.string.base_course_manage_delete_confirm), ((dialog, which) -> {
                            compositeDisposable.add(iRouter.getLiveRoom().getDocListVM().requestDeleteMediaCourseware(fid)
                                    .observeOn(AndroidSchedulers.mainThread())
                                    .subscribe(aBoolean -> {
                                    }));
                        }))
                        .setNegative(context.getString(R.string.base_cancel), null).show();
            }

            @Override
            public void onDeleteCloudFile(LPCloudFileModel cloudFileModel) {
                new CommonDialog(context, CommonDialog.ChoiceMode.Double_Red)
                        .setMainDisplayText(context.getString(R.string.base_course_manage_delete_doc_reminder))
                        .setPositive(context.getString(R.string.base_course_manage_delete_confirm), ((dialog, which) ->
                                iRouter.getLiveRoom().getCloudFileVM().deleteCloudFile(cloudFileModel.getFileId())))
                        .setNegative(context.getString(R.string.base_cancel), null).show();
            }

            @Override
            public void onDeleteHomework(String homeworkId) {
                new CommonDialog(context, CommonDialog.ChoiceMode.Double_Red)
                        .setMainDisplayText(context.getString(R.string.base_course_manage_delete_homework_reminder))
                        .setPositive(context.getString(R.string.base_course_manage_delete_confirm),
                                ((dialog, which) -> presenter.deleteHomework(homeworkId,
                                        LPUploadHomeworkUserModel.copyData(iRouter.getLiveRoom().getCurrentUser()))))
                        .setNegative(context.getString(R.string.base_cancel), null).show();
            }

            @Override
            public void onRemindHomeworkSupport() {
                new CommonDialog(context, CommonDialog.ChoiceMode.Double_Blue)
                        .setMainDisplayText(context.getString(R.string.base_course_manage_homework_support_remind_text))
                        .setPositive(context.getString(R.string.base_has_know), null)
                        .setNegative(context.getString(R.string.base_do_not_remind), (dialog, which) ->
                                homeworkRemindStatus = HomeworkRemindStatus.DO_NOT_REMIND).show();
            }

            @Override
            public void onCancelTransfer(UploadDocModel uploadDocModel) {
                new CommonDialog(context, CommonDialog.ChoiceMode.Double_Red)
                        .setMainDisplayText(context.getString(R.string.base_course_manage_cancel_transfer_reminder))
                        .setPositive(context.getString(R.string.base_course_manage_delete_confirm), ((dialog, which) -> presenter.deleteTranslating(uploadDocModel)))
                        .setNegative(context.getString(R.string.base_cancel), null).show();
            }

            @Override
            public void onUpdateAllowUploadHomework(boolean isAllow) {
                iRouter.getLiveRoom().getDocListVM().requestUpdateAllowUploadHomework(
                        new LPAllowUploadHomeworkModel(isAllow)
                );
            }

            @Override
            public void requestLoadNextPage(String keyword) {
                iRouter.getLiveRoom().getDocListVM().requestNextPageHomework(keyword);
            }
        };
    }

    int nowDocPosIndex = 0;

    @Override
    protected BaseCourseWareContract.BaseCourseWarePresenter initPresenter() {
        return new CourseManagePresenter(this) {
            @Override
            public boolean playDoc(LPDocModel docModel) {
                ConcurrentLinkedQueue<PPTWindow> pptWindows = iRouter.getQueueByKey(EventKey.PPTWindows, PPTWindow.class);
                if (pptWindows.size() >= 15) {
                    showToast(getString(R.string.bjysc_doc_count_out_of_limit));
                    return false;
                }
                LPDocViewUpdateModel lpDocViewUpdateModel = new LPDocViewUpdateModel();
                lpDocViewUpdateModel.docId = docModel.docId;
                lpDocViewUpdateModel.action = "add";
                LPDocViewElementModel lpDocViewElementModel = new LPDocViewElementModel();
                lpDocViewElementModel.docId = docModel.docId;
                lpDocViewElementModel.max = 0;
                lpDocViewElementModel.full = 0;
                lpDocViewElementModel.width = "0.33";
                lpDocViewElementModel.height = String.valueOf((docModel.height / docModel.width * 0.66f));
                List<LPDocViewElementModel> allDocList = new ArrayList<>(
                        iRouter.getListByKey(EventKey.AllDocViewList, LPDocViewElementModel.class));
                if (allDocList.size() == 0) {
                    // 无ppt则从头开始偏移
                    nowDocPosIndex = 0;
                }
                lpDocViewElementModel.x = String.valueOf(0.0125 * nowDocPosIndex++);
                lpDocViewElementModel.y = "0";

                boolean hasAdd = false;
                for (LPDocViewElementModel elementModel : allDocList) {
                    if (docModel.docId.equals(elementModel.docId)) {
                        hasAdd = true;
                        break;
                    }
                }
                if (!hasAdd) {
                    allDocList.add(lpDocViewElementModel);
                    lpDocViewUpdateModel.all = allDocList;
                    iRouter.setObjectByKey(EventKey.AllDocViewList, allDocList);
                    iRouter.getLiveRoom().getDocListVM().requestDocViewUpdate(lpDocViewUpdateModel);
                }
                sendSwitchSignal(lpDocViewElementModel);
                return true;
            }

            @Override
            public void playMediaCourseware(LPMediaCoursewareModel toPlayMedia) {
                if(iRouter.getLiveRoom().getOnlineUserVM().getActiveUserList().contains(iRouter.getLiveRoom().getCurrentUser())) {
                    iRouter.getSubjectByKey(EventKey.openMediaPlayerWindow).onNext(toPlayMedia);
                } else {
                    new ToastUtil(context).setText(context.getString(R.string.bjy_group_play_media_require_on_stage)).show();
                }
            }

            @Override
            public void playBds(LPDocModel bdsModel) {
                new CommonDialog(context, CommonDialog.ChoiceMode.Double_Blue)
                        .setMainDisplayText(context.getString(R.string.bjy_group_open_bds_tip))
                        .setPositive(context.getString(R.string.bjysc_confirm), (dialog, which) -> {
                            iRouter.getSubjectByKey(EventKey.BdsRepaint).onNext(bdsModel.docId);
                        })
                        .setNegative(context.getString(R.string.bjysc_cancel), null)
                        .show();
            }

            @Override
            public void previewDoc(LPDocModel docModel) {
                ConcurrentLinkedQueue<PPTWindow> pptWindows = iRouter.getQueueByKey(EventKey.PPTWindows, PPTWindow.class);
                if (pptWindows.size() >= 15) {
                    showToast(getString(R.string.bjysc_doc_count_out_of_limit));
                } else {
                    iRouter.getSubjectByKey(EventKey.DocPreview).onNext(docModel);
                }
            }
        };
    }

    private void sendSwitchSignal(LPDocViewElementModel elementModel) {
        ConcurrentLinkedQueue<PPTWindow> pptWindows = iRouter.getQueueByKey(EventKey.PPTWindows, PPTWindow.class);
        if (pptWindows == null || pptWindows.isEmpty()) {
            return;
        }
        final PPTWindow maxPPTWindow = iRouter.getValueByKey(EventKey.MaxScreenPPTWindow, PPTWindow.class);
        final PPTWindow fullPPTWindow = iRouter.getValueByKey(EventKey.FullScreenPPTWindow, PPTWindow.class);
        if (maxPPTWindow == null && fullPPTWindow == null) {
            return;
        }
        boolean isFull = fullPPTWindow != null;
        final PPTWindow topWindow = isFull ? fullPPTWindow : maxPPTWindow;
        //打开的课件已经是顶层window不用处理
        if (TextUtils.equals(topWindow.getId(), elementModel.docId)) {
            return;
        }
        List<LPDocViewElementModel> allDocList = iRouter.getListByKey(EventKey.AllDocViewList, LPDocViewElementModel.class);
        LPDocViewUpdateModel lpDocViewUpdateModel = new LPDocViewUpdateModel();
        lpDocViewUpdateModel.docId = topWindow.getRealDocId();
        lpDocViewUpdateModel.action = "normal";
        lpDocViewUpdateModel.nextAction = isFull ? "full" : "max";
        if (allDocList != null && !allDocList.isEmpty() && !topWindow.isPreviewPPT()) {
            for (LPDocViewElementModel model : allDocList) {
                if (model.docId.equals(lpDocViewUpdateModel.docId)) {
                    model.max = 0;
                    model.full = 0;
                    lpDocViewUpdateModel.all = allDocList;
                    iRouter.getLiveRoom().getDocListVM().requestDocViewUpdate(lpDocViewUpdateModel);
                    break;
                }
            }
        }
        elementModel.max = !isFull ? 1 : 0;
        elementModel.full = isFull ? 1 : 0;
        LPDocViewUpdateModel docViewUpdateModel = new LPDocViewUpdateModel();
        docViewUpdateModel.docId = elementModel.docId;
        docViewUpdateModel.action = lpDocViewUpdateModel.nextAction;
        if (allDocList == null) {
            allDocList = new ArrayList<>();
        }
        boolean hasAdd = false;
        for (LPDocViewElementModel viewElementModel : allDocList) {
            if (elementModel.docId.equals(viewElementModel.docId)) {
                viewElementModel.max = elementModel.max;
                viewElementModel.full = elementModel.full;
                hasAdd = true;
                break;
            }
        }
        if (!hasAdd) {
            allDocList.add(elementModel);
        }
        docViewUpdateModel.all = allDocList;
        iRouter.getLiveRoom().getDocListVM().requestDocViewUpdate(docViewUpdateModel);
        //和PPTWindow里面switchTab逻辑一致
        if (isFull && !topWindow.isPreviewPPT()) {
            lpDocViewUpdateModel.nextAction = "";
            if (!allDocList.isEmpty()) {
                for (LPDocViewElementModel model : allDocList) {
                    if (model.docId.equals(lpDocViewUpdateModel.docId)) {
                        model.max = 0;
                        model.full = 0;
                        lpDocViewUpdateModel.all = allDocList;
                        iRouter.getLiveRoom().getDocListVM().requestDocViewUpdate(lpDocViewUpdateModel);
                        break;
                    }
                }
            }
        }
    }

    @Override
    protected LiveRoom getLiveRoom() {
        if (iRouter == null) {
            return null;
        }
        return iRouter.getLiveRoom();
    }

    private AtomicBoolean isWindowInit;
    private AtomicBoolean lateCallRoomStatusChange;

    @Override
    public boolean onRoomStatusChange(boolean isActive) {
        if (isActive) {
            if (isWindowInit.get()) {
                subscribe();
            } else {
                lateCallRoomStatusChange.set(true);
            }
        } else {
            unSubscribe();
            lateCallRoomStatusChange.set(false);
        }
        return true;
    }

    private void subscribe() {
        if (context instanceof RouterListener && iRouter == null) {
            iRouter = ((RouterListener) context).getRouter();
            presenter.subscribe(iRouter.getLiveRoom());
        }

        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.UploadPPTPath)
                .ofType(Map.class)
                .filter(map -> map.size() == 1)
                .subscribe(this::onUpload));
    }

    private void unSubscribe() {
        courseWareAdapter.onRoomLoseConnect();
        compositeDisposable.clear();
        if (presenter != null) {
            presenter.unSubscribe();
        }
        iRouter = null;
    }

    @SuppressLint("ClickableViewAccessibility")
    @Override
    public boolean onTouch(View v, @NotNull MotionEvent event) {
        return true;
    }

    @Override
    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    public void onDestroy() {
        super.onDestroy();
        presenter.destroy();
        unSubscribe();
    }

    @Override
    public View getView() {
        return contentView;
    }
}
