package com.baijiayun.groupclassui.window.coursewaremanage;

import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewStub;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.PopupWindow;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.ListPopupWindow;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.query.QueryPlus;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.bjyutils.screen.DisplayUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.util.FECompatUtil;
import com.baijiayun.groupclassui.util.InteractiveUtils;
import com.baijiayun.groupclassui.window.BaseTitledWindow;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.models.LPPlayerViewUpdateModel;
import com.baijiayun.livecore.models.courseware.LPMediaCoursewareModel;
import com.baijiayun.livecore.wrapper.LPPlayer;
import com.baijiayun.livecore.wrapper.impl.LPVideoView;
import com.baijiayun.livecore.wrapper.listener.LPRTMPPlayerListener;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;

/**
 * @author yongjiaming
 * @date 2021/7/31
 * description：音视频课件播放窗口
 */
public class MediaCoursewarePlayerWindow extends BaseTitledWindow {

    private static LoopType LOOP_TYPE = LoopType.LOOP_IN_ORDER;

    private enum LoopType {
        /**
         * 播放一次
         */
        NO_LOOP,
        /**
         * 循环播放
         */
        LOOP_IN_ORDER,
        /**
         * 单曲循环
         */
        LOOP_SELF
    }

    private View emptyView;
    private RecyclerView playerListRv;
    private TextView playTimeTv, rateTv;
    private LPMediaCoursewareModel currentMediaCourseware;
    private LPVideoView videoView;
    private ImageView playIv, loopIv, menuIv;
    private SeekBar seekBar;

    private List<LPMediaCoursewareModel> mediaCoursewareModelList = new ArrayList<>();

    private Disposable disposableOfInterval;
    private LPPlayer player;
    //是否显示播放列表
    private boolean isShowPlayList;
    private int selectedItemPosition = -1;
    private float playRate = 1.0f;
    private boolean isPlaying;
    private int currentTime;

    private LPRTMPPlayerListener rtmpPlayerListener;
    private CoursewareListAdapter coursewareListAdapter;
    private boolean showTopBottomMenu = true;

    public MediaCoursewarePlayerWindow(Context context, LPMediaCoursewareModel playMedia) {
        super(context);
        currentMediaCourseware = playMedia;
        mediaCoursewareModelList.add(currentMediaCourseware);
        hideContentMask();
        setCanScale(false);
        initView();
        dragScaleRelativeLayout.setCanScaleOrDrag(false);
    }

    @Override
    protected void onCreateTopView(Context context) {
        LayoutInflater.from(context).inflate(R.layout.bjy_group_media_player_top, rlTitleContainer);
    }

    @Override
    protected void onCreateContentView(Context context) {
        LayoutInflater.from(context).inflate(R.layout.bjy_group_media_player_content, rlContentContainer);
    }

    @Override
    protected void onCreateFooterView(Context context) {
        LayoutInflater.from(context).inflate(R.layout.bjy_group_media_player_footer, rlFooterContainer);
    }

    @Override
    protected void initTopTitleView() {
        super.initTopTitleView();
        ivMaxSync.setOnClickListener(l -> {
            isReceiveOrder = false;
            if(isMinInSync) {
                resetMinInSync();
            } else {
                if (isMaxInSync) {
                    resetInSync();
                } else {
                    maxInSync();
                }
            }
        });
        minIv = dragScaleRelativeLayout.findViewById(R.id.window_titled_min_in_sync_layer);
        minIv.setOnClickListener(v -> minInSync());
    }

    @Override
    public void minInSync() {
        ivMaxFull.setVisibility(View.GONE);
        ivMaxSync.setVisibility(View.VISIBLE);
        setOnSingleTapListener(null);
        super.minInSync();
    }

    @Override
    public void resetMinInSync() {
        ivMaxFull.setVisibility(View.VISIBLE);
        setOnSingleTapListener(getSingleTapListener());
        super.resetMinInSync();
    }

    private void initView() {
        player = iRouter.getLiveRoom().getPlayer();

        title(getString(R.string.bjy_group_media_courseware_player));
        setEmptyViewVisibility(mediaCoursewareModelList.isEmpty());
        playerListRv = $.id(R.id.media_player_list_rv).view();
        videoView = $.id(R.id.media_player_videoView).view();
        playIv = $.id(R.id.media_player_play_btn).view();
        playTimeTv = $.id(R.id.media_player_time_tv).view();
        rateTv = $.id(R.id.media_player_rate_tv).view();
        loopIv = $.id(R.id.media_player_loop_iv).view();
        menuIv = $.id(R.id.media_player_menu_iv).view();
        seekBar = $.id(R.id.media_player_seekbar).view();

        playerListRv.addItemDecoration(new DividerItemDecoration(context, DividerItemDecoration.VERTICAL));
        playerListRv.setLayoutManager(new LinearLayoutManager(context));
        coursewareListAdapter = new CoursewareListAdapter();
        playerListRv.setAdapter(coursewareListAdapter);

        switch (LOOP_TYPE) {
            case NO_LOOP:
                loopIv.setImageResource(R.drawable.bjy_group_ic_media_player_no_loop);
                break;
            case LOOP_SELF:
                loopIv.setImageResource(R.drawable.bjy_group_ic_media_player_loop_one);
                break;
            case LOOP_IN_ORDER:
                loopIv.setImageResource(R.drawable.bjy_group_ic_media_player_loop_in_order);
                break;
        }

        initListener();
        startPlay();
    }

    private OnSingleTapListener getSingleTapListener() {
        return event -> {
            showTopBottomMenu = !showTopBottomMenu;
            rlTitleContainer.setVisibility(showTopBottomMenu ? View.VISIBLE : View.GONE);
            rlFooterContainer.setVisibility(showTopBottomMenu ? View.VISIBLE : View.GONE);
            if (!showTopBottomMenu) {
                playerListRv.setVisibility(View.GONE);
            }
        };
    }

    private void initListener() {

        setOnWindowRepositionListener(new OnWindowRepositionListener() {
            @Override
            public void onWindowMove(int x, int y) {
                iRouter.getSubjectByKey(EventKey.MoveVideoWindow).onNext(getPlayerPosition());
            }

            @Override
            public void OnWindowScale(int newWidth, int newHeight) {
                iRouter.getSubjectByKey(EventKey.MoveVideoWindow).onNext(getPlayerPosition());
            }
        });

        setOnSingleTapListener(getSingleTapListener());

        rtmpPlayerListener = new LPRTMPPlayerListener() {
            @Override
            public void onPlayComplete(String mediaId) {
                isPlaying = false;
                //播完之后 currentPosition != duration，手动补一下
                int duration = player.getDurationOfRtmpStream(mediaId);
                playTimeTv.setText(context.getString(R.string.bjy_group_media_time, InteractiveUtils.formatDuration(duration), InteractiveUtils.formatDuration(duration)));
                seekBar.setProgress(duration);

                RxUtils.dispose(disposableOfInterval);
                switch (LOOP_TYPE) {
                    case NO_LOOP:
                        setEmptyViewVisibility(true);
                        seekBar.setProgress(0);
                        playIv.setVisibility(View.GONE);
                        playTimeTv.setVisibility(View.GONE);
                        coursewareListAdapter.notifyItemChanged(selectedItemPosition);
                        selectedItemPosition = -1;
                        iRouter.getLiveRoom().getMediaVM().sendBroadcastOfCloudVideoStatus(currentMediaCourseware.videoInfo.fid, LPConstants.LPCloudVideoStatus.NOT_PLAY);
                        break;
                    case LOOP_SELF:
                        iRouter.getLiveRoom().getMediaVM().sendBroadcastOfCloudVideoStatus(currentMediaCourseware.videoInfo.fid, LPConstants.LPCloudVideoStatus.NOT_PLAY);
                        startPlay();
                        break;
                    case LOOP_IN_ORDER:
                        if(mediaCoursewareModelList.size() == 1) {
                            iRouter.getLiveRoom().getMediaVM().sendBroadcastOfCloudVideoStatus(currentMediaCourseware.videoInfo.fid, LPConstants.LPCloudVideoStatus.NOT_PLAY);
                        }
                        int nextIndex = getNextMediaId(currentMediaCourseware.videoInfo.fid);
                        coursewareListAdapter.notifyItemChanged(selectedItemPosition);
                        selectedItemPosition = nextIndex;
                        coursewareListAdapter.notifyItemChanged(selectedItemPosition);
                        currentMediaCourseware = mediaCoursewareModelList.get(nextIndex);
                        startPlay();
                        break;
                }
            }

            @Override
            public void onBufferingUpdate(String mediaId, int percent) {
                // do nothing
            }

            @Override
            public void onSeekComplete(String mediaId) {
                // do nothing
            }

            @Override
            public void onReadyToPlay(String mediaId) {
                isPlaying = true;
                startInterval();
            }
        };
        player.addPlayerListener(rtmpPlayerListener);

        playIv.setOnClickListener(v -> {
            if (isPlaying) {
                RxUtils.dispose(disposableOfInterval);
                player.pauseRtmpStream(currentMediaCourseware.videoInfo.fid);
                iRouter.getLiveRoom().getMediaVM().sendBroadcastOfCloudVideoStatus(currentMediaCourseware.videoInfo.fid, LPConstants.LPCloudVideoStatus.PAUSE);
                playIv.setImageResource(R.drawable.bjy_group_ic_media_player_play);
            } else {
                player.resumeRtmpStream(currentMediaCourseware.videoInfo.fid);
                startInterval();
                iRouter.getLiveRoom().getMediaVM().sendBroadcastOfCloudVideoStatus(currentMediaCourseware.videoInfo.fid, LPConstants.LPCloudVideoStatus.PLAYING);
                playIv.setImageResource(R.drawable.bjy_group_ic_media_player_pause);
            }
            isPlaying = !isPlaying;
        });

        ivClose.setOnClickListener(v -> {
            mediaCoursewareModelList.clear();
            if (isMaxInFull) {
                resetInFull();
            }
            iRouter.getSubjectByKey(EventKey.closeMediaPlayerWindow).onNext(true);
        });

        rateTv.setOnClickListener(v -> {
            ListPopupWindow listPopupWindow = new ListPopupWindow(context);
            List<String> rateList = Arrays.asList("0.75x", "1.00x", "1.25x", "1.50x", "2.00x");
            listPopupWindow.setAdapter(new ArrayAdapter<>(context, R.layout.bjy_group_media_player_rate_item_layout, rateList));
            listPopupWindow.setWidth(UtilsKt.getDp(80));
            listPopupWindow.setHeight(ViewGroup.LayoutParams.WRAP_CONTENT);
            listPopupWindow.setAnchorView(rateTv);
            listPopupWindow.setHorizontalOffset(-(UtilsKt.getDp(80) - rateTv.getMeasuredWidth()) / 2);
            listPopupWindow.setVerticalOffset(-UtilsKt.getDp(6));
            listPopupWindow.setModal(true);
            listPopupWindow.setOnItemClickListener((parent, view, position, id) -> {
                rateTv.setText(rateList.get(position));
                switch (position) {
                    case 0:
                        playRate = 0.75f;
                        player.setRateOfRtmpStream(currentMediaCourseware.videoInfo.fid, 0.75f);
                        break;
                    case 1:
                        playRate = 1.00f;
                        rateTv.setText(context.getString(R.string.bjy_group_media_rate));
                        player.setRateOfRtmpStream(currentMediaCourseware.videoInfo.fid, 1.00f);
                        break;
                    case 2:
                        playRate = 1.25f;
                        player.setRateOfRtmpStream(currentMediaCourseware.videoInfo.fid, 1.25f);
                        break;
                    case 3:
                        playRate = 1.50f;
                        player.setRateOfRtmpStream(currentMediaCourseware.videoInfo.fid, 1.50f);
                        break;
                    case 4:
                        playRate = 2.00f;
                        player.setRateOfRtmpStream(currentMediaCourseware.videoInfo.fid, 2.00f);
                        break;
                    default:
                        break;
                }
                iRouter.getLiveRoom().getMediaVM().sendBroadcastOfCloudVideoSpeed(currentMediaCourseware.videoInfo.fid, playRate);
                listPopupWindow.dismiss();
            });
            listPopupWindow.show();
        });

        loopIv.setOnClickListener(v -> {
            PopupWindow popupWindow = new PopupWindow(context);
            View contentView = LayoutInflater.from(context).inflate(R.layout.bjy_group_media_player_loop_layout, null);
            QueryPlus $ = QueryPlus.with(contentView);
            $.id(R.id.bjy_group_player_no_loop_ll)
                    .clicked(view -> {
                        LOOP_TYPE = LoopType.NO_LOOP;
                        loopIv.setImageResource(R.drawable.bjy_group_ic_media_player_no_loop);
                        popupWindow.dismiss();
                    })
                    .id(R.id.bjy_group_player_loop_in_order_ll)
                    .clicked(view -> {
                        LOOP_TYPE = LoopType.LOOP_IN_ORDER;
                        loopIv.setImageResource(R.drawable.bjy_group_ic_media_player_loop_in_order);
                        popupWindow.dismiss();
                    })
                    .id(R.id.bjy_group_player_loop_ll)
                    .clicked(view -> {
                        LOOP_TYPE = LoopType.LOOP_SELF;
                        loopIv.setImageResource(R.drawable.bjy_group_ic_media_player_loop_one);
                        popupWindow.dismiss();
                    });
            int[] viewSize = DisplayUtils.getUnDisplayViewSize(contentView);
            popupWindow.setContentView(contentView);
            popupWindow.setWidth(ViewGroup.LayoutParams.WRAP_CONTENT);
            popupWindow.setHeight(ViewGroup.LayoutParams.WRAP_CONTENT);
            popupWindow.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
            popupWindow.setOutsideTouchable(true);
            popupWindow.setFocusable(true);
            popupWindow.showAsDropDown(loopIv, -(viewSize[0] - loopIv.getMeasuredWidth()) / 2, -viewSize[1] - loopIv.getMeasuredHeight() - UtilsKt.getDp(8));
        });

        menuIv.setOnClickListener(v -> {
            isShowPlayList = !isShowPlayList;
            playerListRv.setVisibility(isShowPlayList ? View.VISIBLE : View.GONE);
        });

        seekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                // do nothing
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {
                // do nothing
            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {
                player.seekRtmpStream(currentMediaCourseware.videoInfo.fid, seekBar.getProgress());
                iRouter.getLiveRoom().getMediaVM().sendBroadcastOfCloudVideoTime(currentMediaCourseware.videoInfo.fid, seekBar.getProgress());
                if (!isPlaying) {
                    player.resumeRtmpStream(currentMediaCourseware.videoInfo.fid);
                    startInterval();
                }
            }
        });
    }

    public void startPlay(LPMediaCoursewareModel mediaCoursewareModel) {
        int index = mediaCoursewareModelList.indexOf(mediaCoursewareModel);
        if (index == -1) {
            mediaCoursewareModelList.add(mediaCoursewareModel);
            coursewareListAdapter.notifyItemChanged(selectedItemPosition);
            selectedItemPosition = mediaCoursewareModelList.size() - 1;
            coursewareListAdapter.notifyItemInserted(selectedItemPosition);
        } else {
            coursewareListAdapter.notifyItemChanged(selectedItemPosition);
            selectedItemPosition = index;
            coursewareListAdapter.notifyItemChanged(selectedItemPosition);
        }
        pause();
        this.currentMediaCourseware = mediaCoursewareModel;
        startPlay();
    }

    private void startPlay() {
        if (currentMediaCourseware != null) {
            setEmptyViewVisibility(false);
            playIv.setVisibility(View.VISIBLE);
            playTimeTv.setVisibility(View.VISIBLE);

            player.playVideoOfRtmpStream(currentMediaCourseware.videoInfo.fid, videoView);
            player.setRateOfRtmpStream(currentMediaCourseware.videoInfo.fid, playRate);
            iRouter.getLiveRoom().getMediaVM().sendBroadcastOfCloudVideoStatus(currentMediaCourseware.videoInfo.fid, LPConstants.LPCloudVideoStatus.PLAYING);
            iRouter.getLiveRoom().getMediaVM().sendBroadcastOfCloudVideoSpeed(currentMediaCourseware.videoInfo.fid, playRate);
            playIv.setImageResource(R.drawable.bjy_group_ic_media_player_pause);
            isPlaying = true;
        }
    }

    private void startInterval() {
        RxUtils.dispose(disposableOfInterval);
        disposableOfInterval = Observable.interval(1, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aLong -> {
                    currentTime = player.getCurrentPositionOfRtmpStream(currentMediaCourseware.videoInfo.fid);
                    int duration = player.getDurationOfRtmpStream(currentMediaCourseware.videoInfo.fid);
                    playTimeTv.setText(context.getString(R.string.bjy_group_media_time, InteractiveUtils.formatDuration(currentTime), InteractiveUtils.formatDuration(duration)));
                    seekBar.setProgress(currentTime);
                    seekBar.setMax(duration);
                    iRouter.getLiveRoom().getMediaVM().sendBroadcastOfCloudVideoTime(currentMediaCourseware.videoInfo.fid, currentTime);
                });
    }

    private void setEmptyViewVisibility(boolean visible) {
        if (visible) {
            if (emptyView == null) {
                ViewStub viewStub = $.id(R.id.media_player_empty_viewStub).view();
                emptyView = viewStub.inflate();
            } else {
                emptyView.setVisibility(View.VISIBLE);
            }
        } else {
            if (emptyView != null) {
                emptyView.setVisibility(View.GONE);
            }
        }
    }

    class CoursewareListAdapter extends RecyclerView.Adapter<CoursewareListAdapter.PlayerItemViewHolder> {
        @NonNull
        @Override
        public PlayerItemViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            return new PlayerItemViewHolder(LayoutInflater.from(context).inflate(R.layout.bjy_group_media_player_list_item_layout, parent, false));
        }

        @Override
        public void onBindViewHolder(@NonNull PlayerItemViewHolder holder, int position) {
            holder.itemTv.setText(mediaCoursewareModelList.get(position).name);
            if (selectedItemPosition == position) {
                holder.itemView.setBackground(new ColorDrawable(context.getResources().getColor(R.color.base_item_foreground_color)));
            } else {
                holder.itemView.setBackground(null);
            }
            holder.itemView.setOnClickListener(v -> {
                holder.itemView.setBackground(new ColorDrawable(context.getResources().getColor(R.color.base_item_foreground_color)));
                notifyItemChanged(selectedItemPosition);
                selectedItemPosition = holder.getAdapterPosition();
                notifyItemChanged(selectedItemPosition);
                if (currentMediaCourseware != null) {
                    player.stopRtmpStream(currentMediaCourseware.videoInfo.fid);
                }
                currentMediaCourseware = mediaCoursewareModelList.get(position);
                //mp3最后转码也是mp4了~~
                startPlay();
            });
        }

        @Override
        public int getItemCount() {
            return mediaCoursewareModelList.size();
        }

        class PlayerItemViewHolder extends RecyclerView.ViewHolder {

            TextView itemTv;

            public PlayerItemViewHolder(@NonNull View itemView) {
                super(itemView);
                itemTv = itemView.findViewById(R.id.bjy_group_player_item_tv);
            }
        }
    }

    private int getNextMediaId(String mediaId) {
        for (int i = 0; i < mediaCoursewareModelList.size(); i++) {
            if (mediaId.equals(mediaCoursewareModelList.get(i).videoInfo.fid)) {
                return (i + 1) % mediaCoursewareModelList.size();
            }
        }
        return 0;
    }

    @Override
    public LPPlayerViewUpdateModel.PlayerPosition getPlayerPosition() {
        LPPlayerViewUpdateModel.PlayerPosition playerPosition = super.getPlayerPosition();
        playerPosition.id = playerPosition.mediaId = FECompatUtil.getIOSMediaIdFromMediaId(FECompatUtil.getMediaId(iRouter.getLiveRoom().getCurrentUser().getUserId(), LPConstants.MediaSourceType.Media.getValue()));
        return playerPosition;
    }

    public void pause() {
        if (currentMediaCourseware != null) {
            iRouter.getLiveRoom().getPlayer().stopRtmpStream(currentMediaCourseware.videoInfo.fid);
            iRouter.getLiveRoom().getMediaVM().sendBroadcastOfCloudVideoStatus(currentMediaCourseware.videoInfo.fid, LPConstants.LPCloudVideoStatus.NOT_PLAY);
            RxUtils.dispose(disposableOfInterval);
        }
        currentMediaCourseware = null;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        pause();
        player.removePlayerListener(rtmpPlayerListener);
    }
}
