package com.baijiayun.groupclassui.window.group;

import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.TextView;

import androidx.appcompat.widget.AppCompatImageView;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.drawable.StateListDrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.glide.Glide;
import com.baijiayun.glide.request.RequestOptions;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livecore.models.LPDivideGroupModel;
import com.baijiayun.livecore.models.imodels.IMediaModel;
import com.baijiayun.livebase.models.imodels.IUserModel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.reactivex.disposables.Disposable;


/**
 * 分组管理window
 * Created by Dujuncan on 2018/09/26.
 */
public class GroupManageWindow extends InteractiveBaseWindow {

    //点击事件：选中当前用户。可以多选。
    //长按后，拦截触摸事件，开始拖动。列表无法滚动。
    //根据拖动的位置判断是否进入分组。没有拖进去。消失。进入分组。

    private RecyclerView rvUser;
    private RecyclerView rvGrouped;
    private GroupedUserAdapter groupedUserAdapter;
    private boolean isLongPress;
    private int columns = 2;//分组列数
    private int maxGroupCount;//最大分组数
    private int currGroupCount = 0;

    private View dragingItemView;//拖动中的视图
    private FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT
            , ViewGroup.LayoutParams.WRAP_CONTENT);
    private float originX, originY;
    private List<Disposable> disposables;

    @SuppressLint("ClickableViewAccessibility")
    public GroupManageWindow(Context context, int groupCount) {
        super(context);
        view.setOnTouchListener((v, event) -> true);
        final Drawable selectedDrawable = ThemeDataUtil.getCommonWindowBg(context);
        final Drawable normalDrawable = new DrawableBuilder()
                .solidColor(ContextCompat.getColor(context,R.color.bjysc_transparent))
                .build();
        $.id(R.id.group_window_root_container).view().setBackground(new StateListDrawableBuilder()
                .normal(normalDrawable)
                .selected(selectedDrawable)
                .build());
        List<GroupedBean> groupedUsers = new ArrayList<>();
        List<IUserModel> ungroupUsers = new ArrayList<>();

        rvGrouped = $.id(R.id.group_manage_window_rv_grouped).view();
        rvUser = $.id(R.id.group_window_rv_user).view();
        $.id(R.id.group_window_iv_close).clicked(v -> {
            //右上角关闭
            iRouter.getSubjectByKey(EventKey.GroupManageLayout)
                    .onNext(false);
        });
        $.id(R.id.group_manage_window_btn_create_group).clicked(v -> {
            if (groupedUserAdapter.getItemCount() == maxGroupCount) {
                showToastMessage("最多只能创建" + maxGroupCount + "个分组");
                return;
            }
            //添加分组,是空分组
            groupedUserAdapter.addData(new GroupedBean(new ArrayList<>(), 0));
        });
        $.id(R.id.group_window_tv_all_dissolve).clicked(v -> {
            //全部解散
            iRouter.getSubjectByKey(EventKey.DisplayWindowAllDissolve)
                    .onNext(true);
        });
        $.id(R.id.group_window_tv_cancel).clicked(v -> {
            //取消
            iRouter.getSubjectByKey(EventKey.GroupManageLayout)
                    .onNext(false);
        }).view().setBackground(new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_dialog_negative_bg_color))
                .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .build());
        $.id(R.id.group_window_tv_confirm).clicked(v -> {
            //完成分组
            //同时关闭分组窗口
            iRouter.getSubjectByKey(EventKey.GroupManageLayout)
                    .onNext(false);

            LPDivideGroupModel model = new LPDivideGroupModel();
            Map<String, LPDivideGroupModel.GroupInfo> map = new HashMap<>();
            List<GroupedBean> data = groupedUserAdapter.getData();
            for (int i = 0; i < data.size(); i++) {
                LPDivideGroupModel.GroupInfo info = new LPDivideGroupModel.GroupInfo();
                GroupedBean bean = data.get(i);
                info.color = groupedUserAdapter.getBgColor(i);
                info.name = groupedUserAdapter.getGroupName(i);
                List<String> members = new ArrayList<>();
                for (int j = 0; j < bean.getGroupUsers().size(); j++) {
                    members.add(bean.getGroupUsers().get(j).getUserId());
                }
                info.members = members;
                map.put(String.valueOf(i), info);
            }
            model.groups = map;
            iRouter.getLiveRoom().requestDivideGroup(model);
        }).view().setBackground(new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .build());

        currGroupCount = groupCount;
        //未分组
        UngroupUserAdapter ungroupUserAdapter = new UngroupUserAdapter(context);
        rvUser.setLayoutManager(new LinearLayoutManager(context));
        ungroupUserAdapter.bindToRecyclerView(rvUser);
        rvUser.addOnItemTouchListener(new RecyclerView.OnItemTouchListener() {
            @Override
            public boolean onInterceptTouchEvent(RecyclerView rv, MotionEvent e) {
                return isLongPress;
            }

            @Override
            public void onTouchEvent(RecyclerView rv, MotionEvent e) {
                Log.d("分组", "onTouchEvent: " + e.getRawX() + "..." + e.getRawY());
                switch (e.getAction()) {
                    case MotionEvent.ACTION_MOVE:
                        if (dragingItemView != null) {
                            params.leftMargin += (int) (e.getRawX() - originX);
                            params.topMargin += (int) (e.getRawY() - originY);
                            dragingItemView.setLayoutParams(params);

                            originX = e.getRawX();
                            originY = e.getRawY();
                        }
                        break;
                    case MotionEvent.ACTION_UP:
                        //抬起手，分发事件
                        isLongPress = false;
                        hideDragView();
                        View view = rvGrouped.findChildViewUnder(e.getRawX(), e.getRawY());
                        if (view != null) {
                            //拖动的位置在某个分组内，更新
                            int position = rvGrouped.getChildAdapterPosition(view);
                            IUserModel groupUser = (IUserModel) dragingItemView.getTag();
                            groupedUserAdapter.updateViewPager(position, groupUser);
                            ungroupUserAdapter.removeData(groupUser);
                        }
                        break;
                    default:
                        break;
                }
            }

            @Override
            public void onRequestDisallowInterceptTouchEvent(boolean disallowIntercept) {
                // do nothing
            }
        });
        ungroupUserAdapter.setOnUserEventListener(new UngroupUserAdapter.OnUserEventListener() {
            @Override
            public void onUserSelected(List<IUserModel> selectedUsers) {
                iRouter.getSubjectByKey(EventKey.UngroupSelectedList)
                        .onNext(selectedUsers);
            }

            @Override
            public void onUserLongClick(View view, int position, float viewLeft, float viewTop, float rawX, float rawY) {
                Log.d("分组", "长按item的位置= " + position + "....x=" + view.getX() + "....y=" + view.getY());
                //长按，拦截事件
                isLongPress = true;

                originX = rawX;
                originY = rawY;

                if (dragingItemView == null) {
                    dragingItemView = LayoutInflater.from(context).inflate(R.layout.item_group_window_user, null);
                    dragingItemView.setBackground(new StateListDrawableBuilder()
                            .normal(normalDrawable)
                            .selected(selectedDrawable)
                            .build());
                }
                //长按拖动时背景颜色重新设置为白色半透明
                GradientDrawable bgDrawable = new GradientDrawable();
                bgDrawable.setColor(context.getResources().getColor(R.color.bjysc_half_white));
                bgDrawable.setCornerRadius(UtilsKt.getDp(4));
                dragingItemView.setBackgroundDrawable(bgDrawable);
                IUserModel iUserModel = ungroupUserAdapter.getData().get(position);
                //保存item数据
                dragingItemView.setTag(iUserModel);
                AppCompatImageView ivHead = dragingItemView.findViewById(R.id.item_group_window_iv_head);
                TextView tvName = dragingItemView.findViewById(R.id.item_group_window_tv_name);
                Glide.with(context.getApplicationContext()).load(iUserModel.getAvatar())
                        .apply(RequestOptions.circleCropTransform())
                        .into(ivHead);
                tvName.setText(getEncodePhoneNumber(iUserModel.getName()));
                params.width = view.getMeasuredWidth();
                params.height = view.getMeasuredHeight();
                params.leftMargin = (int) viewLeft;
                params.topMargin = (int) viewTop;
                ((ViewGroup) GroupManageWindow.this.view).addView(dragingItemView, params);
            }
        });

        //开始分组
        groupedUserAdapter = new GroupedUserAdapter(context, iRouter);
        rvGrouped.setLayoutManager(new GridLayoutManager(context, columns));
        rvGrouped.setAdapter(groupedUserAdapter);
        groupedUserAdapter.setData(groupedUsers);
        groupedUserAdapter.setOnEventListener(new GroupedUserAdapter.OnEventListener() {
            @Override
            public void closeGroup(int position, List<IUserModel> groupUsers) {
                groupedUserAdapter.removeData(position);
                ungroupUserAdapter.addData(groupUsers);
            }

            @Override
            public void onUserExitGroup(List<IUserModel> list) {
                ungroupUserAdapter.addData(list);
            }
        });

        //获取发言列表
        List<IMediaModel> speakQueueList = iRouter.getLiveRoom().getSpeakQueueVM().getSpeakQueueList();
        for (IMediaModel model : speakQueueList) {
            ungroupUsers.add(model.getUser());
        }

        maxGroupCount = ungroupUsers.size();
        ungroupUserAdapter.setData(ungroupUsers);
        if (groupCount != 0) {
            //如果选择的数量为0，不分组，全部是未分组用户
            if (ungroupUsers.size() % groupCount == 0) {
                averageGroup(groupCount, ungroupUsers, groupedUsers);
            } else {
                randomGroup(groupCount, ungroupUsers, groupedUsers);
            }
        }


        disposables = new ArrayList<>();
        Disposable joinGroupDisposable = iRouter.getSubjectByKey(EventKey.JoiningGroupList)
                .ofType(List.class)
                .subscribe(list -> {
                    //更新未分组用户
                    ungroupUserAdapter.removeData(list);
                    ungroupUserAdapter.clearSelectState();
                });
        disposables.add(joinGroupDisposable);
    }

    @Override
    protected View onCreateView(Context context) {
        return LayoutInflater.from(context).inflate(R.layout.bjy_group_window_group_manage, null);
    }

    private void hideDragView() {
        ((ViewGroup) this.view).removeView(dragingItemView);
    }


    /**
     * 随机分组
     * 示例
     * 7-2   4 3
     * 7-3   3 2 2
     * 7-4   2 2 2 1
     * 7-5   2 2 1 1 1
     * 12-5     3   3   2   2   2
     * 12-6     2   2   2   2   2   2
     * 12-7     2   2   2   2   2   1   1
     * 12-8     2   2   2   2   1   1   1   1
     * 12-9     2   2   2   1   1   1   1   1   1
     * 12-10    2   2   1   1   1   1   1   1   1   1
     * 12-11    2   1   1   1   1   1   1   1   1   1   1
     * 规则：
     * 第一组为商+1,
     * 第二组为 (人数-第一组分配的人)/(总数组-已分组的数量)+1;如果有余数,递归。直到余数为0,剩余总人数平分
     *
     * @param groupCount   分组数
     * @param ungroupUsers 未分组成员list
     * @param groupedUsers 已分组成员list
     */
    private void randomGroup(int groupCount, List<IUserModel> ungroupUsers, List<GroupedBean> groupedUsers) {
        int userCounts = ungroupUsers.size();
        if (userCounts % groupCount != 0) {
            //每组的人数
            int everyGroupUserCounts = userCounts / groupCount + 1;
            //开始进组
            GroupedBean groupedBean = new GroupedBean();
            List<IUserModel> list = new ArrayList<>();
            for (int j = 0; j < everyGroupUserCounts; j++) {
                list.add(ungroupUsers.get(j));
            }
            groupedBean.setGroupUsers(list);
            groupedBean.setGroupPosition(currGroupCount - groupCount);
            groupedUsers.add(groupedBean);
            //找出未分成成员集合，再次分配
            ungroupUsers.removeAll(list);
            //分组数-1
            randomGroup(--groupCount, ungroupUsers, groupedUsers);
        } else {
            //平分剩余的
            averageGroup(groupCount, ungroupUsers, groupedUsers);
        }
    }

    /**
     * 平均分组
     * 比如3个分组，15个学生.需要3次循环.
     * 3 组  每组5个
     * 0 - 4
     * 5 - 9
     * 10 - 14
     *
     * @param groupCount
     * @param ungroupUsers
     * @param groupedUsers
     */
    private void averageGroup(int groupCount, List<IUserModel> ungroupUsers, List<GroupedBean> groupedUsers) {
        int everyGroupUserCounts = ungroupUsers.size() / groupCount;
        for (int i = 0; i < groupCount; i++) {
            GroupedBean groupedBean = new GroupedBean();
            List<IUserModel> list = new ArrayList<>();
            for (int j = i * everyGroupUserCounts; j < everyGroupUserCounts * (i + 1); j++) {
                list.add(ungroupUsers.get(j));
            }
            groupedBean.setGroupPosition(currGroupCount - groupCount + i);
            groupedBean.setGroupUsers(list);
            groupedUsers.add(groupedBean);
        }
        //分配完毕，更新分组,清空未分组数据
        groupedUserAdapter.notifyDataSetChanged();
        ungroupUsers.clear();
    }

    @Override
    public void onDestroy() {
        for (Disposable disposable : disposables) {
            RxUtils.dispose(disposable);
        }
        disposables.clear();
        groupedUserAdapter.destroy();
        super.onDestroy();
    }
}
