package com.baijiayun.groupclassui.window.ppt;

import android.content.Context;
import android.graphics.drawable.ColorDrawable;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;

import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.container.ShapePaint;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.layer.FullScreenLayer;
import com.baijiayun.groupclassui.util.InteractiveUtils;
import com.baijiayun.groupclassui.window.BaseFixedRatioTitledWindow;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.context.LPError;
import com.baijiayun.livebase.utils.FileUtils;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.widgets.dialog.CommonDialog;
import com.baijiayun.livecore.models.LPDocViewElementModel;
import com.baijiayun.livecore.models.LPDocViewUpdateModel;
import com.baijiayun.livecore.models.LPDocumentModel;
import com.baijiayun.livecore.models.LPH5DocPlayModeModel;
import com.baijiayun.livecore.models.LPMotionEvent;
import com.baijiayun.livecore.models.animppt.LPAnimPPTPageChangeEndModel;
import com.baijiayun.livecore.ppt.PPTView;
import com.baijiayun.livecore.ppt.listener.SimplePPTStatusListener;
import com.baijiayun.livecore.ppt.whiteboard.LaserShapeLayer;
import com.baijiayun.livecore.ppt.whiteboard.OnDoubleTapListener2;
import com.baijiayun.livecore.ppt.whiteboard.shape.Shape;
import com.baijiayun.livecore.ppt.whiteboard.shape.TextShape;
import com.baijiayun.liveuibase.toolbox.browser.Tuple;

import java.util.List;

import android_serialport_api.XYDataPacket;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;

public class PPTWindow extends BaseFixedRatioTitledWindow implements ShapePaint.IShapePaintListener {

    private final String docId;

    private String docName = "";

    private PPTView pptView;
    private CommonDialog pptErrorDialog;

    private TextView pptPage;

    private ImageView ivPrevPage, ivNextPage;
    private ImageView mWindowPptRemarks;
    private TextView mPlayMode;
    private TextView tvPreview;
    private boolean mRemarksEnable = true;

    public static final String PPT_WINDOW_MOVE = "move";

    public static final String PPT_WINDOW_SCALE = "scale";

    private final ShapePaint shapePaint;

    private final LPConstants.LPUserType mUserType;

    private boolean hasPageCountChangeCallback;
    private boolean shouldHidePageView;
    private Disposable disposableOfDocAllRes;
    private final LPMotionEvent lpMotionEvent = new LPMotionEvent();
    private boolean isCacheStateFull, isCacheStateMax;

    public PPTWindow(Context context, String docId, ShapePaint shapePaint, LPConstants.LPUserType userType) {
        super(context);
        this.docId = docId;
        this.shapePaint = shapePaint;
        this.mUserType = userType;
        shapePaint.registerShapePaintListener(this);
        initPPTDoc();
        initRemarks();
        isPPTWindow = true;
    }

    @Override
    public void setWindowLayoutParams(ViewGroup.LayoutParams layoutParams) {
        super.setWindowLayoutParams(layoutParams);
        layoutParamsSmall = layoutParams;
    }

    @Override
    public boolean onRoomStatusChange(boolean isActive) {
        boolean isWindowInit = super.onRoomStatusChange(isActive);
        if (isWindowInit && isActive) {
            subscribe();
        }
        return true;
    }

    private void subscribe() {
        compositeDisposable.add(iRouter.getLiveRoom().getObservableOfH5PlayModeChange()
                .filter(playModeChangeModel -> playModeChangeModel.getPlayModeModels() != null)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(playModeChangeModel -> {
                    for (LPH5DocPlayModeModel playMode : playModeChangeModel.getPlayModeModels()) {
                        if (TextUtils.equals(playMode.getDocId(), getRealDocId())) {
                            pptView.setH5PlayMode(playMode.getPlayMode());
                            mPlayMode.setVisibility(playMode.getPlayMode() == LPConstants.H5PlayMode.Practice
                                    ? View.VISIBLE : View.GONE);
                            break;
                        }
                    }
                }));
    }

    public void setOnTouchEvent(XYDataPacket xyDataPacket) {
        if (pptView != null) {
            if (xyDataPacket.tx <= view.getLeft()) {
                xyDataPacket.tx = view.getLeft();
            } else if (xyDataPacket.tx > view.getRight()) {
                xyDataPacket.tx = view.getRight();
            }
            if (xyDataPacket.ty <= view.getTop()) {
                xyDataPacket.ty = view.getTop();
            } else if (xyDataPacket.ty > view.getBottom()) {
                xyDataPacket.ty = view.getBottom();
            }
            LaserShapeLayer.PositionInfo positionInfo = new LaserShapeLayer.PositionInfo();
            pptView.getPPTPagePositionInfo(positionInfo);
            xyDataPacket.tx = xyDataPacket.tx - view.getLeft() + positionInfo.offsetWidth;
            xyDataPacket.ty = xyDataPacket.ty - view.getTop() + positionInfo.offsetHeight;
            lpMotionEvent.setXyDataPacket(xyDataPacket);
            pptView.onZXYBMotionEvent(lpMotionEvent);
        }
    }

    public boolean contains(float x,float y) {
        return x >= view.getLeft() && x <= view.getRight() && y >= view.getTop() && y <= view.getBottom();
    }

    public void gotoPrevPage() {
        if (pptView != null && pptView.getPPTAuth()) {
            pptView.gotoPrevPage();
        }
    }

    public void gotoNextPage() {
        if (pptView != null && pptView.getPPTAuth()) {
            pptView.gotoNextPage();
        }
    }

    private int currentPage = 1;

    private void initRemarks() {
        if (mUserType == LPConstants.LPUserType.Teacher || mUserType == LPConstants.LPUserType.Assistant) {
            mWindowPptRemarks.setVisibility(View.VISIBLE);
            pptView.setRemarksEnable(true);
            mWindowPptRemarks.setSelected(mRemarksEnable);
        } else {
            mWindowPptRemarks.setVisibility(View.INVISIBLE);
        }
        mWindowPptRemarks.setOnClickListener(v -> {
            mRemarksEnable = !mRemarksEnable;
            pptView.setRemarksEnable(mRemarksEnable);
            mWindowPptRemarks.setSelected(mRemarksEnable);
        });
    }

    @Override
    protected void initDrawableIds() {
        maxDrawableId = R.drawable.ic_title_max_in_sync_layer;
        maxResetDrawableId = R.drawable.ic_title_max_reset_to_sync_layer;
        fullDrawableId = R.drawable.ic_title_full_to_full_layer;
        fullResetDrawableId = R.drawable.ic_title_full_reset_to_sync_layer;
    }

    @Override
    protected void onCreateContentView(Context context) {
        pptView = new PPTView(context);
        pptView.addLifecycle(((AppCompatActivity)context).getLifecycle());
        pptView.setPPTBackground(new ColorDrawable(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_bg_color)));
        pptView.hidePageView();
        pptView.setShowPaintOwnerEnable(true);
        pptView.setOnViewTapListener((view, x, y) -> {
            if (iRouter != null) {
                iRouter.getSubjectByKey(EventKey.BgSingleTapConfirmed).onNext(true);
            }
        });
        pptView.setPPTStatusListener(new SimplePPTStatusListener() {
            @Override
            public void onPPTViewAttached() {
                pptView.setCustomShapeStrokeWidth(shapePaint.getCustomStrokeWidth());
                pptView.setShapeStrokeWidth(shapePaint.getDoodleStrokeWidth());
                pptView.setPaintColor(shapePaint.getShapeColor());
                pptView.setPaintTextSize(shapePaint.getTextSize());
                pptView.setShapeType(shapePaint.getShapeType());
                pptView.setPPTEditMode(shapePaint.getPPtEditMode());

                if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student) {
                    pptView.setFlingEnable(false);
                    pptView.changeTouchAble(InteractiveUtils.hasDrawingAuth(iRouter.getLiveRoom(), iRouter.getLiveRoom().getCurrentUser()));
                }
                setPPTAuthView(iRouter.getLiveRoom().getDocListVM().hasPPTAuth());
            }

            @Override
            public void onPageChange(LPAnimPPTPageChangeEndModel pageChangeEndModel, String remarkInfo) {
                currentPage = pageChangeEndModel.page;
                updatePageInfo(currentPage, pptView.getTotalPageNumber());
            }

            @Override
            public void onH5PageCountChange(String docId, int pageCount) {
                hasPageCountChangeCallback = true;
                updatePageInfo(currentPage, pageCount);
            }

            @Override
            public void onPPTError(LPError lpError) {
                super.onPPTError(lpError);
                if (lpError.getCode() == LPError.CODE_ERROR_ANIMPPT_LOAD_TIMEOUT) {
                    if (pptErrorDialog == null) {
                        pptErrorDialog = new CommonDialog(context, CommonDialog.ChoiceMode.Double_Blue)
                                .setMainDisplayText(lpError.getMessage() + "\n"
                                        + getString(R.string.anim_ppt_error_suggestion_over_time))
                                .setNegative(getString(R.string.anim_ppt_error_skip_anim), ((dialog, which) -> {
                                    if (pptView != null) {
                                        pptView.setAnimPPTEnable(false);
                                    }
                                    dialog.dismiss();
                                }))
                                .setPositive(getString(R.string.anim_ppt_error_reload), ((dialog, which) -> {
                                    if (pptView != null) {
                                        pptView.reloadAnimPPT();
                                    }
                                    dialog.dismiss();
                                }));
                    }
                    pptErrorDialog.show();
                }
            }

            @Override
            public void onAnimPPTLoadFinish() {
                super.onAnimPPTLoadFinish();
                if (pptErrorDialog != null && pptErrorDialog.isShowing()) {
                    pptErrorDialog.dismiss();
                }
            }
        });
        pptView.setOnDoubleTapListener(new OnDoubleTapListener2() {
            @Override
            public void onDoubleTapConfirmed() {
                // do nothing
            }

            @Override
            public void onDoubleTapOnShape(Shape shape) {
                iRouter.getSubjectByKey(EventKey.EditTextShape).onNext(((TextShape) shape).getText());
            }
        });
        pptView.setOnShapeSelectedListener(selected -> iRouter.getSubjectByKey(EventKey.Action2ShapeSelected).onNext(selected));
        rlContentContainer.addView(pptView);
        RelativeLayout.LayoutParams layoutParams = (RelativeLayout.LayoutParams) contentOuterContainer.getLayoutParams();
        layoutParams.removeRule(RelativeLayout.ABOVE);
        layoutParams.removeRule(RelativeLayout.BELOW);
    }

    @Override
    protected void onCreateFooterView(Context context) {
        View footerContainer = LayoutInflater.from(context).inflate(R.layout.layout_ppt_window_footer, rlFooterContainer);
        pptPage = footerContainer.findViewById(R.id.window_center_pageNumber);
        ivPrevPage = footerContainer.findViewById(R.id.window_titled_prev_page);
        ivNextPage = footerContainer.findViewById(R.id.window_titled_next_page);
        mWindowPptRemarks = footerContainer.findViewById(R.id.window_ppt_remarks);
        tvPreview = footerContainer.findViewById(R.id.window_ppt_preview);
        mPlayMode = footerContainer.findViewById(R.id.window_h5_play_mode);
    }

    @Override
    protected void onCreateTopView(Context context) {
        LayoutInflater.from(context).inflate(R.layout.layout_ppt_window_top, rlTitleContainer);
    }

    /**
     * H5课件&没有page_count_change回调则隐藏页码和翻页按钮，事件直接丢给webview
     *
     * @param page
     */
    private void updatePageInfo(int page, int totalPage) {
        if (iRouter.getLiveRoom().getDocListVM().isContainH5PPTDoc(getRealDocId()) && !hasPageCountChangeCallback) {
            shouldHidePageView = true;
            pptPage.setVisibility(View.GONE);
        } else {
            shouldHidePageView = false;
            pptPage.setVisibility(View.VISIBLE);
        }
        String str = ++page + "/" + totalPage;
        pptPage.setText(str);
    }

    public void hideTitleFooter() {
        if (isAdmin()) {
            rlTitleContainer.setVisibility(View.GONE);
            rlFooterContainer.setVisibility(View.GONE);
        }
    }

    public void showTitleFooter() {
        if (isAdmin()) {
            rlTitleContainer.setVisibility(View.VISIBLE);
            rlFooterContainer.setVisibility(View.VISIBLE);
        }
    }

    public void hideTitle() {
        if (isAdmin()) {
            rlTitleContainer.setVisibility(View.GONE);
        }
    }

    public void showTitle() {
        if (isAdmin()) {
            rlTitleContainer.setVisibility(View.VISIBLE);
        }
    }

    private void initPPTDoc() {
        pptView.setDocId(getRealDocId());
        pptView.setPreviewDoc(docId.contains(LPConstants.PREVIEW_PPT_DOC_ID_SUFFIX));
        tvPreview.setVisibility(docId.contains(LPConstants.PREVIEW_PPT_DOC_ID_SUFFIX) ? View.VISIBLE : View.GONE);
        // attention attachLiveRoom需要在设置docId之后，attachLiveRoom之后接收信令需要根据docId判断
        pptView.attachLiveRoom(iRouter.getLiveRoom());
        if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student) {
            ivClose.setVisibility(View.GONE);
            ivMaxFull.setVisibility(View.GONE);
            ivMaxSync.setVisibility(View.GONE);
            rlFooterContainer.setVisibility(View.VISIBLE);
        } else { // 老师和助教
            if (shapePaint.getPPtEditMode() == LPConstants.PPTEditMode.Normal || shapePaint.getPPtEditMode() == LPConstants.PPTEditMode.PPTTouchMode) {
                showTitleFooter();
            } else {
                hideTitleFooter();
            }

            ivClose.setOnClickListener(v -> iRouter.getSubjectByKey(EventKey.ClosePPTWindow).onNext(new Tuple(getId())));

            setOnSingleTapListener(event -> {
                if (iRouter == null) {
                    return;
                }
                iRouter.getSubjectByKey(EventKey.PPTWindowBringToFront).onNext(getId());
            });

            setOnWindowRepositionListener(new OnWindowRepositionListener() {
                @Override
                public void onWindowMove(int x, int y) {
                    if (PPTWindow.this.getParentViewGroup() instanceof FullScreenLayer || !isAdmin()) {
                        return;
                    }
                    iRouter.getSubjectByKey(EventKey.RepositionPPTWindow).onNext(new Tuple(PPT_WINDOW_MOVE, getId(), x, y));
                }

                @Override
                public void OnWindowScale(int newWidth, int newHeight) {
                    if (PPTWindow.this.getParentViewGroup() instanceof FullScreenLayer || !isAdmin()) {
                        return;
                    }
                    iRouter.getSubjectByKey(EventKey.RepositionPPTWindow).onNext(new Tuple(PPT_WINDOW_SCALE, getId(), newWidth, newHeight));
                }
            });

            if (shapePaint.getPPtEditMode() == LPConstants.PPTEditMode.Normal) {
                rlFooterContainer.setVisibility(View.VISIBLE);
            }
        }

        ivNextPage.setOnClickListener(v -> pptView.gotoNextPage());

        ivPrevPage.setOnClickListener(v -> pptView.gotoPrevPage());

        disposableOfDocAllRes = iRouter.getLiveRoom().getDocListVM().getObservableOfDocAll()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpResRoomDocAllModel -> {
                    for (LPDocumentModel lpDocumentModel : lpResRoomDocAllModel.docList) {
                        if (TextUtils.equals(lpDocumentModel.id,getRealDocId())) {
                            //图片不显示底部栏
                            //还是不用isDoc字段判断
                            if (FileUtils.isImageFile(lpDocumentModel.ext) && rlFooterContainer != null) {
                                if (isPreviewPPT()) {
                                    pptPage.setVisibility(View.INVISIBLE);
                                    ivPrevPage.setVisibility(View.INVISIBLE);
                                    ivNextPage.setVisibility(View.INVISIBLE);
                                    mWindowPptRemarks.setVisibility(View.INVISIBLE);
                                    tvPreview.setVisibility(View.VISIBLE);
                                    mPlayMode.setVisibility(View.INVISIBLE);
                                } else {
                                    rlFooterContainer.setVisibility(View.INVISIBLE);
                                }
                            }
                            docName = lpDocumentModel.name;
                            if (isPreviewPPT()) {
                                docName = context.getString(R.string.bjy_group_doc_preview_title_prefix) + lpDocumentModel.name;
                            }
                            tvTitle.setText(docName);
                            //刚获取到名字
                            final PPTTabLayer pptTabLayer = iRouter.getValueByKey(EventKey.PPTTabLayer, PPTTabLayer.class);
                            if (pptTabLayer != null && isAdmin()) {
                                pptTabLayer.refreshTabs();
                            }
                            break;
                        }
                    }
                });
    }

    /**
     * 切换tab
     *
     * @param destWindow
     */
    public void switchTab(PPTWindow destWindow) {
        if (destWindow == PPTWindow.this) {
            return;
        }
        isReceiveOrder = false;
        //把原来的ppt还原，再把点击的放大
        if (isMaxInSync) {
            sendCompatSignal(false, true);
            destWindow.maxInSync(false);
            isCacheStateMax = true;
        }
        if (isMaxInFull) {
            sendCompatSignal(true, true);
            destWindow.maxInFull(false);
            sendCompatSignal(false, false);
            isCacheStateFull = true;
        }
    }

    /**
     * 兼容老版本信令
     */
    private void sendCompatSignal(boolean isFull, boolean hasNextAction) {
        if (!isAdmin() || (getId().contains(LPConstants.PREVIEW_PPT_DOC_ID_SUFFIX))) {
            return;
        }
        String docId = getId();
        List<LPDocViewElementModel> allDocList = iRouter.getListByKey(EventKey.AllDocViewList, LPDocViewElementModel.class);
        LPDocViewUpdateModel lpDocViewUpdateModel = new LPDocViewUpdateModel();
        lpDocViewUpdateModel.docId = docId;
        lpDocViewUpdateModel.action = "normal";
        if (hasNextAction) {
            lpDocViewUpdateModel.nextAction = isFull ? "full" : "max";
        }
        if (allDocList == null || allDocList.isEmpty()) {
            return;
        }
        for (LPDocViewElementModel elementModel : allDocList) {
            if (elementModel.docId.equals(lpDocViewUpdateModel.docId)) {
                elementModel.max = 0;
                elementModel.full = 0;
                lpDocViewUpdateModel.all = allDocList;
                iRouter.getLiveRoom().getDocListVM().requestDocViewUpdate(lpDocViewUpdateModel);
                break;
            }
        }
    }

    @Override
    public void bringToForeground() {
        super.bringToForeground();
        final ViewGroup parentViewGroup = getParentViewGroup();
        if (parentViewGroup == null) {
            return;
        }
        if (iRouter == null) {
            return;
        }
        //在full状态时才会置顶，非full状态不在一个容器中
        final PPTTabLayer pptTabLayer = iRouter.getValueByKey(EventKey.PPTTabLayer, PPTTabLayer.class);
        if (pptTabLayer != null && parentViewGroup.indexOfChild(pptTabLayer) != -1) {
            pptTabLayer.bringToFront();
        }
        iRouter.getSubjectByKey(EventKey.PPTWindowBringToFrontLocal).onNext(getId());
    }

    public void maxInFull(boolean isReceiverOrder) {
        setReceiveOrder(isReceiverOrder);
        maxInFull();
        if (boundView != null && boundView.getShadowView() != null) {
            boundView.getShadowView().setVisibility(View.GONE);
        }
        if (!isAdmin()) {
            ivMaxFull.setVisibility(View.GONE);
        }
    }

    public void maxInFullCache(boolean isReceiverOrder) {
        setReceiveOrder(isReceiverOrder);
        iRouter.setObjectByKey(EventKey.MaxScreenPPTWindow, null);
        isCacheStateFull = true;
        isCacheStateMax = false;
        super.maxInFull();
        setCanScale(false);
        if (boundView != null && boundView.getShadowView() != null) {
            boundView.getShadowView().setVisibility(View.GONE);
        }
        if (!isAdmin()) {
            ivMaxFull.setVisibility(View.GONE);
        }
    }

    @Override
    public void maxInFull() {
        iRouter.setObjectByKey(EventKey.FullScreenPPTWindow, this);
        iRouter.setObjectByKey(EventKey.MaxScreenPPTWindow, null);
        if (isCacheStateFull) {
            dragScaleRelativeLayout.bringToFront();
            iRouter.getSubjectByKey(EventKey.SetFullScreenWindowOn).onNext(this);
        } else {
            super.maxInFull();
            setCanScale(false);
            isCacheStateFull = true;
            isCacheStateMax = false;
        }
    }

    public void maxInSync(boolean isReceiverOrder) {
        setReceiveOrder(isReceiverOrder);
        maxInSync();
        if (boundView != null && boundView.getShadowView() != null) {
            boundView.getShadowView().setVisibility(View.GONE);
        }
        if (!isAdmin()) {
            ivMaxSync.setVisibility(View.GONE);
        }
    }

    public void maxInSyncCache(boolean isReceiverOrder) {
        isCacheStateMax = true;
        setReceiveOrder(isReceiverOrder);
        super.maxInSync();
        setCanScale(false);
        if (boundView != null && boundView.getShadowView() != null) {
            boundView.getShadowView().setVisibility(View.GONE);
        }
        if (!isAdmin()) {
            ivMaxSync.setVisibility(View.GONE);
        }
    }

    @Override
    public void maxInSync() {
        if (checkNotTopPPTWindow(EventKey.FullScreenPPTWindow)) {
            return;
        }
        iRouter.setObjectByKey(EventKey.MaxScreenPPTWindow, this);
        if (isCacheStateMax) {
            dragScaleRelativeLayout.bringToFront();
            iRouter.getSubjectByKey(EventKey.SetMaximumWindowOn).onNext(this);
        } else {
            super.maxInSync();
            setCanScale(false);
            isCacheStateMax = true;
        }
    }

    public void resetInFull(boolean isReceiverOrder) {
        setReceiveOrder(isReceiverOrder);
        resetInFull();
        if (boundView != null && boundView.getShadowView() != null) {
            boundView.getShadowView().setVisibility(View.VISIBLE);
        }
        if (!isAdmin()) {
            ivMaxFull.setVisibility(View.GONE);
            ivMaxSync.setVisibility(View.GONE);
        }
    }

    @Override
    public void resetInFull() {
        if (isMaxInFull && checkNotTopPPTWindow(EventKey.FullScreenPPTWindow)) {
            isCacheStateFull = true;
        } else {
            isCacheStateFull = false;
            if (checkTopPPTWindow(EventKey.FullScreenPPTWindow)) {
                iRouter.setObjectByKey(EventKey.FullScreenPPTWindow, null);
            }
            super.resetInFull();
            setCanScale(true);

        }
    }

    public void resetInSync(boolean isReceiverOrder) {
        setReceiveOrder(isReceiverOrder);
        resetInSync();
        if (boundView != null && boundView.getShadowView() != null) {
            boundView.getShadowView().setVisibility(View.VISIBLE);
        }
        if (!isAdmin()) {
            ivMaxSync.setVisibility(View.GONE);
            ivMaxFull.setVisibility(View.GONE);
        }
    }

    @Override
    public void resetInSync() {
        if (isMaxInSync && checkNotTopPPTWindow(EventKey.MaxScreenPPTWindow)) {
            isCacheStateMax = true;
        } else {
            isCacheStateMax = false;
            iRouter.setObjectByKey(EventKey.MaxScreenPPTWindow, null);
            super.resetInSync();
            setCanScale(true);
        }
    }

    private boolean checkNotTopPPTWindow(EventKey eventKey) {
        final PPTWindow pptWindow = iRouter.getValueByKey(eventKey, PPTWindow.class);
        //存在其他全屏PPT，自身的PPTWindow缓存不缩小
        return pptWindow != null && pptWindow != this;
    }

    private boolean checkTopPPTWindow(EventKey eventKey) {
        final PPTWindow pptWindow = iRouter.getValueByKey(eventKey, PPTWindow.class);
        return pptWindow == this;
    }

    public void setPPTAuthView(boolean isPPTAuth) {
        // 实际操作权限为 isPPTAuth 或 预览 ppt
        boolean realPPTAuth = isPPTAuth || docId.contains(LPConstants.PREVIEW_PPT_DOC_ID_SUFFIX);
        if (realPPTAuth && !shouldHidePageView) {
            ivPrevPage.setVisibility(View.VISIBLE);
            ivNextPage.setVisibility(View.VISIBLE);
        } else {
            ivPrevPage.setVisibility(View.GONE);
            ivNextPage.setVisibility(View.GONE);
        }
        if (!isAdmin() && !iRouter.getLiveRoom().getPartnerConfig().enableH5PPTPageDown && pptView.isH5PPT(getRealDocId())) {
            ivPrevPage.setVisibility(View.GONE);
            ivNextPage.setVisibility(View.GONE);
        }
        pptView.setAnimPPTAuth(realPPTAuth);
    }

    public boolean isCacheStateFull() {
        return isCacheStateFull;
    }

    public boolean isCacheStateMax() {
        return isCacheStateMax;
    }

    @Override
    public boolean isMaxInFull() {
        return super.isMaxInFull() && checkTopPPTWindow(EventKey.FullScreenPPTWindow);
    }

    @Override
    public boolean isMaxInSync() {
        return super.isMaxInSync() && checkTopPPTWindow(EventKey.MaxScreenPPTWindow);
    }

    public PPTView getPPTView() {
        return pptView;
    }


    public long getRealDocIdWithNum() {
        long id = 0;
        try {
            id = Long.parseLong(getRealDocId());
        } catch (NumberFormatException e) {
            e.printStackTrace();
        }
        return id;
    }

    /**
     * 获取当前 window 内真实在显示的 ppt doc id
     *
     * @return 如果不是预览 ppt，则直接返回 docId，否则返回 docId 截去 预览 ppt 后缀的真实 docId
     */
    public String getRealDocId() {
        return docId.replace(LPConstants.PREVIEW_PPT_DOC_ID_SUFFIX, "");
    }

    public boolean isPreviewPPT() {
        return docId.contains(LPConstants.PREVIEW_PPT_DOC_ID_SUFFIX);
    }

    public String getDocName() {
        return docName;
    }

    @Override
    public void onShapeCustomStrokeWidthChange(float strokeWidth) {
        pptView.setCustomShapeStrokeWidth(strokeWidth);
    }

    @Override
    public void onShapeDoodleStrokeWidthChange(float strokeWidth) {
        pptView.setShapeStrokeWidth(strokeWidth);
    }

    @Override
    public void onShapeColorChange(int shapeColor) {
        pptView.setPaintColor(shapeColor);
    }

    @Override
    public void onShapeTypeChange(LPConstants.ShapeType shapeType) {
        pptView.setShapeType(shapeType);
    }

    @Override
    public void onShapeEditModeChange(LPConstants.PPTEditMode pptEditMode) {
        // 若为 normal - 可拖拽拉伸 对应状态 : ToolBar ppt 按钮未选中
        dragScaleRelativeLayout.setCanScaleOrDrag(pptEditMode == LPConstants.PPTEditMode.Normal);
        // 若为 pptTouchMode - 可点击 ppt 点击穿透标题栏 对应状态 : ToolBar ppt 按钮选中
        dragScaleRelativeLayout.setForceNotConsume(pptEditMode == LPConstants.PPTEditMode.PPTTouchMode);
        pptView.setPPTEditMode(pptEditMode);
    }

    @Override
    public void onShapeTextSizeChange(int textSize) {
        pptView.setPaintTextSize(textSize);
    }

    @Override
    public void onSendDrawTextConfirmed(String beforeText, String content) {
        pptView.sendDrawTextConfirmed(beforeText, content);
    }


    /**
     * 获取当前 window 的标识 id
     *
     * @return 预览 ppt 会返回带有 preview 后缀 {@link LPConstants#PREVIEW_PPT_DOC_ID_SUFFIX} 的 docId
     */
    @Override
    public String getId() {
        return docId;
    }

    @Override
    public void onDestroy() {
        if (shapePaint != null) {
            shapePaint.unregisterShapePaintListener(this);
        }
        RxUtils.dispose(disposableOfDocAllRes);
        super.onDestroy();
    }
}


