package com.baijiayun.groupclassui.window.setting;

import android.content.Context;
import android.view.View;

import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.global.GroupClassActivity;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.context.LPError;
import com.baijiayun.livecore.LiveSDK;
import com.baijiayun.liveuibase.utils.BaseUtilsKt;
import com.baijiayun.liveuibase.widgets.SettingSwitch;
import com.baijiayun.liveuibase.widgets.setting.BaseSettingPresenter;
import com.baijiayun.liveuibase.widgets.setting.BaseSettingWindow;
import com.baijiayun.liveuibase.widgets.switchCompat.MultiSwitchCompat;

import java.util.List;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;

public class SettingWindow extends BaseSettingWindow {
    private IRouter iRouter;
    protected Disposable disposableOfOtherStudentVideo, disposableOfMirrorMode, disposableOfAudioBitrate;

    public SettingWindow(Context context) {
        super(context);
    }

    @Override
    protected void initCameraContainer() {
        $camera.clearRefCache();
        updateCameraContainer();
        SettingSwitch definitionSwitch = $camera.id(R.id.bjy_base_setting_camera_quality_switch).view();
        List<LPConstants.LPResolutionType> resolutionTypeList = BaseUtilsKt.getResolutionList(iRouter.getLiveRoom());
        List<String> optionArray = BaseUtilsKt.getResolutionList(context, iRouter.getLiveRoom());
        definitionSwitch.setSubOptionList(optionArray);
        definitionSwitch.setCheckedPosition(resolutionTypeList.indexOf(presenter.getVideoDefinition()));
        definitionSwitch.setOnCheckedChangeListener((MultiSwitchCompat.OnCheckedChangeListener) (buttonView, checkedPosition) -> {
            LPError lpError = presenter.setCaptureVideoDefinition(resolutionTypeList.get(checkedPosition));
            if (lpError != null) {
                showToastMessage(lpError.getMessage());
            }
        });
    }

    private void updateCameraContainer() {
        $camera.id(R.id.bjy_base_setting_camera_open_text).visibility(View.GONE);
        $camera.id(R.id.bjy_base_setting_camera_open_switch).visibility(View.GONE);
        $camera.id(R.id.bjy_base_setting_camera_orientation_text).visibility(View.GONE);
        $camera.id(R.id.bjy_base_setting_camera_orientation_switch).visibility(View.GONE);
    }

    @Override
    protected void initBeautyContainer() {
        super.initBeautyContainer();
    }

    @Override
    protected void initMicContainer() {
        // do nothing
    }

    @Override
    protected void initPPTContainer() {
        $ppt.clearRefCache();
        updatePPTContainer();
        ((SettingSwitch) $ppt.id(R.id.bjy_base_setting_ppt_quality_switch).view()).setCheckedStatus(LiveSDK.SHOW_STATIC_PPT_ORIGIN_SIZE);
        $ppt.id(R.id.bjy_base_setting_ppt_quality_switch)
                .<SettingSwitch>view()
                .setOnCheckedChangeListener((MultiSwitchCompat.OnCheckedChangeListener) (buttonView, checkedPosition) -> {
                    if (checkedPosition == 0) {
                        LiveSDK.SHOW_STATIC_PPT_ORIGIN_SIZE = true;
                        iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(context.getString(R.string.setting_ppt_definition_switch_to_original_tip));
                        iRouter.getSubjectByKey(EventKey.UpdatePPTView).onNext(true);
                    } else {
                        LiveSDK.SHOW_STATIC_PPT_ORIGIN_SIZE = false;
                        iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(context.getString(R.string.setting_ppt_definition_switch_to_smooth_tip));
                        iRouter.getSubjectByKey(EventKey.UpdatePPTView).onNext(false);
                    }
                });
    }

    private void updatePPTContainer() {
        $ppt.id(R.id.bjy_base_setting_ppt_remark_text).visibility(View.GONE);
        $ppt.id(R.id.bjy_base_setting_ppt_remark_switch).visibility(View.GONE);
        $ppt.id(R.id.bjy_base_setting_ppt_show_way_text).visibility(View.GONE);
        $ppt.id(R.id.bjy_base_setting_ppt_show_way_switch).visibility(View.GONE);
        $ppt.id(R.id.bjy_base_setting_ppt_view_type_text).visibility(View.GONE);
        $ppt.id(R.id.bjy_base_setting_ppt_view_type_switch).visibility(View.GONE);
        $ppt.id(R.id.bjy_base_setting_student_preview_text).visibility(View.GONE);
        $ppt.id(R.id.bjy_base_setting_student_preview_switch).visibility(View.GONE);
        $ppt.id(R.id.bjy_base_setting_ppt_quality_text).visibility(View.VISIBLE);
        $ppt.id(R.id.bjy_base_setting_ppt_quality_switch).visibility(View.VISIBLE);
        $ppt.id(R.id.bjy_base_setting_ppt_quality_tip_text).visibility(View.VISIBLE);
    }

    @Override
    protected void initOtherContainer() {
        super.initOtherContainer();
        disposableOfAudioBitrate = iRouter.getLiveRoom().getRecorder().getObservableOfUpPacketLossRate()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(localStreamStats -> $other.id(R.id.bjy_base_setting_music_mode_debug_tv).text("本地音频码率：" + localStreamStats.audioBitrateSent + "Kbps"));
    }

    @Override
    protected void initRoomControlContainer() {
        $roomControl.clearRefCache();
        updateRoomControlContainer();

        final boolean hideOtherStudentVideo = iRouter.getSubjectValueByKey(EventKey.HideOtherStudentVideo, Boolean.class, false);
        ((SettingSwitch) $roomControl.id(R.id.bjy_base_setting_see_teacher_switch).view()).setCheckedStatus(hideOtherStudentVideo);
        disposableOfOtherStudentVideo = iRouter.getLiveRoom().getObservableOfDisableOtherStudentVideo()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpDisableOtherStuVideoModel -> ((SettingSwitch) $roomControl.id(R.id.bjy_base_setting_see_teacher_switch).view()).setCheckedStatus(lpDisableOtherStuVideoModel.isDisable(), true));
        $roomControl.id(R.id.bjy_base_setting_see_teacher_switch)
                .<SettingSwitch>view()
                .setOnCheckedChangeListener((MultiSwitchCompat.OnCheckedChangeListener) (buttonView, checkedPosition) -> iRouter.getSubjectByKey(EventKey.HideOtherStudentVideo).onNext(checkedPosition == 0));

        onMirrorModeChange(null, presenter.getHorizontalMirrorModeSet(), presenter.getVerticalMirrorModeSet());

        disposableOfMirrorMode = iRouter.getLiveRoom().getSpeakQueueVM().getObservableOfMirrorMode()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpMirrorModeModel -> onMirrorModeChange(lpMirrorModeModel,
                        iRouter.getLiveRoom().getSpeakQueueVM().getHorizontalMirrorModeSet(),
                        iRouter.getLiveRoom().getSpeakQueueVM().getVerticalMirrorModeSet()));

        $roomControl.id(R.id.bjy_base_setting_all_horizontal_mirror_open).clicked(v -> {
            iRouter.getLiveRoom().requestMirrorMode(true, iRouter.getLiveRoom().getSpeakQueueVM().getVerticalMirrorModeSet().contains(iRouter.getLiveRoom().getCurrentUser().getNumber()));
            iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(context.getResources().getString(R.string.bjy_group_open_horizontal_mirror_mode_tip));
        });
        $roomControl.id(R.id.bjy_base_setting_all_vertical_mirror_open).clicked(v -> {
            iRouter.getLiveRoom().requestMirrorMode(iRouter.getLiveRoom().getSpeakQueueVM().getHorizontalMirrorModeSet().contains(iRouter.getLiveRoom().getCurrentUser().getNumber()), true);
            iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(context.getResources().getString(R.string.bjy_group_open_vertical_mirror_mode_tip));
        });
        $roomControl.id(R.id.bjy_base_setting_all_horizontal_mirror_close).clicked(v -> {
            iRouter.getLiveRoom().requestMirrorMode(false, iRouter.getLiveRoom().getSpeakQueueVM().getVerticalMirrorModeSet().contains(iRouter.getLiveRoom().getCurrentUser().getNumber()));
            iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(context.getResources().getString(R.string.bjy_group_close_horizontal_mirror_mode_tip));
        });
        $roomControl.id(R.id.bjy_base_setting_all_vertical_mirror_close).clicked(v -> {
            iRouter.getLiveRoom().requestMirrorMode(iRouter.getLiveRoom().getSpeakQueueVM().getHorizontalMirrorModeSet().contains(iRouter.getLiveRoom().getCurrentUser().getNumber()), false);
            iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(context.getResources().getString(R.string.bjy_group_close_vertical_mirror_mode_tip));
        });

    }

    private void updateRoomControlContainer() {
        $roomControl.id(R.id.bjy_base_setting_forbid_raise_hands_text).visibility(View.GONE);
        $roomControl.id(R.id.bjy_base_setting_forbid_raise_hands_switch).visibility(View.GONE);
        $roomControl.id(R.id.bjy_base_setting_forbid_all_text).visibility(View.GONE);
        $roomControl.id(R.id.bjy_base_setting_forbid_all_switch).visibility(View.GONE);
        $roomControl.id(R.id.bjy_base_setting_auto_pip_text).visibility(View.GONE);
        $roomControl.id(R.id.bjy_base_setting_auto_pip_switch).visibility(View.GONE);
        if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student) {
            $roomControl.id(R.id.bjy_base_setting_see_teacher_text).visibility(View.VISIBLE);
            $roomControl.id(R.id.bjy_base_setting_see_teacher_switch).visibility(View.VISIBLE);
            $roomControl.id(R.id.bjy_base_setting_all_horizontal_mirror_text).visibility(View.GONE);
            $roomControl.id(R.id.bjy_base_setting_all_horizontal_mirror_open).visibility(View.GONE);
            $roomControl.id(R.id.bjy_base_setting_all_horizontal_mirror_close).visibility(View.GONE);
            $roomControl.id(R.id.bjy_base_setting_all_vertical_mirror_text).visibility(View.GONE);
            $roomControl.id(R.id.bjy_base_setting_all_vertical_mirror_open).visibility(View.GONE);
            $roomControl.id(R.id.bjy_base_setting_all_vertical_mirror_close).visibility(View.GONE);
        }
    }

    @Override
    protected BaseSettingPresenter initPresenter() {
        return new BaseSettingPresenter(this) {
            @Override
            public void showBleDialog() {
                iRouter.getSubjectByKey(EventKey.DisPlayBleDialog).onNext(true);
            }
        };
    }


    @Override
    protected void initChildData() {
        iRouter = ((GroupClassActivity) context).getRouter();
        presenter.subscribe(iRouter.getLiveRoom());
    }

    @Override
    public void onDestroy() {
        RxUtils.dispose(disposableOfMirrorMode);
        RxUtils.dispose(disposableOfOtherStudentVideo);
        RxUtils.dispose(disposableOfAudioBitrate);
        super.onDestroy();
    }

    @Override
    protected void onCloseWindow() {
        routerListener.onDismissSetting();
    }
}
