package com.baijiayun.groupclassui.window.study

import android.content.Context
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.constraintlayout.widget.ConstraintSet
import androidx.core.content.ContextCompat
import com.baijiayun.bjyutils.drawable.DrawableBuilder
import com.baijiayun.bjyutils.drawable.StateListDrawableBuilder
import com.baijiayun.bjyutils.kt.dp
import com.baijiayun.bjyutils.reactivex.RxUtils
import com.baijiayun.bjyutils.screen.DisplayUtils.setRoundCorner
import com.baijiayun.groupclassui.R
import com.baijiayun.groupclassui.global.EventKey
import com.baijiayun.groupclassui.global.IRouter
import com.baijiayun.livebase.utils.ThemeDataUtil
import com.baijiayun.liveuibase.widgets.popupwindow.BaseAutoArrangePopupWindow
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable

/**
 * Created by pengwenjian on 2021/03/10
 * Describe:
 */
class QuestionSelectWindow(context: Context, val router: IRouter, hasDot: Boolean) : BaseAutoArrangePopupWindow(context) {
    var itemClickListener: OnItemClickListener? = null
    private var disposableOfNewMsg: Disposable? = null
    private var disposableOfNewMsgClose: Disposable? = null

    init {
        val container = ConstraintLayout(context)
        setDirectionMode(HORIZONTAL)
        container.background = DrawableBuilder()
                .cornerRadius(context.resources.getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .strokeColor(ContextCompat.getColor(context, R.color.base_bg_stroke_10))
                .strokeWidth(1)
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_brand_container_color))
                .build()
        setRoundCorner(container, context.resources.getDimension(R.dimen.bjy_base_common_bg_radius))
        val text1 = addTextView(container, null, context.getString(R.string.bjysc_study_room_quick_question), isAddDiv = true)
                .apply {
                    setOnClickListener {
                        itemClickListener?.onItemClick(0)
                        dismiss()
                    }
                }
        addTextView(container, text1, context.getString(R.string.bjysc_study_room_question_record), showDot = hasDot)
                .run {
                    setOnClickListener {
                        itemClickListener?.onItemClick(1)
                        dismiss()
                    }
                }

        createView(container, false)
        disposableOfNewMsg = router.liveRoom.studyRoomVM.observableOfTutorAnswer
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    dismiss()
                }
        disposableOfNewMsgClose = router.getSubjectByKey<DisplayStudyQuestion>(EventKey.DisplayStudyQuestion)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    if (it == DisplayStudyQuestion.Question_Record_Open && container.tag != null) {
                        val view = container.tag as View
                        view.visibility = View.GONE
                    }
                }
    }

    private fun addTextView(container: ConstraintLayout, topTextView: TextView? = null, text: String?, isAddDiv: Boolean = false, showDot: Boolean? = false): TextView {
        val textView = TextView(context)
        textView.id = View.generateViewId()
        textView.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_main_text_color))
        val params = ConstraintLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        params.startToStart = ConstraintSet.PARENT_ID
        params.endToEnd = ConstraintSet.PARENT_ID
        if (topTextView == null) {
            params.topToTop = ConstraintSet.PARENT_ID
        } else {
            params.topToBottom = topTextView.id
        }
        val topPadding = 8.dp
        textView.setPadding(2 * topPadding, topPadding, 2 * topPadding, topPadding)
        textView.text = text
        val drawable = DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                .build()
        textView.background = StateListDrawableBuilder()
                .normal(DrawableBuilder().build())
                .pressed(drawable)
                .build()
        container.addView(textView, params)
        if (isAddDiv) {
            val div = TextView(context)
            div.setBackgroundColor(ContextCompat.getColor(context, R.color.base_divider_line_other))
            val layoutParams = ConstraintLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, 1.dp)
            layoutParams.topToBottom = textView.id
            container.addView(div, layoutParams)
        }
        if (showDot != null) {
            val dot = ImageView(context)
            dot.id = View.generateViewId()
            dot.setImageResource(R.drawable.shape_red_exclamation)
            val radius = 6.dp
            val layoutParams = ConstraintLayout.LayoutParams(radius, radius)
            layoutParams.topToTop = textView.id
            layoutParams.endToEnd = textView.id
            layoutParams.marginEnd = topPadding * 2 - radius / 2
            layoutParams.topMargin = topPadding - radius / 2
            container.tag = dot
            dot.visibility = if (showDot == true) View.VISIBLE else View.GONE
            container.addView(dot, layoutParams)
        }
        return textView
    }

    fun onDestroy() {
        RxUtils.dispose(disposableOfNewMsg)
        RxUtils.dispose(disposableOfNewMsgClose)
    }

    override fun initViewDataHashMap() {
        // do nothing
    }

    override fun show(anchor: View?) {
        showWithViewOfDirection(anchor, LEFT_TOP)
    }

    interface OnItemClickListener {
        fun onItemClick(position: Int)
    }
}