package com.baijiayun.groupclassui.window.studyreport;

import android.app.Dialog;
import android.os.Bundle;
import android.view.Gravity;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.screen.DisplayUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.base.BaseGroupDialogFragment;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livecore.LiveSDK;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.context.LiveRoom;
import com.baijiayun.liveuibase.zxing.ZxingUtils;

import org.jetbrains.annotations.NotNull;

/**
 * Created by wubanglin on 21/9/02.
 * description:
 */
public class StudyReportDialogFragment extends BaseGroupDialogFragment implements StudyReportDialogContract.View {
    private StudyReportDialogContract.Presenter presenter;
    private LiveRoom liveRoom;

    public static final int HEIGHT_MARGIN_LANDSCAPE = 12;
    public static final int WIDTH_DEFAULT = 540;

    @Override
    protected int getLayoutId() {
        return R.layout.bjy_fragment_study_report;
    }

    @Override
    protected void init(Bundle savedInstanceState, Bundle arguments) {
        $.id(R.id.bjy_group_close_study_report_text).clicked(v -> presenter.closeStudyReportDialog());
        $.id(R.id.bjy_group_close_study_report_image).clicked(v -> presenter.closeStudyReportDialog());
        initUrl();
    }

    @Override
    public void setPresenter(StudyReportDialogContract.Presenter presenter) {
        this.presenter = presenter;
        this.liveRoom = presenter.getLiveRoom();
    }

    @NonNull
    @NotNull
    @Override
    public Dialog onCreateDialog(@Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        Dialog dialog = new Dialog(getActivity(), R.style.BJYGroupDialogStyle);
        dialog.setCanceledOnTouchOutside(false);
        return dialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().getWindow().setBackgroundDrawable(ThemeDataUtil.getCommonWindowBg(getContext()));
    }

    @Override
    protected void setWindowParams(WindowManager.LayoutParams windowParams) {
        windowParams.windowAnimations = R.style.BJYGroupLiveBaseSendMsgDialogAnim;
        windowParams.gravity = Gravity.CENTER;

        int height = getContext() == null ? 0 : DisplayUtils.getScreenHeightPixels(getContext()) - DisplayUtils.getStatusBarHeight(getActivity());
        int width = DisplayUtils.getScreenWidthPixels(getContext());
        windowParams.width = UtilsKt.getDp(WIDTH_DEFAULT);
        windowParams.height = Math.min(height, width) - UtilsKt.getDp(HEIGHT_MARGIN_LANDSCAPE);
    }

    public void initUrl() {
        String params = "?room_id=" + liveRoom.getRoomInfo().roomId + "&user_number=" + liveRoom.getCurrentUser().getNumber();
        String host = LPConstants.HOSTS_WEB[LiveSDK.getDeployType().getType()];
        String url = host.concat("web/room/studyReport").concat(params);
        $.id(R.id.bjy_group_study_report_url).clicked(v -> presenter.openStudyReportBrowser(url)).text(url);
        $.id(R.id.bjy_group_copy_study_report_url).clicked(v -> presenter.copyStudyReportUrl(url));
        $.id(R.id.bjy_group_study_report_qrcode).setBitmap(ZxingUtils.generateQRCodeBitmap(url,
                150,
                150
        ));
    }
}
