package com.baijiayun.groupclassui.window.toolbar;

import android.content.Context;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.FrameLayout;

import com.baijiayun.bjyutils.widgets.SimpleTextWatcher;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;

import java.util.HashMap;


public class TextEditWindow extends ToolbarBaseWindow {

    private EditText editText;
    private String beforeText;
    private TextWatcher textWatcher;

    public TextEditWindow(Context context) {
        super(context);
        init();
    }

    @Override
    protected View onCreateView(Context context) {
        view = LayoutInflater.from(context).inflate(R.layout.bjy_group_window_text_edit, null);
        FrameLayout.LayoutParams layoutParams = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        layoutParams.gravity = Gravity.BOTTOM;
        view.setLayoutParams(layoutParams);
        return view;
    }

    private void init() {
        editText = $.id(R.id.window_message_send_et).view();
        view.addOnAttachStateChangeListener(new View.OnAttachStateChangeListener() {
            @Override
            public void onViewAttachedToWindow(View v) {
                $.id(R.id.window_message_send_et).view().setFocusable(true);
                $.id(R.id.window_message_send_et).view().setFocusableInTouchMode(true);
                $.id(R.id.window_message_send_et).view().requestFocus();
                v.postDelayed(() -> {
                    InputMethodManager imm = (InputMethodManager) context.getSystemService(Context.INPUT_METHOD_SERVICE);
                    if (imm == null) return;

                    imm.showSoftInput($.id(R.id.window_message_send_et).view(), 0);
                }, 100);
            }

            @Override
            public void onViewDetachedFromWindow(View v) {
                if (context == null) return;
                InputMethodManager imm = (InputMethodManager) context.getSystemService(Context.INPUT_METHOD_SERVICE);
                if (imm == null) return;
                if (imm.isActive())
                    imm.hideSoftInputFromWindow($.id(R.id.window_message_send_et).view().getWindowToken(), 0);
            }
        });

        textWatcher = new SimpleTextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                beforeText = s.toString();
            }

            @Override
            public void afterTextChanged(Editable s) {
                HashMap<String, String> map = new HashMap<>();
                map.put("beforeText", beforeText);
                map.put("afterText", editText.getEditableText().toString());
                iRouter.getSubjectByKey(EventKey.DrawTextSend).onNext(map);
            }
        };
        editText.addTextChangedListener(textWatcher);

        $.id(R.id.window_message_send_btn).clicked(v -> {
            HashMap<String, String> map = new HashMap<>();
            map.put("beforeText", beforeText);
            map.put("afterText", editText.getEditableText().toString());
            map.put("end", "true");
            iRouter.getSubjectByKey(EventKey.DrawTextSend).onNext(map);
            setEditText("");
            InputMethodManager imm = (InputMethodManager) context.getSystemService(Context.INPUT_METHOD_SERVICE);
            if (imm == null) return;
            if (imm.isActive())
                imm.hideSoftInputFromWindow($.id(R.id.window_message_send_et).view().getWindowToken(), 0);
        });
    }



    public void setEditText(String text){
        editText.removeTextChangedListener(textWatcher);
        editText.setText(text);
        //移动光标到末尾
        editText.setSelection(editText.getText().length());
        editText.addTextChangedListener(textWatcher);
    }
}
