package com.baijiayun.groupclassui.window.toolbox.smallblackboard

import android.annotation.SuppressLint
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.baijiayun.bjyutils.reactivex.RxUtils
import com.baijiayun.groupclassui.R
import com.baijiayun.groupclassui.global.IRouter
import com.baijiayun.groupclassui.window.toolbox.smallblackboard.SbbStudentAdapter.OnSelectListener
import com.baijiayun.livebase.models.LPUserModel
import com.baijiayun.livebase.models.imodels.IUserModel
import com.baijiayun.liveuibase.utils.LinearLayoutWrapManager
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable

/**
 * Created by pengwenjian on 2021/11/24
 * Describe:
 */
class SbbStudentSelectView(
    val iRouter: IRouter,
    val contentContainer: FrameLayout,
) {
    private var lastVisibleItem = 0
    private var isLoading = false
    private var disposableOfOnlineUser: Disposable? = null
    private lateinit var studentSelectAdapter: SbbStudentAdapter
    private lateinit var studentAdapter: SbbStudentAdapter
    var selectListener: SelectListener? = null
    @SuppressLint("ClickableViewAccessibility")
    fun createView() {
        contentContainer.removeAllViews()
        val context = contentContainer.context
        val studentSelectView = LayoutInflater.from(context)
            .inflate(R.layout.bjy_group_layout_sbb_student_select, contentContainer)
        val studentSelectCountTv =
            studentSelectView.findViewById<TextView>(R.id.sbb_student_select_count_tv)
        val studentCountTv = studentSelectView.findViewById<TextView>(R.id.sbb_student_count_tv)
        val studentRecyclerView: RecyclerView =
            studentSelectView.findViewById(R.id.sbb_student_recycler_view)
        val studentSelectRecyclerView: RecyclerView =
            studentSelectView.findViewById(R.id.sbb_student_select_recycler_view)
        val noStudentIv = studentSelectView.findViewById<ImageView>(R.id.sbb_no_student_iv)
        val noStudentTv = studentSelectView.findViewById<TextView>(R.id.sbb_no_student_tv)
        val studentList: List<LPUserModel> = iRouter.liveRoom.onlineUserVM.studentList
        studentSelectCountTv.text = context.getString(R.string.bjy_group_sbb_student_select_count, 0)
        studentCountTv.text = context.getString(R.string.bjy_group_sbb_student_count, studentList.size)
        studentRecyclerView.layoutManager = LinearLayoutWrapManager(context)
        studentSelectRecyclerView.layoutManager = LinearLayoutWrapManager(context)
        studentAdapter = SbbStudentAdapter(studentList.toMutableList(), false)
        studentSelectAdapter = SbbStudentAdapter(ArrayList(), true)
        studentRecyclerView.adapter = studentAdapter
        studentSelectRecyclerView.adapter = studentSelectAdapter
        studentAdapter.onSelectListener =
            object : OnSelectListener {
                override fun select(userModel: IUserModel, selected: Boolean) {
                    if (selected) {
                        studentSelectAdapter.notifyItemInserted(userModel)
                    } else {
                        studentSelectAdapter.notifyItemRemoved(userModel)
                    }
                    studentSelectCountTv.text = context.getString(
                        R.string.bjy_group_sbb_student_select_count,
                        studentSelectAdapter.itemCount
                    )
                    selectListener?.select(studentSelectAdapter.studentList)
                }

                override fun unSelect(users: List<IUserModel>) {
                    studentSelectAdapter.notifyItemRemoved(users)
                }
            }
        studentSelectAdapter.onSelectListener =
            object : OnSelectListener {
                override fun select(userModel: IUserModel, selected: Boolean) {
                    studentAdapter.notifyItemChanged(
                        userModel
                    )
                    studentSelectCountTv.text = context.getString(
                        R.string.bjy_group_sbb_student_select_count,
                        studentSelectAdapter.itemCount
                    )
                    selectListener?.select(studentSelectAdapter.studentList)
                }

                override fun unSelect(users: List<IUserModel>) {
                }
            }
        studentSelectView.findViewById<View>(R.id.sbb_student_clear_iv)
            .setOnClickListener {
                studentSelectAdapter.clear()
                studentAdapter.clear()
                selectListener?.select(studentSelectAdapter.studentList)
            }
        studentRecyclerView.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)
                if (dy > 0) {
                    val linearLayoutManager = recyclerView.layoutManager as LinearLayoutWrapManager
                    lastVisibleItem = linearLayoutManager.findLastVisibleItemPosition()
                    if (!isLoading && lastVisibleItem == studentAdapter.itemCount - 1) {
                        isLoading = true
                        iRouter.liveRoom.onlineUserVM.loadMoreUser()
                        studentAdapter.notifyDataSetChanged()
                    }
                }
            }
        })
        disposableOfOnlineUser = iRouter.liveRoom.onlineUserVM.observableOfOnlineUser
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe {
                isLoading = false
                studentAdapter.setData(iRouter.liveRoom.onlineUserVM.studentList.toMutableList())
                if (studentAdapter.studentList.isEmpty()) {
                    noStudentIv.visibility = View.VISIBLE
                    noStudentTv.visibility = View.VISIBLE
                } else {
                    noStudentIv.visibility = View.GONE
                    noStudentTv.visibility = View.GONE
                }
            }
        if (studentList.isEmpty()) {
            noStudentIv.visibility = View.VISIBLE
            noStudentTv.visibility = View.VISIBLE
        } else {
            noStudentIv.visibility = View.GONE
            noStudentTv.visibility = View.GONE
        }
    }

    fun setSelectList(selectList: List<IUserModel>) {
        studentSelectAdapter.setSelectList(selectList)
        studentAdapter.setSelectList(selectList)
    }

    fun getSelectUsers(): List<IUserModel> = studentSelectAdapter.studentList

    fun destroy() {
        RxUtils.dispose(disposableOfOnlineUser)
        contentContainer.removeAllViews()
    }
    interface SelectListener{
        fun select(users:List<IUserModel>)
    }
}