package com.baijiayun.groupclassui.window.toolbox.smallblackboard;

import static android.util.TypedValue.COMPLEX_UNIT_SP;
import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;
import static com.baijiayun.bjyutils.kt.UtilsKt.removeViewFromParent;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.os.Build;
import android.os.Environment;
import android.text.Editable;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AbsListView;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ExpandableListView;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;

import com.baijiahulian.common.cropperv2.uikit.CircleColorView;
import com.baijiahulian.common.permission.AppPermissions;
import com.baijiahulian.common.utils.StringUtils;
import com.baijiayun.bjyutils.TimeUtilsKt;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.log.LPLogger;
import com.baijiayun.bjyutils.query.QueryPlus;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.bjyutils.screen.DisplayUtils;
import com.baijiayun.bjyutils.toast.ToastCompat;
import com.baijiayun.bjyutils.widgets.SimpleTextWatcher;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.container.ShapePaint;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.model.SbbCloseWindowEvent;
import com.baijiayun.groupclassui.model.SbbStatusModel;
import com.baijiayun.groupclassui.window.BaseFixedRatioTitledWindow;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.models.LPUserModel;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livecore.context.LiveRoom;
import com.baijiayun.livecore.models.LPDocViewElementModel;
import com.baijiayun.livecore.models.LPGroupItem;
import com.baijiayun.livecore.models.LPShapeModel;
import com.baijiayun.livecore.models.LPSnippetPublishModel;
import com.baijiayun.livecore.models.LPSnippetPullReqModel;
import com.baijiayun.livecore.models.LPSnippetPullResModel;
import com.baijiayun.livecore.models.LPSnippetSubmitModel;
import com.baijiayun.livecore.models.LPWhiteboardViewInfo;
import com.baijiayun.livecore.models.roomresponse.LPResRoomShapeDelModel;
import com.baijiayun.livecore.models.roomresponse.LPResRoomShapeMultipleModel;
import com.baijiayun.livecore.models.roomresponse.LPResRoomShapeSingleModel;
import com.baijiayun.livecore.ppt.smallblackboard.IWhiteboardViewCallback;
import com.baijiayun.livecore.ppt.smallblackboard.SmallBlackboardView;
import com.baijiayun.livecore.ppt.util.LPShapeConverter;
import com.baijiayun.livecore.ppt.whiteboard.OnDoubleTapListener2;
import com.baijiayun.livecore.ppt.whiteboard.ShapeDispatcher;
import com.baijiayun.livecore.ppt.whiteboard.WhiteboardView;
import com.baijiayun.livecore.ppt.whiteboard.shape.DoodleShape;
import com.baijiayun.livecore.ppt.whiteboard.shape.LaserShape;
import com.baijiayun.livecore.ppt.whiteboard.shape.Shape;
import com.baijiayun.livecore.ppt.whiteboard.shape.TextShape;
import com.baijiayun.livecore.viewmodels.impl.ShapeVM;
import com.baijiayun.liveuibase.toolbox.browser.Tuple;
import com.baijiayun.liveuibase.utils.BaseUtilsKt;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;
import io.reactivex.disposables.Disposable;


/**
 * Created by yongjiaming on 2019/3/20
 * 小黑板
 */
public class SmallBlackboardWindow extends BaseFixedRatioTitledWindow implements IWhiteboardViewCallback, ShapePaint.IShapePaintListener, WhiteboardView.LPShapeSendListener {

    //最大自动显示到老师管理页人数
    public static final String SBB_ID_PREFIX = "s_0_";
    public static final String SBB_ID_INFIX = "_";
    private static final int MAX_AUTO_LOAD_COUNT = 20;
    public static final String FAKE_TEACHER_ID_PREFIX = "0_s_0";
    private static final String FAKE_TEACHER_NUMBER = "0";
    private static final String NUM_ZERO = "0";

    private SmallBlackboardView smallBlackboardView;
    private ShapePaint shapePaint;
    private ShapeDispatcher shapeDispatcher;
    private LiveRoom liveRoom;

    //老师底栏控件
    private ImageView uploadImgIv, deleteIv, prePageIV, nextPageIv;
    private LinearLayout pageContainer;
    private EditText inputCountDownTimeEt;
    private LinearLayout inputTimeLl;
    private Button reselectBtn, publishBtn, revokeBtn, takeBackBtn;//发布、撤销、收回
    private TextView takeBackCountingDownTv, pageTipTv;
    //展示区域顶栏
    private RelativeLayout contentTopRl;
    private TextView currentReviewPageTv, sbbOwnerTv;
    private View groupLayout;
    private TextView tvGroupName;
    private CircleColorView circleColorView;
    private View viewContainer;
    private TextView tvGroupNameSbb;
    private CircleColorView circleColorViewSbb;
    private ImageView publishSbbToAllIv, captureIv;
    //老师查看提交学生列表
    private View ssbReviewStudentFl;
    private CheckBox showNickNameCheckbox;
    private Button commitBtn; //提交&关闭

    private ExpandableListView commitExpandableListView;
    private SbbExpandableListViewAdapter adapter;
    private ShapeVM shapeVM;
    private String originWindowId,windowId, layer;
    private final CompositeDisposable disposables = new CompositeDisposable();
    private final List<SbbStatusModel> sbbAllList = new CopyOnWriteArrayList<>();
    //台上用户
    private final List<SbbStatusModel> sbbOnSeatUserList = new CopyOnWriteArrayList<>();
    //台下用户
    private final List<SbbStatusModel> sbbDownSeatUserList = new CopyOnWriteArrayList<>();
    //公开窗口的用户信息
    private final Set<IUserModel> sbbAllSubmitSet = new HashSet<>();
    //SmallBlackboardView viewPager 数据源
    private final List<String> layerList = new ArrayList<>();
    private int countDownLeftTime, duration;
    private long timeStart;
    private Disposable timerDisposable, previewUpdateCountingDown;
    private boolean isPublishToAll;
    private boolean isShowPaintOwnerEnable;

    public String cursor = "";
    private Context context;
    // 是否是全员小黑板
    private boolean toAll = true;
    private SbbStudentSelectView sbbStudentSelectView;
    private List<IUserModel> selectUsers = new ArrayList<>();
    private Disposable disposableOfPermission;

    public SmallBlackboardWindow(Context context, ShapePaint shapePaint, boolean isPublishToAll,
                                 boolean isShowPaintOwnerEnable) {
        super(context);
        this.shapePaint = shapePaint;
        shapePaint.registerShapePaintListener(this);
        this.isPublishToAll = isPublishToAll;
        this.isShowPaintOwnerEnable = isShowPaintOwnerEnable;
        this.context = context;

        getView().addOnAttachStateChangeListener(new View.OnAttachStateChangeListener() {
            @Override
            public void onViewAttachedToWindow(View v) {
                setWindowLayoutParams(v.getLayoutParams());
            }

            @Override
            public void onViewDetachedFromWindow(View v) {
                // do nothing
            }
        });
        attachLiveRoom();
        sbbOwnerTv = getView().findViewById(R.id.sbb_window_owner_tv);
        viewContainer = getView().findViewById(R.id.layout_group_container);
        tvGroupNameSbb = viewContainer.findViewById(R.id.tv_group_name);
        circleColorViewSbb = viewContainer.findViewById(R.id.view_group_color);

        if (BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) && !isPublishToAll) {
            View footer = LayoutInflater.from(context).inflate(R.layout.layout_small_blackboard_teacher_footer, rlFooterContainer);
            initFooterView(footer, true);
        } else {
            View footer = LayoutInflater.from(context).inflate(R.layout.layout_small_blackboard_student_footer, rlFooterContainer);
            initFooterView(footer, false);
        }

        if (iRouter.getLiveRoom().isAudition()) {
            deleteIv.setEnabled(false);
            if (commitBtn != null) {
                commitBtn.setEnabled(false);
            }
        }
    }


    @Override
    public boolean onRoomStatusChange(boolean isActive) {
        boolean isWindowInit = super.onRoomStatusChange(isActive);
        if(isWindowInit && isActive){
            startObservable();
        }
        return true;
    }

    private void attachLiveRoom() {
        this.liveRoom = iRouter.getLiveRoom();
        shapeDispatcher = new ShapeDispatcher();
        shapeDispatcher.setIsInSbbMode(true);
        shapeDispatcher.setShowPaintOwnerEnable(isShowPaintOwnerEnable);

        shapeVM = liveRoom.newShapeVM(shapeDispatcher);

        setOnWindowRepositionListener(new OnWindowRepositionListener() {
            @Override
            public void onWindowMove(int x, int y) {
                if (BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) && isPublishToAll) {
                    iRouter.getSubjectByKey(EventKey.SbbRepositionWindow).onNext(new Tuple("move", layer, x, y));
                }
            }

            @Override
            public void OnWindowScale(int newWidth, int newHeight) {
                if (BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) && isPublishToAll) {
                    iRouter.getSubjectByKey(EventKey.SbbRepositionWindow).onNext(new Tuple("scale", layer, newWidth, newHeight));
                }
            }
        });
    }


    private void startObservable() {
        Disposable participateDisposable = iRouter.getLiveRoom().getToolBoxVM().getObservableOfSnippetParticipate()
                .observeOn(AndroidSchedulers.mainThread())
                .filter(lpSnippetSubmitModel -> TextUtils.equals(lpSnippetSubmitModel.id,getId()))
                .filter(snippetSubmitModel -> BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) && !isPublishToAll && layerList.size() < MAX_AUTO_LOAD_COUNT)
                .subscribe(snippetSubmitModel -> {
                    sortSbbUser(new SbbStatusModel(snippetSubmitModel.from, SbbStatusModel.SbbStatus.NORMAL), false);
                });

        Disposable submitDisposable = iRouter.getLiveRoom().getToolBoxVM().getObservableOfSnippetSubmit()
                .observeOn(AndroidSchedulers.mainThread())
                .filter(lpSnippetSubmitModel -> TextUtils.equals(lpSnippetSubmitModel.id,getId()))
                .filter(snippetSubmitModel -> !isPublishToAll && !isCloseByTeacher && layerList.size() < MAX_AUTO_LOAD_COUNT)
                .subscribe(snippetSubmitModel -> {
                    if (BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser())) {
                        sortSbbUser(new SbbStatusModel(snippetSubmitModel.from, SbbStatusModel.SbbStatus.COMMIT), false);
                    } else {
                        if (snippetSubmitModel.from.getNumber().equals(iRouter.getLiveRoom().getCurrentUser().getNumber())) {
                            View tipRootView = SmallBlackboardDialogUtil.showTips(getView(), getView().getContext().getString(R.string.sbb_submit_close_tip, 3));
                            TextView textView = tipRootView.findViewById(R.id.lp_sbb_tip_content_tv);
                            textView.setVisibility(isMinInSync ? View.GONE : View.VISIBLE);
                            Disposable intervalDisposable = Observable.interval(1, TimeUnit.SECONDS)
                                    .take(3)
                                    .observeOn(AndroidSchedulers.mainThread())
                                    .subscribe(aLong -> {
                                        textView.setVisibility(isMinInSync ? View.GONE : View.VISIBLE);
                                        textView.setText(getView().getContext().getString(R.string.sbb_submit_close_tip, 2 - aLong));
                                        if (aLong == 2) {
                                            iRouter.getSubjectByKey(EventKey.SbbCloseWindow).onNext(new SbbCloseWindowEvent(getId(), layer,true));
                                        }
                                    });
                            disposables.add(intervalDisposable);
                        }
                    }
                });

        disposables.add(iRouter.getLiveRoom().getToolBoxVM().getObservableOfSnippetPullRes()
                .filter(lpSnippetPullResModel -> TextUtils.equals(lpSnippetPullResModel.id,getId()))
                .observeOn(AndroidSchedulers.mainThread())
                .filter(snippetPullResModel -> BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()))
                .subscribe(lpSnippetPullResModel -> {
                    cursor = SbbUtils.getLastUserNumber(lpSnippetPullResModel);
                    addSbbUser(lpSnippetPullResModel);
                    notifyList();
                    isLoadMore = false;
                    hasMore = lpSnippetPullResModel.hasMore;
                }));

        disposables.add(iRouter.getLiveRoom().getOnlineUserVM().getObservableOfOnGroupItem()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpGroupItem -> {
                    notifyList();
                }));


        disposables.add(iRouter.getLiveRoom().getOnlineUserVM().getPublishSubjectOfActiveUserAdd()
                .filter(lpResRoomUserInModel -> !isPublishToAll && BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) && !liveRoom.getCurrentUser().getNumber().equals(lpResRoomUserInModel.getUser().getNumber()))
                .subscribe(lpResRoomUserInModel -> {
                    sortSbbUser(new SbbStatusModel(lpResRoomUserInModel.getUser()), true);
                }));

        disposables.add(iRouter.getLiveRoom().getOnlineUserVM().getPublishSubjectOfActiveUserRemove()
                .filter(lpResRoomUserInModel -> !isPublishToAll && BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) && !liveRoom.getCurrentUser().getNumber().equals(lpResRoomUserInModel.getUser().getNumber()))
                .subscribe(lpResRoomUserInModel -> {
                    sortSbbUser(new SbbStatusModel(lpResRoomUserInModel.getUser()), true);
                }));

        Disposable publishDisposable = iRouter.getLiveRoom().getToolBoxVM().getObservableOfSnippetPublish()
                .filter(lpSnippetPublishModel -> TextUtils.equals(lpSnippetPublishModel.id,getId()))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(snippetPublishModel -> {
                    //公开窗口不切换状态
                    if (isPublishToAll) {
                        //再次发布需要关闭公开的小黑板
                        if (snippetPublishModel.operate == 1) {
                            iRouter.getSubjectByKey(EventKey.SbbCloseWindow).onNext(new SbbCloseWindowEvent(getId(), layer,true));
                        }
                        return;
                    }
                    LPConstants.LPUserType userType = iRouter.getLiveRoom().getCurrentUser().getType();
                    if (userType == LPConstants.LPUserType.Teacher) {
                        switch (snippetPublishModel.operate) {
                            case 0:
                                //收回
                                showTips(getString(R.string.sbb_all_take_back), 1);
                                changeSbbStatus(SmallBlackboardStatus.ALL_COMMITTED);
                                break;
                            case 1:
                                //开始，进入查看模式
                                changeSbbStatus(SmallBlackboardStatus.TEACHER_REVIEW);
                                break;
                            case 2:
                                //撤销，进入编辑模式
                                smallBlackboardView.goToPage(0);
                                changeSbbStatus(SmallBlackboardStatus.TEACHER_TO_PUBLISH);
                                break;
                        }
                    } else {
                        switch (snippetPublishModel.operate) {
                            case 0:
                                //老师主动收回
                                commitBtn.setEnabled(false);
                                isCloseByTeacher = true;
                                cancelCountDown();
                                requestSnippetSubmit();
                                showTips(getString(R.string.sbb_student_revoke_tip), 3);
                                if (iRouter != null) {
                                    if (subSbbWindowCount() <= 0) {
                                        iRouter.getSubjectByKey(EventKey.SbbShowToolBarWindow).onNext(-1);
                                        iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext(true);
                                    }
                                }
                                Disposable timerDisposable = Observable.timer(3, TimeUnit.SECONDS)
                                        .observeOn(AndroidSchedulers.mainThread())
                                        .subscribe(aLong -> {
                                            if (iRouter != null) {
                                                iRouter.getSubjectByKey(EventKey.SbbCloseWindow).onNext(new SbbCloseWindowEvent(getId(), layer, true));
                                            }
                                        });
                                disposables.add(timerDisposable);
                                break;
                            case 1:
                                break;
                            case 2:
                                cancelCountDown();
                                if (subSbbWindowCount() <= 0) {
                                    iRouter.getSubjectByKey(EventKey.SbbShowToolBarWindow).onNext(0);
                                    iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext(true);
                                }
                                iRouter.getSubjectByKey(EventKey.SbbCloseWindow).onNext(new SbbCloseWindowEvent(getId(), layer, true));
                                break;
                            default:
                                break;
                        }
                    }
                });

        // 老师端收到公开窗口关闭之后把公开窗口添加进来
        Disposable closeDisposable = iRouter.getSubjectByKey(EventKey.SbbCloseWindow)
                .observeOn(AndroidSchedulers.mainThread())
                .ofType(SbbCloseWindowEvent.class)
                .filter(closeWindowEvent -> BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) && !isPublishToAll)
                .subscribe(closeWindowEvent -> {
                    if (windowStatus == SmallBlackboardStatus.ALL_COMMITTED || windowStatus == SmallBlackboardStatus.EMPTY) {
                        String layerUserNumber = closeWindowEvent.layer;
                        IUserModel userModel = null;
                        for (IUserModel model : sbbAllSubmitSet) {
                            if (model.getNumber().equals(layerUserNumber)) {
                                userModel = model;
                                break;
                            }
                        }
                        if (userModel != null) {
                            if (FAKE_TEACHER_NUMBER.equals(layerUserNumber)) {
                                sbbAllList.add(0, new SbbStatusModel(userModel, SbbStatusModel.SbbStatus.NORMAL));
                                sortSbbUserInternal();
                            } else {
                                sortSbbUser(new SbbStatusModel(userModel, SbbStatusModel.SbbStatus.READ), true);
                            }
                        }
                        if (windowStatus == SmallBlackboardStatus.EMPTY) {
                            windowStatus = SmallBlackboardStatus.ALL_COMMITTED;
                            changeSbbStatus(windowStatus);
                        }
                        updateCurrentReviewTips();
                        smallBlackboardView.goToPage(currentPage);
                    }
                });

        Disposable classEndDisposable = iRouter.getLiveRoom().getObservableOfClassEnd()
                .filter(integer -> isAdmin())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(integer -> {
                    requestSnippetPublish(0);
                    cancelCountDown();
                });

        disposables.add(participateDisposable);
        disposables.add(submitDisposable);
        disposables.add(publishDisposable);
        disposables.add(closeDisposable);
        disposables.add(classEndDisposable);
    }

    private int subSbbWindowCount() {
        int count = iRouter.getValueByKey(EventKey.SbbWindowCount, Integer.class, 0);
        iRouter.setObjectByKey(EventKey.SbbWindowCount, Math.max(count - 1, 0));
        return Math.max(count - 1, 0);
    }

    private boolean isCloseByTeacher;

    // 老师的公开小黑板
    public void initStatus(LPSnippetPullResModel snippetPullResModel, SmallBlackboardStatus status) {
        layerList.clear();
        sbbAllList.clear();
        sbbAllSubmitSet.clear();

        if (BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) && !isPublishToAll) {
            LPUserModel userModel = (LPUserModel) iRouter.getLiveRoom().getCurrentUser();

            // 保存所有可被公开学员的信息（包括老师自己）
            // 老师自己的number是0,拼接成0_s_0_时间戳
            LPUserModel currentUserCopy = new LPUserModel();
            currentUserCopy.number = FAKE_TEACHER_NUMBER;
            currentUserCopy.name = userModel.name;
            currentUserCopy.userId = userModel.userId;

            //老师不在公布状态才添加到管理页
            if (!snippetPullResModel.isTeacherInPublishStatus) {
                //添加老师,老师的number在小黑板强制替换成0
                layerList.add(FAKE_TEACHER_NUMBER);
                //初始化侧边栏列表
                sbbAllList.add(new SbbStatusModel(currentUserCopy, SbbStatusModel.SbbStatus.NORMAL));
            }
            initExpandableListView();
            addSbbUser(snippetPullResModel);
        } else {
            layerList.add(layer);
        }

        if (!isPublishToAll) {
            if (snippetPullResModel.isActive) {
                duration = snippetPullResModel.duration;
                timeStart = snippetPullResModel.timeStart;
                startCountingDown(duration, timeStart);
            }
        }
        smallBlackboardView.initData(layerList);

        if (snippetPullResModel.isTeacherInPublishStatus && layerList.size() > 0) {
            layer = layerList.get(0);
        }
        shapeVM.requestPageAllShape(windowId, 0, layer, true);
        if (layerList.isEmpty() && BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser())) {
            status = SmallBlackboardStatus.EMPTY;
        }
        changeSbbStatus(status);
        //打开一次小黑板即开启画笔工具栏
        if (status != SmallBlackboardStatus.PUBLISH_TO_ALL) {
            iRouter.getSubjectByKey(EventKey.SbbShowToolBarWindow).onNext(1);
        }
    }

    /**
     * 老师打开新的小黑板
     */
    public void init() {
        layerList.clear();
        sbbAllList.clear();
        sbbAllSubmitSet.clear();
        layerList.add(FAKE_TEACHER_NUMBER);
        LPUserModel userModel = (LPUserModel) iRouter.getLiveRoom().getCurrentUser();
        LPUserModel currentUserCopy = new LPUserModel();
        currentUserCopy.number = FAKE_TEACHER_NUMBER;
        currentUserCopy.name = userModel.name;
        currentUserCopy.userId = userModel.userId;
        sbbAllList.add(new SbbStatusModel(currentUserCopy, SbbStatusModel.SbbStatus.NORMAL));
        smallBlackboardView.initData(layerList);
        initExpandableListView();
        changeSbbStatus(SmallBlackboardStatus.TEACHER_INIT);
    }

    @Override
    protected View onCreateView(Context context) {
        //最小宽高 360*244 dp
        DEFAULT_MIN_SIZE = 192;
        otherSizeRatio = 1.875f;
        return super.onCreateView(context);
    }

    @Override
    protected void onCreateContentView(Context context) {
        View contentView = LayoutInflater.from(context).inflate(R.layout.layout_sbb_window_content, rlContentContainer);
        if (smallBlackboardView == null) {
            smallBlackboardView = new SmallBlackboardView(context, this);
            smallBlackboardView.initViewpager(ThemeDataUtil.getResIdByAttributeId(context, R.attr.base_theme_blackboard_color));
            smallBlackboardView.setMaxPage(2);
            smallBlackboardView.setOnShapeSelectedListener(selected -> iRouter.getSubjectByKey(EventKey.Action2ShapeSelected).onNext(selected));
        }
        ssbReviewStudentFl = contentView.findViewById(R.id.sbb_window_commit_status_root_ll);
        FrameLayout ssbContainer = contentView.findViewById(R.id.sbb_window_content_container);
        removeViewFromParent(smallBlackboardView.getView());
        ssbContainer.addView(smallBlackboardView.getView(), new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
        initCommitStatusView(contentView);
    }

    @Override
    protected void onCreateFooterView(Context context) {
        // do nothing
    }

    @Override
    protected void onCreateTopView(Context context) {
        LayoutInflater.from(context).inflate(R.layout.layout_sbb_top_title, rlTitleContainer);
        takeBackCountingDownTv = rlTitleContainer.findViewById(R.id.sbb_counting_down_iv);
    }

    @Override
    protected void initTopTitleView() {
        ivClose = dragScaleRelativeLayout.findViewById(R.id.window_titled_close);
        ivMaxSync = dragScaleRelativeLayout.findViewById(R.id.window_titled_max_in_sync_layer);
        minIv = dragScaleRelativeLayout.findViewById(R.id.window_titled_min_in_sync_layer);

        ivMaxSync.setOnClickListener(l -> {
            isReceiveOrder = false;
            if(isMinInSync) {
                resetMinInSync();
            } else {
                if (isMaxInSync) {
                    resetInSync();
                } else {
                    maxInSync();
                }
            }
        });

        minIv.setOnClickListener(v -> {
            minInSync();
        });
    }

    private void initFooterView(View parent, boolean isTeacher) {
        if (isTeacher) {
            uploadImgIv = parent.findViewById(R.id.window_sbb_teacher_upload_image_iv);
            deleteIv = parent.findViewById(R.id.window_sbb_teacher_delete_iv);
            prePageIV = parent.findViewById(R.id.window_titled_prev_page);
            nextPageIv = parent.findViewById(R.id.window_titled_next_page);
            inputCountDownTimeEt = parent.findViewById(R.id.sbb_count_down_et);
            pageContainer = parent.findViewById(R.id.sbb_page_tip_ll);
            pageTipTv = parent.findViewById(R.id.window_center_pageNumber);
            reselectBtn = parent.findViewById(R.id.sbb_reselect_btn);
            publishBtn = parent.findViewById(R.id.sbb_publish_btn);
            revokeBtn = parent.findViewById(R.id.sbb_revoke_btn);
            takeBackBtn = parent.findViewById(R.id.sbb_take_back_btn);
            inputTimeLl = parent.findViewById(R.id.sbb_count_down_ll);
            publishSbbToAllIv = parent.findViewById(R.id.sbb_publish_iv);
            captureIv = parent.findViewById(R.id.sbb_capture_iv);
        } else {
            uploadImgIv = parent.findViewById(R.id.window_sbb_student_upload_image_iv);
            deleteIv = parent.findViewById(R.id.window_sbb_student_delete_iv);
            takeBackCountingDownTv = parent.findViewById(R.id.sbb_student_counting_down_tv);
            showNickNameCheckbox = parent.findViewById(R.id.sbb_show_nickname_cb);
            commitBtn = parent.findViewById(R.id.sbb_commit_btn);
        }
    }


    private boolean isLoadMore = false;
    private boolean hasMore = true;
    private long loadMoreTimeStamp = 0L;

    //初始化学生提交查看view
    private void initCommitStatusView(View parent) {
        //学生提交状态查看按钮
        FrameLayout commitReviewToggleView = parent.findViewById(R.id.sbb_window_commit_status_toggle_fl);
        commitExpandableListView = parent.findViewById(R.id.sbb_window_commit_status_rl);
        commitReviewToggleView.setOnClickListener(v -> {
            boolean visible = commitExpandableListView.getVisibility() == View.VISIBLE;
            commitExpandableListView.setVisibility(visible ? View.GONE : View.VISIBLE);
            $.id(R.id.sbb_window_commit_status_icon).image(visible ? R.drawable.ic_sbb_open : R.drawable.ic_sbb_close);
        });
        adapter = new SbbExpandableListViewAdapter();

        commitExpandableListView.setOnChildClickListener((parent1, v, groupPosition, childPosition, id) -> {
            SbbStatusModel statusModel = (SbbStatusModel) adapter.getChild(groupPosition, childPosition);
            if (statusModel.sbbStatus == SbbStatusModel.SbbStatus.COMMIT) {
                statusModel.sbbStatus = SbbStatusModel.SbbStatus.READ;
                notifyList();
            }
            int position;
            if (groupPosition == 1) {
                position = adapter.getActiveGroupCount() + childPosition + 1;
            } else {
                position = childPosition + 1;
            }
            smallBlackboardView.goToPage(position);
            return false;
        });

        commitExpandableListView.setOnGroupClickListener((parent12, v, groupPosition, id) -> {
            commitExpandableListView.setSelectedGroup(groupPosition);
            return false;
        });

        commitExpandableListView.setOnScrollListener(new AbsListView.OnScrollListener() {
            @Override
            public void onScrollStateChanged(AbsListView view, int scrollState) {
                if (!isLoadMore && scrollState == AbsListView.OnScrollListener.SCROLL_STATE_IDLE && view.getLastVisiblePosition() == view.getCount() - 1) {
                    loadMore();
                }
            }

            @Override
            public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount, int totalItemCount) {
                // do nothing
            }
        });

        contentTopRl = parent.findViewById(R.id.sbb_window_content_top_rl);
        currentReviewPageTv = parent.findViewById(R.id.sbb_current_page_owner_tv);
        groupLayout = parent.findViewById(R.id.rl_layout_group);
        tvGroupName = parent.findViewById(R.id.tv_group_name);
        circleColorView = parent.findViewById(R.id.view_group_color);
    }

    private SmallBlackboardStatus windowStatus = SmallBlackboardStatus.TEACHER_INIT;
    private boolean hasNotice = false;

    private void changeSbbStatus(SmallBlackboardStatus blackboardStatus) {
        windowStatus = blackboardStatus;
        switch (blackboardStatus) {
            case TEACHER_INIT:
                selectUsers.clear();
                rlFooterContainer.setVisibility(View.GONE);
                rlContentContainer.removeAllViews();
                sbbOwnerTv.setText(getString(R.string.small_blackboard));
                View initView = LayoutInflater.from(context).inflate(R.layout.bjy_group_layout_sbb_window_init, rlContentContainer);
                initView.findViewById(R.id.sbb_few_iv).setOnClickListener(v -> {
                    toAll = false;
                    changeSbbStatus(SmallBlackboardStatus.TEACHER_SELECT);
                    sbbOwnerTv.setText(getString(R.string.bjy_group_sbb_few));
                });
                initView.findViewById(R.id.sbb_all_iv).setOnClickListener(v -> {
                    toAll = true;
                    changeSbbStatus(SmallBlackboardStatus.TEACHER_TO_PUBLISH);
                    sbbOwnerTv.setText(getString(R.string.bjy_group_sbb_all));
                });
                ivClose.setOnClickListener(v -> {
                    iRouter.getSubjectByKey(EventKey.SbbCloseWindow).onNext(new SbbCloseWindowEvent(getId(),layer, isPublishToAll));
                    requestSnippetPublish(3);
                });
                break;
            case TEACHER_SELECT:
                if (!selectUsers.isEmpty()) {
                    // 离开教室的学生需要过滤掉
                    final List<LPUserModel> studentList = iRouter.getLiveRoom().getOnlineUserVM().getStudentList();
                    final Iterator<IUserModel> iterator = selectUsers.iterator();
                    while (iterator.hasNext()) {
                        final IUserModel userModel = iterator.next();
                        if (!studentList.contains(userModel)) {
                            iterator.remove();
                        }
                    }
                }
                // 清除sbbAllList、sbbOnSeatUserList保留老师自己
                if (!sbbAllList.isEmpty() && FAKE_TEACHER_NUMBER.equals(sbbAllList.get(0).user.getNumber())) {
                    final SbbStatusModel sbbStatusModel = sbbAllList.get(0);
                    sbbAllList.clear();
                    sbbAllList.add(sbbStatusModel);
                    sbbOnSeatUserList.clear();
                    sbbDownSeatUserList.clear();
                    sbbAllList.add(sbbStatusModel);
                }
                uploadImgIv.setVisibility(View.GONE);
                deleteIv.setVisibility(View.GONE);
                inputCountDownTimeEt.setVisibility(View.GONE);
                publishBtn.setVisibility(View.GONE);
                inputTimeLl.setVisibility(View.GONE);
                reselectBtn.setVisibility(View.GONE);
                publishSbbToAllIv.setVisibility(View.GONE);
                pageContainer.setVisibility(View.GONE);
                captureIv.setVisibility(View.GONE);

                revokeBtn.setVisibility(View.VISIBLE);
                takeBackBtn.setVisibility(View.VISIBLE);
                revokeBtn.setText(getString(R.string.bjy_group_sbb_previous_step));
                takeBackBtn.setText(getString(R.string.bjy_group_sbb_next_step));
                takeBackBtn.setEnabled(!selectUsers.isEmpty());
                revokeBtn.setOnClickListener(v -> {
                    sbbStudentSelectView.destroy();
                    changeSbbStatus(SmallBlackboardStatus.TEACHER_INIT);
                });
                takeBackBtn.setOnClickListener(v -> {
                    sbbStudentSelectView.destroy();
                    changeSbbStatus(SmallBlackboardStatus.TEACHER_TO_PUBLISH);
                    selectUsers = sbbStudentSelectView.getSelectUsers();
                });
                if (sbbStudentSelectView == null) {
                    sbbStudentSelectView = new SbbStudentSelectView(iRouter, rlContentContainer);
                }
                ivClose.setOnClickListener(v -> {
                    iRouter.getSubjectByKey(EventKey.SbbCloseWindow).onNext(new SbbCloseWindowEvent(getId(),layer, isPublishToAll));
                    requestSnippetPublish(3);
                });
                sbbStudentSelectView.setSelectListener(users -> takeBackBtn.setEnabled(!users.isEmpty()));
                sbbStudentSelectView.createView();
                sbbStudentSelectView.setSelectList(selectUsers);
                rlFooterContainer.setVisibility(View.VISIBLE);
                break;
            case TEACHER_TO_PUBLISH:
                rlContentContainer.removeAllViews();
                rlFooterContainer.setVisibility(View.VISIBLE);
                onCreateContentView(context);
                if (BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) && !isPublishToAll) {
                    initExpandableListView();
                }
                deleteIv.setVisibility(View.VISIBLE);
                publishSbbToAllIv.setVisibility(View.GONE);
                captureIv.setVisibility(View.GONE);
                inputCountDownTimeEt.setVisibility(View.VISIBLE);
                publishBtn.setVisibility(View.VISIBLE);
                inputTimeLl.setVisibility(View.VISIBLE);
                reselectBtn.setVisibility(View.VISIBLE);

                revokeBtn.setVisibility(View.GONE);
                takeBackBtn.setVisibility(View.GONE);
                pageContainer.setVisibility(View.GONE);
                takeBackCountingDownTv.setVisibility(View.GONE);
                ssbReviewStudentFl.setVisibility(View.GONE);
                contentTopRl.setVisibility(View.GONE);

                reselectBtn.setText(getString(toAll ? R.string.bjy_group_sbb_previous_step : R.string.bjy_group_sbb_reselect));

                shapeDispatcher.setTouchAble(true);

                ivClose.setOnClickListener(v -> {
                    iRouter.getSubjectByKey(EventKey.SbbCloseWindow).onNext(new SbbCloseWindowEvent(getId(), layer, isPublishToAll));
                    requestSnippetPublish(3);
                });

                reselectBtn.setOnClickListener(v -> {
                    changeSbbStatus(toAll ? SmallBlackboardStatus.TEACHER_INIT : SmallBlackboardStatus.TEACHER_SELECT);
                });

                deleteIv.setOnClickListener(v -> {
                    clearAllShapeWithDialog();
                });


                inputCountDownTimeEt.setOnFocusChangeListener((v, hasFocus) -> {
                    if (hasFocus && NUM_ZERO.equals(inputCountDownTimeEt.getText().toString()) && !hasNotice) {
                        showToastMessage(getString(R.string.sbb_input_count_down_tip));
                        hasNotice = true;
                    }
                });

                inputCountDownTimeEt.addTextChangedListener(new SimpleTextWatcher() {

                    @Override
                    public void afterTextChanged(Editable s) {
                        if (!TextUtils.isEmpty(s.toString()) && Integer.parseInt(s.toString()) > 150) {
                            inputCountDownTimeEt.setText("150");
                        }
                    }
                });

                Disposable publishDisposable = QueryPlus.with(rlFooterContainer).id(R.id.sbb_publish_btn)
                        .clicks()
                        .observeOn(AndroidSchedulers.mainThread())
                        .throttleFirst(1, TimeUnit.SECONDS)
                        .subscribe(o -> {
                            LPSnippetPublishModel snippetPublishModel = new LPSnippetPublishModel();
                            int duration = Integer.parseInt(inputCountDownTimeEt.getText().toString());
                            if (TextUtils.isEmpty(windowId)) {
                                windowId = SBB_ID_PREFIX + System.currentTimeMillis();
                            }
                            snippetPublishModel.id = windowId;
                            snippetPublishModel.toAll = toAll;
                            snippetPublishModel.duration = duration * 60;
                            snippetPublishModel.operate = 1;
                            snippetPublishModel.from = (LPUserModel) iRouter.getLiveRoom().getCurrentUser();
                            if (!toAll) {
                                List<LPSnippetPublishModel.User> users = new ArrayList<>();
                                for (IUserModel userModel : selectUsers) {
                                    users.add(new LPSnippetPublishModel.User(userModel.getUserId(), userModel.getNumber()));
                                }
                                snippetPublishModel.toUserList = users;
                            }
                            iRouter.getLiveRoom().getToolBoxVM().requestSnippetPublish(snippetPublishModel);
                            timeStart = System.currentTimeMillis() / 1000;
                            this.duration = duration * 60;
                            startCountingDown(this.duration, timeStart);
                            clearEditText();
                        });
                disposables.add(publishDisposable);
                break;
            case TEACHER_REVIEW:
                captureIv.setVisibility(View.VISIBLE);
                publishSbbToAllIv.setVisibility(View.GONE);
                uploadImgIv.setVisibility(View.GONE);

                deleteIv.setVisibility(View.GONE);
                inputCountDownTimeEt.setVisibility(View.GONE);
                publishBtn.setVisibility(View.GONE);
                inputTimeLl.setVisibility(View.GONE);
                reselectBtn.setVisibility(View.GONE);
                revokeBtn.setVisibility(View.VISIBLE);
                takeBackBtn.setVisibility(View.VISIBLE);
                ssbReviewStudentFl.setVisibility(View.VISIBLE);

                contentTopRl.setVisibility(View.VISIBLE);

                shapeDispatcher.setTouchAble(false);

                updateCurrentReviewTips();
                updateFooterPageTips(currentPage);

                takeBackBtn.setText(getView().getContext().getString(R.string.sbb_take_back));
                revokeBtn.setText(getView().getContext().getString(R.string.sbb_revoke));

                ivClose.setOnClickListener(v -> {
                    SmallBlackboardDialogUtil.showDialog(getView(), getString(R.string.sbb_close_all_window_in_publish_status),
                            v12 -> {
                                requestSnippetPublish(0);
                                iRouter.getSubjectByKey(EventKey.SbbCloseWindow).onNext(new SbbCloseWindowEvent(getId(), layer, true));
                                requestSnippetPublish(3);
                            });
                });

                captureIv.setOnClickListener(v -> {
                    capture();
                });

                revokeBtn.setOnClickListener(v -> {
                    SmallBlackboardDialogUtil.showDialog(view, getString(R.string.sbb_revoke_hint), o -> {
                        requestSnippetPublish(2);
                        cancelCountDown();
                        takeBackCountingDownTv.setVisibility(View.GONE);
                        eraseAllShape();
                    });
                });

                takeBackBtn.setOnClickListener(v -> {
                    SmallBlackboardDialogUtil.showDialog(getView(), getString(R.string.sbb_close_confirm), v1 -> {
                        requestSnippetPublish(0);
                        cancelCountDown();
                        takeBackCountingDownTv.setVisibility(View.GONE);
                        changeSbbStatus(SmallBlackboardStatus.ALL_COMMITTED);
                    });
                });

                prePageIV.setOnClickListener(v -> {
                    smallBlackboardView.prePage();
                });

                nextPageIv.setOnClickListener(v -> {
                    smallBlackboardView.nextPage();
                });
                if (!isRePublish) {
                    startCountingDown(duration, timeStart);
                    takeBackCountingDownTv.setVisibility(View.VISIBLE);
                } else {
                    takeBackCountingDownTv.setVisibility(View.GONE);
                }
                isRePublish = false;
                break;
            case ALL_COMMITTED:
                revokeBtn.setVisibility(View.VISIBLE);
                takeBackBtn.setVisibility(View.VISIBLE);
                contentTopRl.setVisibility(View.VISIBLE);
                ssbReviewStudentFl.setVisibility(View.VISIBLE);
                reselectBtn.setVisibility(View.GONE);
                updateCurrentReviewTips();
                updateFooterPageTips(currentPage);

                captureIv.setVisibility(View.VISIBLE);
                publishSbbToAllIv.setVisibility(View.VISIBLE);

                publishSbbToAllIv.setOnClickListener(v -> {
                    int position = -1;
                    for (int i = 0; i < layerList.size(); i++) {
                        if (layer.equals(layerList.get(i))) {
                            position = i;
                            break;
                        }
                    }

                    IUserModel userModel = null;
                    for (SbbStatusModel statusModel : sbbAllList) {
                        if (statusModel.user.getNumber().equals(layer)) {
                            userModel = statusModel.user;
                            sbbAllList.remove(statusModel);
                            sbbOnSeatUserList.remove(statusModel);
                            sbbDownSeatUserList.remove(statusModel);
                            adapter.notifyDataSetChanged();
                            break;
                        }
                    }
                    //移除当前分享的页面
                    layerList.remove(layer);
                    smallBlackboardView.initData(layerList);
                    if (position > 0) {
                        smallBlackboardView.prePage();
                    } else if (position == 0 && layerList.size() > 0) {
                        smallBlackboardView.goToPage(0);
                        layer = layerList.get(0);
                        updateCurrentReviewTips();
                        updateFooterPageTips(position + 1);
                    }

                    if (userModel != null) {
                        LPSnippetSubmitModel lpSnippetSubmitModel = new LPSnippetSubmitModel();
                        lpSnippetSubmitModel.id = windowId;
                        if (userModel instanceof LPUserModel) {
                            lpSnippetSubmitModel.from = (LPUserModel) userModel;
                        }
                        iRouter.getSubjectByKey(EventKey.SbbAddWindow).onNext(lpSnippetSubmitModel);
                        sbbAllSubmitSet.add(userModel);
                    }
                    if (layerList.size() == 0) {
                        changeSbbStatus(SmallBlackboardStatus.EMPTY);
                    }
                });
                takeBackBtn.setText(getView().getContext().getString(R.string.sbb_republish));
                revokeBtn.setText(getView().getContext().getString(R.string.sbb_edit_again));
                //再次发布
                takeBackBtn.setOnClickListener(v -> {
                    isRePublish = true;
                    LPSnippetPublishModel snippetPublishModel = new LPSnippetPublishModel();
                    if (TextUtils.isEmpty(windowId)) {
                        windowId = SBB_ID_PREFIX + System.currentTimeMillis();
                    }
                    snippetPublishModel.id = windowId;
                    snippetPublishModel.toAll = toAll;
                    snippetPublishModel.duration = 0;
                    snippetPublishModel.operate = 1;
                    snippetPublishModel.from = (LPUserModel) iRouter.getLiveRoom().getCurrentUser();
                    if (!toAll) {
                        List<LPSnippetPublishModel.User> users = new ArrayList<>();
                        for (IUserModel userModel : selectUsers) {
                            users.add(new LPSnippetPublishModel.User(userModel.getUserId(), userModel.getNumber()));
                        }
                        snippetPublishModel.toUserList = users;
                    }
                    iRouter.getLiveRoom().getToolBoxVM().requestSnippetPublish(snippetPublishModel);
                    clearEditText();
                });
                //重新编辑
                revokeBtn.setOnClickListener(v -> {
                    smallBlackboardView.goToPage(0);
                    changeSbbStatus(SmallBlackboardStatus.TEACHER_TO_PUBLISH);
                    eraseAllShape();
                });
                shapeDispatcher.setTouchAble(true);

                ivClose.setOnClickListener(v -> {
                    iRouter.getSubjectByKey(EventKey.SbbCloseWindow).onNext(new SbbCloseWindowEvent(getId(), layer, isPublishToAll));
                    requestSnippetPublish(3);
                });

                prePageIV.setOnClickListener(v -> {
                    smallBlackboardView.prePage();
                });

                nextPageIv.setOnClickListener(v -> {
                    smallBlackboardView.nextPage();
                });

                captureIv.setOnClickListener(v -> {
                    capture();
                });
                //小黑板收回后，默认状态修改为commit
                for (SbbStatusModel statusModel : sbbAllList) {
                    if (statusModel.sbbStatus != SbbStatusModel.SbbStatus.READ) {
                        statusModel.sbbStatus = SbbStatusModel.SbbStatus.COMMIT;
                    }
                }
                adapter.notifyDataSetChanged();
                break;
            case PUBLISH_TO_ALL:
                uploadImgIv.setVisibility(View.GONE);
                deleteIv.setVisibility(View.GONE);
                takeBackCountingDownTv.setVisibility(View.GONE);
                cancelCountDown();
                shapeDispatcher.setTouchAble(true);
                if (BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser())) {
                    View.OnClickListener onClickListener = v -> {
                        iRouter.getSubjectByKey(EventKey.SbbCloseWindow).onNext(new SbbCloseWindowEvent(getId(), layer, true));
                    };
                    ivClose.setOnClickListener(onClickListener);
                    commitBtn.setOnClickListener(onClickListener);
                    commitBtn.setText(getView().getContext().getString(R.string.sbb_close));
                    showNickNameCheckbox.setVisibility(View.VISIBLE);
                    showNickNameCheckbox.setOnClickListener(v -> {
                        iRouter.getSubjectByKey(EventKey.SbbShowName).onNext(this);
                    });
                    setOnSingleTapListener(event -> {
                        if (iRouter == null) {
                            return;
                        }
                        iRouter.getSubjectByKey(EventKey.SbbBringToFront).onNext(getId());
                    });
                } else {
                    ivClose.setVisibility(View.GONE);
                    ivMaxSync.setVisibility(View.GONE);
                    minIv.setVisibility(View.GONE);
                    rlFooterContainer.setVisibility(View.GONE);
                }
                break;
            case STUDENT_WRITING:
                ivClose.setVisibility(View.GONE);
                RelativeLayout.LayoutParams layoutParams = (RelativeLayout.LayoutParams) ivMaxSync.getLayoutParams();
                layoutParams.addRule(RelativeLayout.ALIGN_PARENT_RIGHT);


                deleteIv.setOnClickListener(v -> {
                    clearAllShapeWithDialog();
                });

                commitBtn.setOnClickListener(v -> {
                    commitBtn.setEnabled(false);
                    cancelCountDown();
                    requestSnippetSubmit();
                    if (subSbbWindowCount() <= 0) {
                        iRouter.getSubjectByKey(EventKey.SbbShowToolBarWindow).onNext(0);
                        iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext(true);
                    }
                });
                //允许窗口移动&小黑板画笔权限
                setAllowTouch(true);
                shapeDispatcher.setTouchAble(true);
                sbbOwnerTv.setText(getString(toAll ? R.string.bjy_group_sbb_all : R.string.bjy_group_sbb_few));
                break;
            case EMPTY:
                ssbReviewStudentFl.setVisibility(View.GONE);
                contentTopRl.setVisibility(View.GONE);
                takeBackBtn.setVisibility(View.GONE);
                revokeBtn.setVisibility(View.GONE);
                reselectBtn.setVisibility(View.GONE);
                break;
            default:
                break;
        }
    }

    private void capture() {
        if (context instanceof Activity) {
            RxUtils.dispose(disposableOfPermission);
            String[] permissions = new String[1];
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                permissions[0] = Manifest.permission.READ_MEDIA_IMAGES;
            } else {
                permissions[0] = Manifest.permission.WRITE_EXTERNAL_STORAGE;
            }
            disposableOfPermission = AppPermissions.newPermissions((Activity) context)
                    .request(permissions)
                    .subscribe(aBoolean -> {
                        if (aBoolean) {
                            DisplayUtils.saveViewCapture(getView(), Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES).getAbsolutePath());
                            showTips(getString(R.string.sbb_capture_tip), 3);
                        } else {
                            ToastCompat.showToast(context, context.getString(com.baijiayun.livebase.R.string.live_no_write_permission), Toast.LENGTH_SHORT);
                        }
                    });
        } else {
            DisplayUtils.saveViewCapture(getView(), Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES).getAbsolutePath());
            showTips(getString(R.string.sbb_capture_tip), 3);
        }
    }

    private void clearEditText() {
        final Object value = iRouter.getSubjectByKey(EventKey.DrawTextEnable).getValue();
        if (value instanceof Boolean && (Boolean) value) {
            iRouter.getSubjectByKey(EventKey.ClearEditText).onNext(true);
        }
    }

    private boolean isRePublish = false;

    private void startReviewPageUpdateCountingDown(String text) {
        currentReviewPageTv.setVisibility(View.VISIBLE);
        currentReviewPageTv.setText(text);

        RxUtils.dispose(previewUpdateCountingDown);

        previewUpdateCountingDown = Observable.interval(0, 1, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aLong -> {
                    if (aLong >= 3) {
                        currentReviewPageTv.setVisibility(View.INVISIBLE);
                        RxUtils.dispose(previewUpdateCountingDown);
                    }
                });
    }

    private void updateCurrentReviewTips() {
        LPUserModel currentUser = getUserByLayer(layer);
        if (currentUser != null) {
            startReviewPageUpdateCountingDown(currentUser.getName());
        } else {
            currentReviewPageTv.setVisibility(View.INVISIBLE);
        }
        if (currentUser != null && currentUser.getType() != LPConstants.LPUserType.Teacher && currentUser.getType() != LPConstants.LPUserType.Assistant) {
            final LPUserModel lpUserModel = convertGroupUser(currentUser.number);
            if (lpUserModel != null) {
                currentUser = lpUserModel;
            }
            if (currentUser.groupId != 0 && currentUser.groupItem != null) {
                groupLayout.setVisibility(View.VISIBLE);
                tvGroupName.setText(currentUser.groupItem.name);
                circleColorView.setColor(Color.parseColor(currentUser.groupItem.color));
            } else {
                groupLayout.setVisibility(View.GONE);
            }
        } else {
            groupLayout.setVisibility(View.GONE);
        }
    }

    private void showTips(String content,int delay) {
        if (isMinInSync) {
            showToastMessage(content);
        } else {
            SmallBlackboardDialogUtil.showTips(view, content, delay);
        }
    }

    private int currentPage = 1;

    private void updateFooterPageTips(int currentPos) {
        if (pageContainer == null || windowStatus == SmallBlackboardStatus.TEACHER_TO_PUBLISH
                || windowStatus == SmallBlackboardStatus.TEACHER_INIT || windowStatus == SmallBlackboardStatus.TEACHER_SELECT) {
            return;
        }
        currentPage = currentPos;
        if (smallBlackboardView.getItemCount() <= 1) {
            pageContainer.setVisibility(View.GONE);
        } else {
            pageContainer.setVisibility(View.VISIBLE);
            pageTipTv.setText(currentPos + "/" + smallBlackboardView.getItemCount());
        }
    }

    /**
     * @param duration  单位s
     * @param timeStart 单位s
     */
    private void startCountingDown(int duration, long timeStart) {
        RxUtils.dispose(timerDisposable);
        //没有时间限制
        if (duration == 0) {
            takeBackCountingDownTv.setVisibility(View.GONE);
        } else {
            long time = System.currentTimeMillis() / 1000 - timeStart;
            countDownLeftTime = (int) (duration - time);
            if (countDownLeftTime <= 0) {
                //submit
                onTimeToSubmit();
            } else {
                //显示倒计时
                timerDisposable = Observable.interval(0, 1, TimeUnit.SECONDS)
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe(aLong -> {
                            countDownLeftTime--;
                            if (countDownLeftTime == 0) {
                                takeBackCountingDownTv.setVisibility(View.GONE);
                                onTimeToSubmit();
                                timerDisposable.dispose();
                                return;
                            }
                            if (BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser())) {
                                takeBackCountingDownTv.setText(getView().getContext().getString(R.string.sbb_counting_down_sec_time, TimeUtilsKt.format24HoursTime(countDownLeftTime)));
                            } else {
                                takeBackCountingDownTv.setText(TimeUtilsKt.format24HoursTime(countDownLeftTime));
                            }
                        });
            }
            takeBackCountingDownTv.setVisibility(View.VISIBLE);
        }
    }

    private void cancelCountDown() {
        RxUtils.dispose(timerDisposable);
    }

    private void onTimeToSubmit() {
        isCloseByTeacher = true;
        //老师收卷
        if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher) {
            requestSnippetPublish(0);
        }
        //学生提交
        if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student) {
            requestSnippetSubmit();
        }
    }


    @Override
    public void instantiateWhiteboardView(WhiteboardView whiteboardView, String layer, int width, int height) {
        whiteboardView.setShapeSendListener(this);
        whiteboardView.setIdentity(layer + SBB_ID_INFIX + windowId, 0, 0);
        if (windowStatus == SmallBlackboardStatus.TEACHER_REVIEW) {
            whiteboardView.setTouchAble(false);
        }
        whiteboardView.setCustomShapeStrokeWidth(shapePaint.getCustomStrokeWidth());
        whiteboardView.setShapeStrokeWidth(shapePaint.getDoodleStrokeWidth());
        whiteboardView.setShapeColor(shapePaint.getShapeColor());
        whiteboardView.setPaintTextSize(shapePaint.getTextSize());
        whiteboardView.setCustomShapeType(shapePaint.getShapeType());
        whiteboardView.setPPTEditMode(shapePaint.getPPtEditMode());
        whiteboardView.setUserRole(liveRoom.getCurrentUser().getType());
        whiteboardView.setEnableEraseTeacherPaint(liveRoom.getPartnerConfig().enableEraseTeacherPaint);

        whiteboardView.setOnDoubleTapListener(new OnDoubleTapListener2() {
            @Override
            public void onDoubleTapConfirmed() {
                // do nothing
            }

            @Override
            public void onDoubleTapOnShape(Shape shape) {
                iRouter.getSubjectByKey(EventKey.EditTextShape).onNext(((TextShape)shape).getText());
            }
        });

        shapeDispatcher.addWhiteboard(whiteboardView);
        shapeDispatcher.setWhiteboardWidthAndHeight(width, height);
        shapeVM.requestPageAllShape(windowId, 0, layer, true);
    }

    @Override
    public void destroyWhiteboardView(WhiteboardView whiteboardView) {
        shapeDispatcher.removeWhiteboard(whiteboardView);
    }

    @Override
    public void onViewTap(View view, float x, float y) {
        if (iRouter != null) {
            iRouter.getSubjectByKey(EventKey.BgSingleTapConfirmed).onNext(true);
        }
    }

    @Override
    public void onPageChange(int position, String layer) {
        LPLogger.e("yjm", "shape onPageChange position=" + position + ", layer=" + layer);
        this.layer = layer;
        updateCurrentReviewTips();
        updateFooterPageTips(position + 1);
        if (position < sbbAllList.size() && sbbAllList.get(position).sbbStatus == SbbStatusModel.SbbStatus.COMMIT) {
            sbbAllList.get(position).sbbStatus = SbbStatusModel.SbbStatus.READ;
            adapter.notifyDataSetChanged();
        }
        shapeVM.requestPageAllShape(windowId, 0, layer, true);
    }

    @Override
    public void onShapeCustomStrokeWidthChange(float strokeWidth) {
        shapeDispatcher.setCustomShapeStrokeWidth(strokeWidth);
    }

    @Override
    public void onShapeDoodleStrokeWidthChange(float strokeWidth) {
        shapeDispatcher.setShapeStrokeWidth(strokeWidth);
    }

    @Override
    public void onShapeColorChange(int shapeColor) {
        shapeDispatcher.setPaintColor(shapeColor);
    }

    @Override
    public void onShapeTypeChange(LPConstants.ShapeType shapeType) {
        shapeDispatcher.setCustomShapeType(shapeType);
    }

    @Override
    public void onShapeEditModeChange(LPConstants.PPTEditMode pptEditMode) {
        shapeDispatcher.setPPTEditMode(pptEditMode);
    }

    @Override
    public void onShapeTextSizeChange(int textSize) {
        shapeDispatcher.setPaintTextSize(textSize);
    }

    @Override
    public void onSendDrawTextConfirmed(String beforeText, String content) {
        shapeDispatcher.sendDrawTextConfirmed(beforeText, content);
    }

    public void invalidateTextBoundary(){
        shapeDispatcher.invalidateCurrentPage();
    }


    @Override
    public void sendShape(Shape shape, LPWhiteboardViewInfo whiteboardViewInfo) {
        LPResRoomShapeSingleModel model = new LPResRoomShapeSingleModel();
        model.shape = LPShapeConverter.getModelFromShape(shape, whiteboardViewInfo.viewWidth, whiteboardViewInfo.viewHeight,
                whiteboardViewInfo.offsetWidth, whiteboardViewInfo.offsetHeight, shape instanceof DoodleShape ? 0 : 1);
        model.docId = windowId;
        model.page = 0;
        model.layer = layer;
        model.shape.dashType = whiteboardViewInfo.dashType;
        shapeVM.requestShapeAdd(model, whiteboardViewInfo.hasAppend, true);
    }

    @Override
    public boolean eraseShape(Shape shape, String docId, int page) {
        LPResRoomShapeDelModel lpResRoomShapeDelModel = new LPResRoomShapeDelModel();
        lpResRoomShapeDelModel.docId = windowId;
        lpResRoomShapeDelModel.page = page;
        lpResRoomShapeDelModel.shapeId = shape.id;
        lpResRoomShapeDelModel.layer = layer;
        shapeVM.eraseShape(lpResRoomShapeDelModel, true);
        return true;
    }

    @Override
    public void shapeUpdate(List<Shape> shapes, LPWhiteboardViewInfo whiteboardViewInfo) {
        if (shapes == null || shapes.size() == 0) {
            return;
        }
        LPResRoomShapeMultipleModel multipleModel = new LPResRoomShapeMultipleModel();
        multipleModel.docId = windowId;
        multipleModel.layer = layer;
        List<LPShapeModel> shapeModels = new ArrayList<>();
        for (Shape shape : shapes) {
            shapeModels.add(LPShapeConverter.getModelFromShape(shape, whiteboardViewInfo.viewWidth, whiteboardViewInfo.viewHeight,
                    whiteboardViewInfo.offsetWidth, whiteboardViewInfo.offsetHeight, 1));
        }
        multipleModel.shapeList = shapeModels;
        shapeVM.requestShapeUpdate(multipleModel, true);
    }

    @Override
    public void laserShapeUpdate(Shape shape, LPWhiteboardViewInfo whiteboardViewInfo) {
        if (shape == null){
            return;
        }
        if (shape.isHoverTrace && !liveRoom.getPartnerConfig().livePainterShowTraceBeforeDraw) {
            return;
        }
        LPResRoomShapeSingleModel singleModel = new LPResRoomShapeSingleModel();
        singleModel.docId = windowId;
        singleModel.layer = layer;
        singleModel.shape = LPShapeConverter.getModelFromShape(shape, whiteboardViewInfo.viewWidth, whiteboardViewInfo.viewHeight,
                whiteboardViewInfo.offsetWidth, whiteboardViewInfo.offsetHeight, 1);
        if (shape.isHoverTrace && liveRoom.getPartnerConfig().livePainterShowTraceBeforeDraw) {
            singleModel.shape.setShapeType(LPConstants.ShapeType.Doodle);
            singleModel.shape.strokeStyle = LPShapeConverter.getColorHex(shape.getPaint().getColor());
        }
        if (shape instanceof LaserShape) {
            shapeVM.requestShapeLaserUpdate(singleModel, true);
        }
    }

    @Override
    public void sendShapeAppend(Shape shape, LPWhiteboardViewInfo whiteboardViewInfo, int index) {
        LPResRoomShapeSingleModel model = new LPResRoomShapeSingleModel();
        model.docId = windowId;
        model.layer = layer;
        model.shape = LPShapeConverter.getModelFromShape(shape, whiteboardViewInfo.viewWidth, whiteboardViewInfo.viewHeight,
                whiteboardViewInfo.offsetWidth, whiteboardViewInfo.offsetHeight, 0);
        model.shape.index = index;
        model.shape.isEnd = whiteboardViewInfo.isEnd;
        shapeVM.requestShapeAppend(model, true);
    }

    public void setCurrentLayer(String windowId) {
        if (windowId.startsWith(SBB_ID_PREFIX)) {
            this.windowId = originWindowId = windowId;
        } else {
            final int index = windowId.indexOf(SBB_ID_PREFIX);
            if (index != -1) {
                this.windowId = windowId.substring(index);
            } else {
                this.windowId = windowId;
            }
            originWindowId = windowId;
        }
        // 判断是否是分享的小黑板，分享的小黑板是别人的number
        final int index = windowId.indexOf(SBB_ID_INFIX);
        if (index != -1) {
            layer = windowId.substring(0, index);
            if (!StringUtils.isNumeric(layer)) {
                layer = FAKE_TEACHER_NUMBER;
            }
        }
        // 不是分享的小黑板且不是老师则是自己的小黑板
        if (FAKE_TEACHER_NUMBER.equals(layer) && !BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) && windowId.startsWith(SBB_ID_PREFIX)) {
            layer = iRouter.getLiveRoom().getCurrentUser().getNumber();
        }
    }

    /**
     * layer = id = userNumber
     * 小黑板老师页对应的wb id 为s_0(大黑板id为0)
     * 公开老师页的layer为s_0，和老师的管理窗口（0）区分开
     */
    public void eraseShapes() {
        String deleteIds;
        if (liveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Teacher
                || liveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Assistant) {
            deleteIds = shapeDispatcher.eraseShapes(layer + SBB_ID_INFIX + windowId, 0);
        } else {
            if (liveRoom.getPartnerConfig().enableEraseTeacherPaint) {
                deleteIds = shapeDispatcher.eraseShapes(layer + SBB_ID_INFIX + windowId, 0);
            } else  {
                deleteIds = shapeDispatcher.eraseFilterShapes(layer + SBB_ID_INFIX + windowId, 0,
                        LPConstants.LPUserType.Teacher, LPConstants.LPUserType.Assistant);
            }
        }
        if (TextUtils.isEmpty(deleteIds)) {
            return;
        }
        LPResRoomShapeDelModel model = new LPResRoomShapeDelModel();
        model.docId = windowId;
        model.page = 0;
        model.shapeId = deleteIds;
        model.layer = layer;
        shapeVM.eraseShape(model, true);
    }

    /**
     * 清除除老师之外的所有画笔
     */
    public void eraseAllShape() {
        LPResRoomShapeDelModel model = new LPResRoomShapeDelModel();
        model.docId = windowId;
        model.page = 0;
        model.layer = "all";
        model.shapeId = "";
        shapeVM.eraseShape(model, true);
    }

    public void setPPTEditMode(LPConstants.PPTEditMode editMode){
        shapeDispatcher.setPPTEditMode(editMode);
    }

    private void requestSnippetPublish(int operate) {
        if (iRouter.getLiveRoom().isAudition()) {
            return;
        }
        LPSnippetPublishModel snippetPublishModel = new LPSnippetPublishModel();
        snippetPublishModel.from = (LPUserModel) iRouter.getLiveRoom().getCurrentUser();
        snippetPublishModel.operate = operate;
        if (TextUtils.isEmpty(windowId)) {
            windowId = SBB_ID_PREFIX + System.currentTimeMillis();
        }
        snippetPublishModel.id = windowId;
        snippetPublishModel.toAll = toAll;
        if (!toAll) {
            List<LPSnippetPublishModel.User> users = new ArrayList<>();
            for (IUserModel userModel : selectUsers) {
                users.add(new LPSnippetPublishModel.User(userModel.getUserId(), userModel.getNumber()));
            }
            snippetPublishModel.toUserList = users;
        }
        iRouter.getLiveRoom().getToolBoxVM().requestSnippetPublish(snippetPublishModel);
    }

    private void requestSnippetSubmit() {
        LPSnippetSubmitModel submitModel = new LPSnippetSubmitModel();
        submitModel.from = (LPUserModel) iRouter.getLiveRoom().getCurrentUser();
        submitModel.id = windowId;
        iRouter.getLiveRoom().getToolBoxVM().requestSnippetSubmit(submitModel);
    }

    @Override
    public String getId() {
        if (TextUtils.isEmpty(originWindowId)) {
            originWindowId = windowId;
        }
        return originWindowId;
    }

    public static String getUserNumber(@NonNull String windowId) {
        final int index = windowId.indexOf("_");
        if (index != -1) {
            return windowId.substring(0, index);
        } else {
            return "";
        }
    }

    @Override
    public void onDestroy() {
        smallBlackboardView = null;

        if (shapePaint != null) {
            shapePaint.unregisterShapePaintListener(this);
        }
        if (shapeDispatcher != null) {
            shapeDispatcher.onDestroy();
        }
        if (liveRoom != null) {
            liveRoom = null;
        }
        disposables.clear();

        cancelCountDown();
        RxUtils.dispose(previewUpdateCountingDown);
        RxUtils.dispose(disposableOfPermission);
        super.onDestroy();
    }

    private LPUserModel getUserByLayer(String layer) {
        for (SbbStatusModel sbbStatusModel : sbbAllList) {
            if (sbbStatusModel.user.getNumber().equals(layer)) {
                return (LPUserModel) sbbStatusModel.user;
            }
        }
        return null;
    }

    public void setToAll(boolean toAll) {
        this.toAll = toAll;
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof SmallBlackboardWindow)) {
            return false;
        }
        if (layer == null) {
            return false;
        }
        return layer.equals(((SmallBlackboardWindow) obj).layer);
    }


    /**
     * 显示当前窗口的用户名
     *
     * @param model
     */
    public void showWindowOwnerName(LPDocViewElementModel model) {
        if (TextUtils.isEmpty(model.name)) {
            sbbOwnerTv.setText(getString(R.string.small_blackboard));
            viewContainer.setVisibility(View.GONE);
        } else {
            String userNum = model.docId.substring(0, model.docId.indexOf(SBB_ID_INFIX));
            final LPUserModel lpUserModel = convertGroupUser(userNum);
            if (lpUserModel != null && lpUserModel.groupId != 0 && lpUserModel.groupItem != null) {
                viewContainer.setVisibility(View.VISIBLE);
                tvGroupNameSbb.setText(lpUserModel.groupItem.name);
                circleColorViewSbb.setColor(Color.parseColor(lpUserModel.groupItem.color));
            } else {
                viewContainer.setVisibility(View.GONE);
            }
            sbbOwnerTv.setText(getView().getContext().getString(R.string.sbb_owner_name, getEncodePhoneNumber(model.name)));
        }
    }

    public boolean shouldShowOwnerName() {
        if (!isPublishToAll || showNickNameCheckbox == null) {
            return false;
        }
        return showNickNameCheckbox.isChecked();
    }

    private void clearAllShapeWithDialog() {
        SmallBlackboardDialogUtil.showDialog(getView(), getString(R.string.sbb_clear_all_shape_tip), v -> {
            LPResRoomShapeDelModel delModel = new LPResRoomShapeDelModel();
            delModel.docId = windowId;
            delModel.layer = layer;
            shapeVM.eraseAllShape(delModel, true);
        });
    }

    private void addSbbUser(LPSnippetPullResModel snippetPullResModel) {
        if (windowStatus == SmallBlackboardStatus.ALL_COMMITTED) {
            //小黑板已收回状态下，loadMore学员的状态都初始为commit
            if (snippetPullResModel.userParticipatedList != null) {
                List<SbbStatusModel> statusModels = new ArrayList<>();
                for (IUserModel participateUser : snippetPullResModel.userParticipatedList) {
                    SbbStatusModel model = new SbbStatusModel(participateUser, SbbStatusModel.SbbStatus.COMMIT);
                    statusModels.add(model);
                }
                sortSbbUser(statusModels);
            }
        } else {
            if (snippetPullResModel.userParticipatedList != null) {
                List<SbbStatusModel> statusModels = new ArrayList<>();
                for (IUserModel participateUser : snippetPullResModel.userParticipatedList) {
                    SbbStatusModel model = new SbbStatusModel(participateUser, SbbStatusModel.SbbStatus.NORMAL);
                    statusModels.add(model);
                }
                sortSbbUser(statusModels);
            }

            if (snippetPullResModel.userSubmittedList != null) {
                List<SbbStatusModel> statusModels = new ArrayList<>();
                for (IUserModel submitUser : snippetPullResModel.userSubmittedList) {
                    SbbStatusModel model = new SbbStatusModel(submitUser, SbbStatusModel.SbbStatus.COMMIT);
                    statusModels.add(model);
                }
                sortSbbUser(statusModels);
            }
        }
    }

    /**
     * 按先台上后台下排序，更新数据集
     *
     * @param model
     * @param remainStatus 保留status
     */
    private void sortSbbUser(SbbStatusModel model, boolean remainStatus) {
        SbbStatusModel removedModel = null;
        int index1 = sbbOnSeatUserList.indexOf(model);
        int index2 = sbbDownSeatUserList.indexOf(model);
        if (index1 >= 0) {
            removedModel = sbbOnSeatUserList.get(index1);
            sbbOnSeatUserList.remove(model);
        }
        if (index2 >= 0) {
            removedModel = sbbDownSeatUserList.get(index2);
            sbbDownSeatUserList.remove(index2);
        }
        if (remainStatus && removedModel != null) {
            model.sbbStatus = removedModel.sbbStatus;
        }
        if (isActiveUser(model.user)) {
            if (!sbbOnSeatUserList.contains(model)) {
                sbbOnSeatUserList.add(model);
            }
        } else {
            if (!sbbDownSeatUserList.contains(model)) {
                sbbDownSeatUserList.add(model);
            }
        }
        sortSbbUserInternal();
    }
    //用number判断，把LPUserModel里面的hashCode是用userId判断的
    //如果改为用number判断，user_out信令没带number值此法不可行
    private boolean isActiveUser(IUserModel userModel) {
        final List<IUserModel> activeUserList = iRouter.getLiveRoom().getOnlineUserVM().getActiveUserList();
        for (IUserModel user : activeUserList) {
            if (TextUtils.equals(user.getNumber(), userModel.getNumber())) {
                return true;
            }
        }
        return false;
    }

    private void sortSbbUser(List<SbbStatusModel> sbbStatusModelList) {
        for (SbbStatusModel model : sbbStatusModelList) {
            if (sbbAllList.contains(model)) {
                continue;
            }
            if (isActiveUser(model.user)) {
                if (!sbbOnSeatUserList.contains(model)) {
                    sbbOnSeatUserList.add(model);
                }
            } else {
                if (!sbbDownSeatUserList.contains(model)) {
                    sbbDownSeatUserList.add(model);
                }
            }
        }
        sortSbbUserInternal();
    }

    private void sortSbbUserInternal() {
        SbbStatusModel sbbTeacher = null;
        if (!sbbAllList.isEmpty() && FAKE_TEACHER_NUMBER.equals(sbbAllList.get(0).user.getNumber())) {
            sbbTeacher = sbbAllList.get(0);
        }
        sbbAllList.clear();
        if (sbbTeacher != null) {
            sbbAllList.add(sbbTeacher);
        }
        sbbAllList.addAll(sbbOnSeatUserList);
        sbbAllList.addAll(sbbDownSeatUserList);
        notifyList();

        layerList.clear();
        if (sbbTeacher != null) {
            layerList.add(FAKE_TEACHER_NUMBER);
        }
        for (SbbStatusModel statusModel : sbbOnSeatUserList) {
            layerList.add(statusModel.user.getNumber());
        }
        for (SbbStatusModel statusModel : sbbDownSeatUserList) {
            layerList.add(statusModel.user.getNumber());
        }
        smallBlackboardView.initData(layerList);
        updateFooterPageTips(currentPage);
    }

    /**
     * 加载更多作答学生
     */
    private void loadMore() {
        if (BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) && (hasMore || System.currentTimeMillis() - loadMoreTimeStamp > 5000)) {
            LPSnippetPullReqModel pullReqModel = new LPSnippetPullReqModel(windowId);
            pullReqModel.count = LPConstants.SNIPPET_COUNT_PER_PAGE;
            pullReqModel.cursor = cursor;
            List<IUserModel> activeUserList = iRouter.getLiveRoom().getOnlineUserVM().getActiveUserList();
            List<String> activeUserNumbers = new ArrayList<>();
            for (int i = 0; i < activeUserList.size(); i++) {
                if (iRouter.getLiveRoom().getCurrentUser().equals(activeUserList.get(i))) {
                    continue;
                }
                activeUserNumbers.add(activeUserList.get(i).getNumber());
            }
            pullReqModel.activedUser = activeUserNumbers.toArray(new String[0]);
            iRouter.getLiveRoom().getToolBoxVM().requestSnippetPullReq(pullReqModel);
            isLoadMore = true;
            loadMoreTimeStamp = System.currentTimeMillis();
        }
    }

    private LPUserModel convertGroupUser(String userNum) {
        final List<LPGroupItem> groupList = iRouter.getLiveRoom().getOnlineUserVM().getGroupList();
        for (LPGroupItem groupItem : groupList) {
            final List<LPUserModel> userModelList = groupItem.userModelList;
            for (LPUserModel lpUserModel : userModelList) {
                if (userNum.equals(lpUserModel.getNumber())) {
                    lpUserModel.groupItem = groupItem.groupItemModel;
                    return lpUserModel;
                }
            }
        }
        return null;
    }

    private void convertGroupUser(List<SbbStatusModel> sbbStatusModels) {
        final List<LPGroupItem> groupList = iRouter.getLiveRoom().getOnlineUserVM().getGroupList();
        List<LPUserModel> userModels = new ArrayList<>();
        for (LPGroupItem groupItem : groupList) {
            final List<LPUserModel> userModelList = groupItem.userModelList;
            for (LPUserModel lpUserModel : userModelList) {
                lpUserModel.groupItem = groupItem.groupItemModel;
                userModels.add(lpUserModel);
            }
        }
        for (SbbStatusModel statusModel : sbbStatusModels) {
            for (LPUserModel userModel : userModels) {
                if (statusModel.user.getNumber().equals(userModel.getNumber())) {
                    LPUserModel lpUserModel = (LPUserModel) statusModel.user;
                    lpUserModel.groupId = userModel.groupId;
                    lpUserModel.groupItem = userModel.groupItem;
                }
            }
        }
    }

    public void notifyList() {
        convertGroupUser(sbbOnSeatUserList);
        convertGroupUser(sbbDownSeatUserList);
        adapter.notifyDataSetChanged();
    }

    private void initExpandableListView() {
        convertGroupUser(sbbOnSeatUserList);
        convertGroupUser(sbbDownSeatUserList);
        adapter.setData(sbbOnSeatUserList, sbbDownSeatUserList);
        LinearLayout linearLayout = new LinearLayout(context);
        linearLayout.setOrientation(LinearLayout.HORIZONTAL);
        TextView textView = new TextView(context);
        textView.setTextSize(COMPLEX_UNIT_SP, 16);
        textView.setText(sbbAllList.get(0).user.getName());
        textView.setGravity(Gravity.START);
        textView.setMaxLines(1);
        textView.setEllipsize(TextUtils.TruncateAt.END);
        int _4dp = UtilsKt.getDp(4);
        textView.setPadding(_4dp, _4dp, _4dp, _4dp);
        textView.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_main_text_color));
        linearLayout.addView(textView, new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
        linearLayout.setOnClickListener(v -> {
            smallBlackboardView.goToPage(0);
        });
        commitExpandableListView.addHeaderView(linearLayout);
        commitExpandableListView.setAdapter(adapter);
        commitExpandableListView.expandGroup(0);
    }

    @Override
    public void maxInFull() {
        super.maxInFull();
        invalidateCurrentPage();
    }

    @Override
    public void resetInFull() {
        super.resetInFull();
        invalidateCurrentPage();
    }

    @Override
    public void maxInSync() {
        super.maxInSync();
        invalidateCurrentPage();
    }

    @Override
    public void resetInSync() {
        super.resetInSync();
        invalidateCurrentPage();
    }

    private void invalidateCurrentPage(){
        smallBlackboardView.getView().post(() -> {
            shapeDispatcher.setWhiteboardWidthAndHeight(smallBlackboardView.getView().getMeasuredWidth(), smallBlackboardView.getView().getMeasuredHeight());
            shapeVM.requestPageAllShape(windowId, 0, layer, true);
        });
    }

    // 学生点击多个小黑板也切换层级
    @Override
    protected boolean canBringToForeground() {
        return true;
    }

    public void minSbbOwner() {
        if (sbbOwnerTv.getText().toString().equals(getString(R.string.bjy_group_sbb_multi))) {
            return;
        }
        final List<SmallBlackboardWindow> smallBlackboardWindows = iRouter.getListByKey(EventKey.MinimizeWindows, SmallBlackboardWindow.class);
        if (smallBlackboardWindows.size() > 1) {
            // 多个最小化显示"多个小黑板"
            sbbOwnerTv.setTag(sbbOwnerTv.getText().toString());
            sbbOwnerTv.setText(getString(R.string.bjy_group_sbb_multi));
        }
    }

    public void resetMinSbbOwner() {
        final Object tag = sbbOwnerTv.getTag();
        // 判断值是否"多个小黑板"，分享之后的小黑板昵称可能变化
        if (tag instanceof String && sbbOwnerTv.getText().toString().equals(getString(R.string.bjy_group_sbb_multi))) {
            sbbOwnerTv.setText((String)tag);
        }
    }

    @Override
    public void minInSync() {
        final List<SmallBlackboardWindow> smallBlackboardWindows = iRouter.getListByKey(EventKey.MinimizeWindows, SmallBlackboardWindow.class);
        smallBlackboardWindows.add(this);
        iRouter.setObjectByKey(EventKey.MinimizeWindows, smallBlackboardWindows);
        if (smallBlackboardWindows.size() > 1) {
            // 多个最小化显示"多个小黑板"
            sbbOwnerTv.setTag(sbbOwnerTv.getText().toString());
            sbbOwnerTv.setText(getString(R.string.bjy_group_sbb_multi));
            smallBlackboardWindows.get(0).minSbbOwner();
        }
        super.minInSync();
    }

    @Override
    public void resetMinInSync() {
        super.resetMinInSync();
        if (windowStatus == SmallBlackboardStatus.STUDENT_WRITING) {
            ivClose.setVisibility(View.GONE);
        }
    }

    public void resetMinInSyncWithoutEvent() {
        if (isMinInSync) {
            contentOuterContainer.setVisibility(View.VISIBLE);
            rlFooterContainer.setVisibility(View.VISIBLE);
            minIv.setVisibility(View.VISIBLE);
            ivScale.setVisibility(View.VISIBLE);
            ivClose.setVisibility(View.VISIBLE);
        }
        ivMaxSync.setImageDrawable(maxDrawable);
        if (isMaxInSync) {
            maxInSync();
            ivMaxSync.setImageDrawable(maxResetDrawable);
        } else if (isMaxInFull) {
            maxInFull();
            ivMaxSync.setImageDrawable(fullResetDrawable);
        }
        isMinInSync = false;
        if (windowStatus == SmallBlackboardStatus.STUDENT_WRITING) {
            ivClose.setVisibility(View.GONE);
        }
    }
}
