package com.baijiayun.groupclassui.window.toolbox.turntable

import android.content.Context
import android.graphics.drawable.GradientDrawable
import android.view.View
import android.widget.LinearLayout
import androidx.core.content.ContextCompat
import com.baijiayun.bjyutils.drawable.DrawableBuilder
import com.baijiayun.groupclassui.R
import com.baijiayun.livebase.utils.ThemeDataUtil
import com.baijiayun.liveuibase.base.BaseWindow
import com.baijiayun.liveuibase.widgets.TurntableView

class TurntableWindow(context: Context) : BaseWindow(context) {
    private var turntable: TurntableView? = null
    private var closeBtn: LinearLayout? = null
    private var onClose: (() -> Unit)? = null

    override fun onCreateView(context: Context?): View {
        val view = View.inflate(context, R.layout.bjy_group_layout_turntable_window, null)
        turntable = view.findViewById(R.id.bjy_group_turntable_view)
        closeBtn = view.findViewById(R.id.bjy_group_turntable_close)
        turntable?.apply {
            setBackgroundColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_bg_color))
            setSectorColor(ThemeDataUtil.getColorFromThemeConfigByAttrIdWithAlpha(context, R.attr.base_theme_bottom_menu_text_color, 0.2f))
            setCenterBottomColor(ThemeDataUtil.getColorFromThemeConfigByAttrIdWithAlpha(context, R.attr.base_theme_bottom_menu_text_color, 0.2f))
        }
        closeBtn?.background = DrawableBuilder()
            .shape(GradientDrawable.OVAL)
            .size(40, 40)
            .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_bg_color))
            .strokeWidth(1)
            .strokeColor(ContextCompat.getColor(context!!, R.color.base_main_color_layer_10))
            .build()
        closeBtn?.setOnClickListener {
            onClose?.invoke()
        }
        allowTouch = true
        return view
    }

    /**
     * 老师转动
     * @return 选中的值
     */
    fun teacherTurn(): Int {
        val select = (Math.random() * 100).toInt() % 6
        if (turntable?.isTurning() != true) {
            turntable?.startTurn((select + 1).toString(), 5)
        }
        return select + 1
    }

    /**
     * 学生转动
     */
    fun studentTurn(select: String) {
        if (turntable?.isTurning() != true) {
            turntable?.startTurn(select, 5)
        }
    }

    /**
     * 设置转盘课主动触发
     */
    fun setClickable(isClickable: Boolean) {
        turntable?.isClickable = isClickable
    }

    /**
     * 设置是否可主动关闭转盘
     */
    fun setCloseEnable(isEnable: Boolean) {
        closeBtn?.visibility = if (isEnable) View.VISIBLE else View.GONE
    }

    /**
     * 设置关闭事件
     */
    fun setCloseListener(close: () -> Unit) {
        onClose = close
    }

    /**
     * 设置点击事件
     */
    fun setOnClickListener(listener: View.OnClickListener) {
        turntable?.setOnClickListener(listener)
    }

    override fun onDestroy() {
        super.onDestroy()
        turntable?.release()
    }
}