package com.baijiayun.groupclassui.window.video;

import android.annotation.SuppressLint;
import android.content.Context;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.core.content.ContextCompat;
import androidx.fragment.app.FragmentActivity;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.log.LPLogger;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventCode;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.util.KUtilsKt;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.widgets.dialog.NoteDialog;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.listener.OnCountDownListener;
import com.baijiayun.livecore.models.LPStudyRoomTutorModel;
import com.baijiayun.livecore.models.LPStudyUserStatus;
import com.baijiayun.livecore.models.imodels.IMediaModel;

import java.util.List;
import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.ObservableSource;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Function;
import io.reactivex.subjects.BehaviorSubject;


/**
 * 自习室视频item
 */
public class StudyVideoWindow extends VideoWindow {
    private boolean isLeave = false;
    private long selfStudyDuration = 0;
    private Disposable disposableOfStudyRoomSwitch;
    private Disposable disposableOfPublishDefaultStream;
    private Disposable disposableOfStudyRank;
    private Disposable disposableOfActiveUserStatus;
    private Disposable disposableOfTutorApplyReq, disposableOfTutorApplyRes,
            disposableOfTutorStart, disposableOfTutorEnd, disposableOfTutorGroup, disposableOfForHelp;
    private Disposable disposableOfUserAdd, disposableOfUserRemove;
    private Disposable disposableOfDurationTimer;
    private TextView tvTutorAgree, tvTutorDisagree;
    private ImageView ivTutorHelp;
    //是否在求助中
    private boolean isWaitingHelp = false;
    //是否在辅导中、只更新状态,窗口的状态，注意区分自己的状态,
    //标记该窗口是否在辅导中，不区分同组还是不同组
    private boolean isTutorHelping = false;
    //标记自己是否进入了辅导
    private boolean isSelfTutor = false;
    //单人辅导对象
    private LPStudyRoomTutorModel lpStudyRoomTutorModel;
    // 求助等待时间
    private static final int TUTOR_HELP_WAITING_DURATION = 30;

    protected StudyVideoWindow(Context context) {
        super(context);
        //自习室、单人辅导
        if (enableSingleTutor()) {
            initSingleTutorViews();
        }
    }

    private void initSingleTutorViews() {
        tvTutorAgree = $.id(R.id.window_video_tutor_agree).view();
        tvTutorDisagree = $.id(R.id.window_video_tutor_disagree).view();
        ivTutorHelp = $.id(R.id.window_video_tutor_help_him).view();
        ivTutorHelp.setOnClickListener(v -> {
            showTutorHandleIcon(true);
            showTutorHelpIcon(false);
        });
        tvTutorAgree.setOnClickListener(v -> {
            showTutorHandleIcon(false);
            iRouter.getLiveRoom().getStudyRoomVM().agreeTutorApply(getUserId());
        });
        tvTutorDisagree.setOnClickListener(v -> {
            showTutorHandleIcon(false);
            iRouter.getLiveRoom().getStudyRoomVM().disagreeTutorApply(getUserId());
        });
        $.id(R.id.window_video_tutor_help).clicked(v -> {
            //求助或者取消
            if (isWaitingHelp) {
                endForHelp();
            } else {
                startForHelp();
            }
        });
        $.id(R.id.window_video_tutor_help_waiting).background(new DrawableBuilder()
                .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .solidColor(0x66000000)
                .build());
        $.id(R.id.cl_top_left_tutor).background(new DrawableBuilder()
                .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .solidColor(0xFFFF1F49)
                .build());
        $.id(R.id.window_tutor_close).clicked(v -> endTutorHelp());
    }

    /**
     * 单人辅导在自习和讨论模式下才有
     *
     * @return
     */
    private boolean enableSingleTutor() {
        return iRouter != null && iRouter.getLiveRoom().getStudyRoomVM().enableSingleTutor();
    }

    @Override
    public void setMediaModel(IMediaModel iMediaModel) {
        super.setMediaModel(iMediaModel);
        initStudyStatus(iMediaModel);
        RxUtils.dispose(disposableOfStudyRoomSwitch);
        RxUtils.dispose(disposableOfPublishDefaultStream);
        RxUtils.dispose(disposableOfStudyRank);
        RxUtils.dispose(disposableOfActiveUserStatus);
        RxUtils.dispose(disposableOfTutorApplyReq);
        RxUtils.dispose(disposableOfTutorApplyRes);
        RxUtils.dispose(disposableOfTutorStart);
        RxUtils.dispose(disposableOfTutorEnd);
        RxUtils.dispose(disposableOfTutorGroup);
        RxUtils.dispose(disposableOfForHelp);
        RxUtils.dispose(disposableOfUserAdd);
        RxUtils.dispose(disposableOfUserRemove);
        disposableOfStudyRoomSwitch = iRouter.getSubjectByKey(EventKey.StudyRoomSwitch)
                .ofType(LPConstants.StudyRoomMode.class)
                .doOnNext(studyRoomMode -> {
                    if (isLocalVideo() && !isLeave) {
                        detachAVideo();
                        publishLocalStream();
                        changePlaceholderState(ivPlaceholder.getVisibility() != View.VISIBLE);
                    }
                })
                .subscribe(studyRoomMode -> updateUserStatus(studyRoomMode, false));
        disposableOfPublishDefaultStream = iRouter.getPublishSubjectByKey(EventCode.PublishDefaultStream)
                .observeOn(AndroidSchedulers.mainThread())
                .filter(o -> isLocalVideo())
                .subscribe(o -> {
                    publishLocalStream();
                });
        //只有学生数据没有助教
        disposableOfActiveUserStatus = iRouter.getLiveRoom().getStudyRoomVM().getObservableOfActiveUserStatus()
                .observeOn(AndroidSchedulers.mainThread())
                .flatMap((Function<List<LPStudyUserStatus>, ObservableSource<LPStudyUserStatus>>) Observable::fromIterable)
                .filter(lpStudyUserStatus -> TextUtils.equals(lpStudyUserStatus.userNumber, iMediaModel.getUser().getNumber()))
                .subscribe(lpStudyUserStatus -> {
                    if (!isAdminWindow()) {
                        isLeave = lpStudyUserStatus.isLeave == 1 || lpStudyUserStatus.isHangUp == 1;
                    }
                    isTutorHelping = lpStudyUserStatus.isTutor == 1;
                    selfStudyDuration = lpStudyUserStatus.duration;
                    updateUserStatus(iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode(), true);
                });
        final BehaviorSubject<List<LPStudyUserStatus>> subjectByKey = iRouter.getSubjectByKey(EventKey.StudyRankList);
        disposableOfStudyRank = subjectByKey.flatMap((Function<List<LPStudyUserStatus>, ObservableSource<LPStudyUserStatus>>) Observable::fromIterable)
                .filter(lpStudyUserStatus -> TextUtils.equals(lpStudyUserStatus.userNumber, iMediaModel.getUser().getNumber()))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpStudyUserStatus -> {
                    ImageView imageView = $.id(R.id.window_study_rank).view();
                    final int visibility = $.id(R.id.cl_top_left).view().getVisibility();
                    imageView.setVisibility(lpStudyUserStatus.rank <= 3 && iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout()
                            && iMediaModel.getUser().getType() == LPConstants.LPUserType.Student && visibility == View.VISIBLE ? View.VISIBLE : View.GONE);
                    switch (lpStudyUserStatus.rank) {
                        case 1:
                            imageView.setImageResource(R.drawable.uibase_ic_study_rank_1);
                            break;
                        case 2:
                            imageView.setImageResource(R.drawable.uibase_ic_study_rank_2);
                            break;
                        case 3:
                            imageView.setImageResource(R.drawable.uibase_ic_study_rank_3);
                            break;
                        default:
                            imageView.setVisibility(View.GONE);
                            break;
                    }
                });

        disposableOfTutorApplyReq = iRouter.getLiveRoom().getStudyRoomVM().getObservableOfStuTutorApply()
                .filter(lpStudyRoomTutorModel -> TextUtils.equals(lpStudyRoomTutorModel.to, iRouter.getLiveRoom().getCurrentUser().getUserId()))
                .filter(lpStudyRoomTutorModel -> TextUtils.equals(lpStudyRoomTutorModel.from, getUserId()))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpStudyRoomTutorModel -> showTutorHelpIcon(true));

        disposableOfTutorApplyRes = iRouter.getLiveRoom().getStudyRoomVM().getObservableOfTutorApplyResponse()
                .filter(this::isTargetVideoWindow)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpStudyRoomTutorModel -> {
                    //助教拒绝即提示
                    if (lpStudyRoomTutorModel.tutorState == 0 && !isAdmin() && isLocalVideo()
                            && !TextUtils.equals(lpStudyRoomTutorModel.from, iRouter.getLiveRoom().getCurrentUser().getUserId())) {
                        iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(context.getString(R.string.bjysc_study_room_for_help_deny));
                    }
                    if (isAdmin()) {
                        showTutorHelpIcon(false);
                        showTutorHandleIcon(false);
                    } else {
                        showEndForHelp();
                    }
                });
        //在辅导中的学生或助教的窗口
        disposableOfTutorStart = iRouter.getLiveRoom().getStudyRoomVM().getObservableOfStudyRoomTutorStart()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(this::handleTutorHelpStart);
        disposableOfTutorEnd = iRouter.getLiveRoom().getStudyRoomVM().getObservableOfStudyRoomTutorEnd()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(this::handleTutorHelpEnd);
        disposableOfTutorGroup = iRouter.getLiveRoom().getStudyRoomVM().getObservableOfStudyRoomTutorGroup()
                .flatMap(Observable::fromIterable)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(this::handleTutorHelpStart);
        disposableOfForHelp = iRouter.getSubjectByKey(EventKey.StudyStatusForHelping)
                .ofType(Boolean.class)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> {
                    //寻求辅导后其余按钮消失
                    //学生只支持找一个助教辅导
                    if (aBoolean) {
                        if (!isWaitingHelp) {
                            hideStuTutorHelpBtn();
                        }
                    } else {
                        showStuTutorHelpBtn();
                    }
                });
        //让窗口监听自己的上下台
        disposableOfUserAdd = iRouter.getLiveRoom().getOnlineUserVM().getPublishSubjectOfActiveUserAdd()
                .filter(lpResRoomUserInModel -> isAdminWindow())
                .filter(lpResRoomUserInModel ->
                        TextUtils.equals(lpResRoomUserInModel.getUser().getUserId(), iRouter.getLiveRoom().getCurrentUser().getUserId()))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpResRoomUserInModel -> showStuTutorHelpBtn());
        disposableOfUserRemove = iRouter.getLiveRoom().getOnlineUserVM().getPublishSubjectOfActiveUserRemove()
                .filter(lpResRoomUserInModel -> isAdminWindow())
                .filter(lpResRoomUserInModel ->
                        TextUtils.equals(lpResRoomUserInModel.getUser().getUserId(), iRouter.getLiveRoom().getCurrentUser().getUserId()))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpResRoomUserInModel -> showStuTutorHelpBtn());
        disposables.add(disposableOfTutorStart);
        disposables.add(disposableOfTutorEnd);
        disposables.add(disposableOfTutorGroup);
        disposables.add(disposableOfTutorApplyRes);
        disposables.add(disposableOfTutorApplyReq);
        disposables.add(disposableOfStudyRoomSwitch);
        disposables.add(disposableOfStudyRank);
        disposables.add(disposableOfPublishDefaultStream);
        disposables.add(disposableOfActiveUserStatus);
        disposables.add(disposableOfForHelp);
        disposables.add(disposableOfUserAdd);
        disposables.add(disposableOfUserRemove);
    }

    /**
     * 是否是目标窗口
     *
     * @param lpStudyRoomTutorModel
     * @return
     */
    private boolean isTargetVideoWindow(LPStudyRoomTutorModel lpStudyRoomTutorModel) {
        if (lpStudyRoomTutorModel == null) {
            return false;
        }
        return TextUtils.equals(getUserId(), lpStudyRoomTutorModel.from) || TextUtils.equals(getUserId(), lpStudyRoomTutorModel.to);
    }

    /**
     * 自己是否在该单人辅导中
     *
     * @param lpStudyRoomTutorModel
     * @return
     */
    private boolean isSelfSingleTutorHelping(LPStudyRoomTutorModel lpStudyRoomTutorModel) {
        if (lpStudyRoomTutorModel == null) {
            return false;
        }
        String selfId = iRouter.getLiveRoom().getCurrentUser().getUserId();
        return TextUtils.equals(selfId, lpStudyRoomTutorModel.from)
                || TextUtils.equals(selfId, lpStudyRoomTutorModel.to);
    }

    @Override
    public boolean isSelfSingleTutorHelping() {
        return isSelfSingleTutorHelping(lpStudyRoomTutorModel);
    }

    @Override
    public boolean isSingleTutorHelping() {
        return isTutorHelping;
    }

    /**
     * 处理收到单人辅导状态，单人辅导组内只拉组内的流
     * 其余相当于手动playAVClose
     * 这里不做过滤，我需要处理每个人的视频状态
     * 在辅导组内不拉组外的流，反之亦然
     *
     * @param lpStudyRoomTutorModel
     */
    private void handleTutorHelpStart(LPStudyRoomTutorModel lpStudyRoomTutorModel) {
        //当前窗口是否在辅导中
        boolean isTargetVideoWindow = isTargetVideoWindow(lpStudyRoomTutorModel);
        if (isTargetVideoWindow) {
            this.lpStudyRoomTutorModel = lpStudyRoomTutorModel;
            //getObservableOfActiveUserStatus只有学生没有助教这里也需要初始化变量isTutorHelping
            isTutorHelping = true;
            if (isLocalVideo() && !isSelfTutor) {
                isSelfTutor = true;
                iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(context.getString(R.string.bjysc_study_room_tutor_enter));
                iRouter.getPublishSubjectByKey(EventCode.AttachVideo).onNext(true);
                iRouter.getPublishSubjectByKey(EventCode.AttachAudio).onNext(true);
            }
        }
        //自己不在辅导中，不拉辅导中的流
        if (!isSelfSingleTutorHelping(lpStudyRoomTutorModel)) {
            if (isTargetVideoWindow) {
                showTutorPlaceholder();
                closeRemoteStreaming();
            }
        } else {
            iRouter.getSubjectByKey(EventKey.StudyStatusTutor).onNext(lpStudyRoomTutorModel.tutorId);
            //自己在辅导中，不拉其余人的流
            if (!isTargetVideoWindow) {
                showTutorPlaceholder();
                closeRemoteStreaming();
            }
        }
        showLabelChange();
    }

    private void showLabelChange() {
        showStuTutorHelpBtn();
        showUserLabel();
        showLeftLabelContainer(iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode());
    }

    private void showTutorPlaceholder() {
        if (isTutorHelping) {
            ivPlaceholder.setImageResource(R.drawable.base_ic_video_tutoring);
            $.id(R.id.tv_video_placeholder).text(R.string.bjysc_study_room_tutor_helping);
            $.id(R.id.tv_video_placeholder).visible();
        } else {
            ivPlaceholder.setImageResource(R.drawable.base_ic_video_studying);
            $.id(R.id.tv_video_placeholder).text(R.string.bjysc_study_room_tutor_studying);
            $.id(R.id.tv_video_placeholder).visible();
        }
        ivPlaceholderOnlyAudio.setVisibility(View.GONE);
        ivPlaceholder.setVisibility(View.VISIBLE);
    }

    /**
     * 处理辅导结束状态
     * 取消不拉其他人流
     *
     * @param lpStudyRoomTutorModel
     */
    private void handleTutorHelpEnd(LPStudyRoomTutorModel lpStudyRoomTutorModel) {
        //结束辅导
        //本地恢复默认推流方式
        if (isLocalVideo() && (this.lpStudyRoomTutorModel != null && TextUtils.equals(lpStudyRoomTutorModel.tutorId, this.lpStudyRoomTutorModel.tutorId))) {
            detachAVideo();
            iRouter.getPublishSubjectByKey(EventCode.PublishDefaultStream).onNext(true);
            iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(context.getString(R.string.bjysc_study_room_tutor_exit));
            isSelfTutor = false;
        }
        String tutorId = getSelfTutorId();
        //自己所在组结束辅导，拉不在辅导中的流
        if (TextUtils.equals(lpStudyRoomTutorModel.tutorId, tutorId)) {
            iRouter.getSubjectByKey(EventKey.StudyStatusTutor).onNext("");
            if (this.lpStudyRoomTutorModel != null && TextUtils.equals(lpStudyRoomTutorModel.tutorId, this.lpStudyRoomTutorModel.tutorId)) {
                //同组辅导窗口不用重新拉流
                this.lpStudyRoomTutorModel = null;
                isTutorHelping = false;
            } else {
                if (!isTutorHelping) {
                    openRemoteStreaming();
                    this.lpStudyRoomTutorModel = null;
                }
            }
        } else {
            //不是自己所在组结束辅导，且自己不在辅导中则拉此窗口的流
            if (this.lpStudyRoomTutorModel != null && TextUtils.equals(lpStudyRoomTutorModel.tutorId, this.lpStudyRoomTutorModel.tutorId)) {
                isTutorHelping = false;
            }
            //结束辅导的组员拉不在辅导中的流
            if (TextUtils.isEmpty(tutorId) && !isTutorHelping) {
                openRemoteStreaming();
                this.lpStudyRoomTutorModel = null;
            }
        }
        showLabelChange();
    }

    /**
     * 不拉流
     */
    private void closeRemoteStreaming() {
        if (isLocalVideo()) {
            return;
        }
        this.remoteVideoStatus = REMOTE_VIDEO_AUDIO_CLOSE;
        placeHolderContainer.setVisibility(View.VISIBLE);
        videoView.setVisibility(View.GONE);
        ivPlaceholderOnlyAudio.setVisibility(View.GONE);
        stopStreaming(getMediaId());
    }

    /**
     * 拉流
     */
    private void openRemoteStreaming() {
        if (isLocalVideo()) {
            return;
        }
        this.remoteVideoStatus = REMOTE_VIDEO_HAND_OPEN;
        placeHolderContainer.setVisibility(View.GONE);
        ivPlaceholderOnlyAudio.setVisibility(View.GONE);
        videoView.setVisibility(View.VISIBLE);
        stopStreaming(getMediaId());
        streamVideo(getMediaId());
    }

    private void showLeftLabelContainer(LPConstants.StudyRoomMode studyRoomMode) {
        $.id(R.id.cl_top_left).visibility(studyRoomMode.isGalleryLayout() ? View.VISIBLE : View.GONE);
        if (!studyRoomMode.isGalleryLayout()) {
            $.id(R.id.window_study_rank).gone();
        }
        if (!isAdminWindow() && isTutorHelping && isSelfSingleTutorHelping()) {
            $.id(R.id.cl_top_left_tutor).visible();
            $.id(R.id.cl_top_left).gone();
            $.id(R.id.window_study_rank).gone();
        } else {
            $.id(R.id.cl_top_left_tutor).gone();
        }
    }

    private void updateUserStatus(LPConstants.StudyRoomMode studyRoomMode, boolean startTimer) {
        showLeftLabelContainer(studyRoomMode);
        //在辅导中不显示下台等按钮
        String tutorId = iRouter.getSubjectValueByKey(EventKey.StudyStatusTutor, String.class, "");
        $.id(R.id.cl_top_right).visibility(TextUtils.isEmpty(tutorId) && !isTutorHelping && isAdmin() && !isAdminWindow() && isLeave && studyRoomMode.isGalleryLayout() ? View.VISIBLE : View.GONE);
        showUserLabel();
        showStuTutorHelpBtn();
        if (startTimer) {
            $.id(R.id.window_study_duration).text(KUtilsKt.convertTime(selfStudyDuration));
            $.id(R.id.window_study_duration_tutor).text(KUtilsKt.convertTime(selfStudyDuration));
            startStudyTimer(selfStudyDuration);
        }
    }

    /**
     * 显示用户标签
     */
    private void showUserLabel() {
        if (mediaModel.getUser().getType() == LPConstants.LPUserType.Teacher || mediaModel.getUser().getType() == LPConstants.LPUserType.Assistant) {
            showAssistantLabel();
        } else {
            if (isLeave) {
                $.id(R.id.window_study_status).text(R.string.bjysc_study_room_status_leave);
                ivPlaceholder.setImageResource(R.drawable.base_ic_video_leave);
                $.id(R.id.tv_video_placeholder).text(R.string.bjysc_study_room_study_status_leave);
                $.id(R.id.window_study_status).view().setBackground(new DrawableBuilder()
                        .solidColor(ContextCompat.getColor(context, R.color.base_warning_color))
                        .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                        .build());
            } else if (isTutorHelping) {
                $.id(R.id.window_study_status).view().setBackground(new DrawableBuilder()
                        .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                        .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                        .build());
                $.id(R.id.window_study_status).text(R.string.bjysc_study_room_tutor_status_helping);
            } else {
                $.id(R.id.window_study_status).view().setBackground(new DrawableBuilder()
                        .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                        .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                        .build());
                $.id(R.id.window_study_status).text(R.string.bjysc_study_room_status_study);
            }
        }
    }

    private void showAssistantLabel() {
        $.id(R.id.window_study_duration).gone();
        final TextView tvStatus = $.id(R.id.window_study_status).view();
        final RelativeLayout.LayoutParams tvParams = (RelativeLayout.LayoutParams) tvStatus.getLayoutParams();
        //橘色的助教标签
        tvStatus.setBackground(new DrawableBuilder()
                .solidColor(0x33FA6400)
                .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .build());
        tvStatus.setTextColor(0xFFFA6400);
        if (isTutorHelping) {
            tvStatus.setTextSize(12);
            tvParams.width = ViewGroup.LayoutParams.WRAP_CONTENT;
            tvParams.height = UtilsKt.getDp(24);
            tvStatus.setLayoutParams(tvParams);
            int padding = UtilsKt.getDp(8);
            tvStatus.setPadding(padding, 0, padding, 0);
            tvStatus.setText(context.getString(R.string.bjysc_study_room_tutor_assistant_helping));
        } else {
            tvParams.width = UtilsKt.getDp(36);
            tvParams.height = UtilsKt.getDp(24);
            tvStatus.setTextSize(12);
            tvStatus.setPadding(0, 0, 0, 0);
            tvStatus.setLayoutParams(tvParams);
            tvStatus.setText(context.getString(R.string.bjysc_role_assistant));
        }
    }


    @Override
    protected void changePlaceholderState(boolean checked) {
        super.changePlaceholderState(checked);
        $.id(R.id.tv_video_placeholder).visibility(iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout()
                && (!isVideoOn || isLeave || isRemoteVideoClose()) ? View.VISIBLE : View.GONE);
        //暂时离开的状态，辅助中和自习室中都能看到
        if (isLeave) {
            final ImageView ivPlaceholder = $.id(R.id.iv_video_placeholder).view();
            ivPlaceholder.setImageResource(R.drawable.base_ic_video_leave);
            $.id(R.id.tv_video_placeholder).text(R.string.bjysc_study_room_study_status_leave);
        } else {
            if (isRemoteVideoClose()) {
                if (!isLocalVideo()) {
                    showTutorPlaceholder();
                }
            } else {
                if (videoState == LPConstants.MediaState.Normal) {
                    final ImageView ivPlaceholder = $.id(R.id.iv_video_placeholder).view();
                    ivPlaceholder.setImageResource(R.drawable.base_ic_video_camera_mute);
                    $.id(R.id.tv_video_placeholder).text(R.string.bjysc_study_room_study_status_camera_mute);
                }
            }
            if (remoteVideoStatus == VideoWindow.REMOTE_VIDEO_HAND_CLOSE && iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout()) {
                $.id(R.id.tv_video_placeholder).text(R.string.bjysc_study_room_study_status_only_audio);
            }
        }
    }

    @Override
    public void setRemoteVideoStatus(int remoteVideoStatus) {
        super.setRemoteVideoStatus(remoteVideoStatus);
        //只听声音
        if (remoteVideoStatus == VideoWindow.REMOTE_VIDEO_HAND_CLOSE && iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout()) {
            $.id(R.id.tv_video_placeholder).visible().text(R.string.bjysc_study_room_study_status_only_audio);
        }
    }

    private boolean isAdminWindow() {
        return getUserModel().getType() == LPConstants.LPUserType.Assistant || getUserModel().getType() == LPConstants.LPUserType.Teacher;
    }

    /**
     * 自习、讨论模式下，非辅导状态的台上学生能求助助教窗口
     * 助教和学生都在非辅导状态
     *
     * @return
     */
    private boolean canShowTutorHelp() {
        String tutorId = getSelfTutorId();
        return !isTutorHelping && !isAdmin() && isAdminWindow() && enableSingleTutor() && TextUtils.isEmpty(tutorId)
                && iRouter.getLiveRoom().getOnlineUserVM().isActiveUser(iRouter.getLiveRoom().getCurrentUser())
                && iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout();
    }

    private void showStuTutorHelpBtn() {
        $.id(R.id.window_video_tutor_help).visibility(canShowTutorHelp() ? View.VISIBLE : View.GONE);
    }

    private void hideStuTutorHelpBtn() {
        $.id(R.id.window_video_tutor_help).gone();
    }

    private void initStudyStatus(IMediaModel iMediaModel) {
        if (!iRouter.getLiveRoom().getStudyRoomVM().isStudyRoom()) {
            return;
        }
        showStuTutorHelpBtn();
        showLeftLabelContainer(iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode());
        if (iMediaModel.getUser().getType() == LPConstants.LPUserType.Teacher || iMediaModel.getUser().getType() == LPConstants.LPUserType.Assistant) {
            showAssistantLabel();
        } else {
            $.id(R.id.cl_top_left).view().setBackground(new DrawableBuilder()
                    .solidColor(0x33000000)
                    .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                    .build());
            $.id(R.id.window_study_kickout).view().setBackground(new DrawableBuilder()
                    .solidColor(ContextCompat.getColor(context, R.color.base_warning_color))
                    .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                    .build());
            $.id(R.id.window_study_seat_down).view().setBackground(new DrawableBuilder()
                    .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                    .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                    .build());
            $.id(R.id.window_study_kickout).clicked(v -> iRouter.getSubjectByKey(EventKey.KickOutConfirm).onNext(iMediaModel.getUser()));
            $.id(R.id.window_study_seat_down).clicked(v -> iRouter.getLiveRoom().getOnlineUserVM().requestRemoveActiveUser(iMediaModel.getUser()));
        }
        //上台有数据
        final List<LPStudyRoomTutorModel> tutorGroup = iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomCurrentTutorGroup();
        for (LPStudyRoomTutorModel lpStudyRoomTutorModel : tutorGroup) {
            handleTutorHelpStart(lpStudyRoomTutorModel);
        }
    }

    /**
     * 结束辅导
     */
    private void endTutorHelp() {
        NoteDialog noteDialog = new NoteDialog();
        noteDialog.setMainText(context.getString(R.string.bjysc_study_room_tutor_end_tip));
        noteDialog.setHideAssistantText(true);
        noteDialog.setOnPositiveClickListener((dialog, checked) -> {
            if (lpStudyRoomTutorModel != null) {
                iRouter.getLiveRoom().getStudyRoomVM().requestTutorEnd(lpStudyRoomTutorModel.tutorId);
            } else {
                LPLogger.e("lpStudyRoomTutorModel == null");
            }
            dialog.dismissAllowingStateLoss();
        });
        noteDialog.setOnNegativeClickListener((dialog, checked) -> dialog.dismissAllowingStateLoss());
        noteDialog.setWidth(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_dialog_container_width));
        noteDialog.setHeight(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_dialog_container_height));
        FragmentActivity activity = (FragmentActivity) context;
        noteDialog.show(activity.getSupportFragmentManager(), "endTutorHelp");
    }

    private void startForHelp() {
        if (isWaitingHelp) {
            return;
        }
        final boolean isForHelping = iRouter.getSubjectValueByKey(EventKey.StudyStatusForHelping, Boolean.class, false);
        if (isForHelping) {
            return;
        }
        isWaitingHelp = true;
        iRouter.getSubjectByKey(EventKey.StudyStatusForHelping).onNext(true);
        ImageView imageView = $.id(R.id.window_video_tutor_help).view();
        imageView.setImageResource(R.drawable.bjysc_ic_help_waiting);
        $.id(R.id.window_video_tutor_help_waiting).visible();
        iRouter.getLiveRoom().getStudyRoomVM().requestTutorApply(getUserId(), TUTOR_HELP_WAITING_DURATION, new OnCountDownListener() {
            @Override
            public void onTimeOut() {
                if (isDestroyed()) {
                    return;
                }
                endForHelp();
            }

            @Override
            public void onTimeCountDown(int counter, int timeOut) {
                if (isDestroyed()) {
                    return;
                }
                $.id(R.id.window_video_tutor_help_waiting).text(context.getString(R.string.bjysc_study_room_tutor_waiting_seconds, timeOut - counter));
            }
        });
    }

    private void showEndForHelp() {
        if (!isWaitingHelp) {
            return;
        }
        isWaitingHelp = false;
        iRouter.getSubjectByKey(EventKey.StudyStatusForHelping).onNext(false);
        ImageView imageView = $.id(R.id.window_video_tutor_help).view();
        imageView.setImageResource(R.drawable.bjysc_ic_help);
        $.id(R.id.window_video_tutor_help_waiting).invisible();
    }

    private void endForHelp() {
        showEndForHelp();
        iRouter.getLiveRoom().getStudyRoomVM().cancelTutorApply(iRouter.getLiveRoom().getCurrentUser().getUserId());
    }

    /**
     * @param seconds
     */
    @SuppressLint("DefaultLocale")
    private void startStudyTimer(long seconds) {
        RxUtils.dispose(disposableOfDurationTimer);
        disposableOfDurationTimer = Observable.interval(1, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aLong -> {
                    $.id(R.id.window_study_duration).text(KUtilsKt.convertTime(aLong + seconds));
                    $.id(R.id.window_study_duration_tutor).text(KUtilsKt.convertTime(aLong + seconds));
                });
    }

    @Override
    public void publishLocalStream() {
        boolean audioOn = false;
        if (iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode() == LPConstants.StudyRoomMode.Discuss) {
            audioOn = true;
        } else if (iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode() == LPConstants.StudyRoomMode.Tutor) {
            audioOn = iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher;
        }
        publishLocalStream(audioOn);
    }

    /**
     * 自习室权限说明：
     * 自习室模式
     * 学生：默认只开视频，可控制视频
     * 助教：默认只开视频，可控制学生视频
     * 讨论模式
     * 学生：默认开音视频，可控制音视频
     * 助教：默认开音视频，可控制音视频，可控制学生音视频
     * 辅导模式
     * 同小班课
     * 学生：默认开视频，可控制视频
     * 助教、老师：可控制学生音视频
     * 助教：默认开视频
     * 老师：默认开音视频
     *
     * @param audioOn
     */
    @Override
    public void publishLocalStream(boolean audioOn) {
        //挂机
        final Boolean isHangUp = iRouter.getValueByKey(EventKey.StudyStatusHangUp, Boolean.class);
        if (isHangUp != null && isHangUp.equals(true)) {
            return;
        }
        LPConstants.StudyRoomMode studyRoomMode = iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode();
        //自习模式
        if (studyRoomMode == LPConstants.StudyRoomMode.SelfStudy) {
            audioOn = false;
        }
        if (studyRoomMode == LPConstants.StudyRoomMode.Discuss) {
            audioOn = true;
        }
        super.publishLocalStream(audioOn);
    }

    @Override
    public boolean isGalleryLayout() {
        return super.isGalleryLayout() || iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout();
    }

    @Override
    public boolean isTouchAble() {
        return !isLeave && (super.isTouchAble() || iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout());
    }

    //自习模式不打开音频,讨论和辅导和打开音频
    @Override
    protected boolean canAttachAudio() {
        return isTutorHelping || iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode() == LPConstants.StudyRoomMode.Discuss
                || iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode() == LPConstants.StudyRoomMode.Tutor;
    }

    @Override
    public void showTutorHelpIcon(boolean isShow) {
        if (!isAdmin()) {
            return;
        }
        if (ivTutorHelp != null) {
            ivTutorHelp.setVisibility(isShow ? View.VISIBLE : View.GONE);
        }
    }

    @Override
    public void showTutorHandleIcon(boolean isShow) {
        if (!isAdmin()) {
            return;
        }
        if (tvTutorAgree != null) {
            tvTutorAgree.setVisibility(isShow ? View.VISIBLE : View.GONE);
        }
        if (tvTutorDisagree != null) {
            tvTutorDisagree.setVisibility(isShow ? View.VISIBLE : View.GONE);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        RxUtils.dispose(disposableOfDurationTimer);
    }
}
