package com.baijiayun.groupclassui.window.video;

import static com.baijiayun.livebase.context.LPConstants.RoomLayoutMode.BOARD;

import android.content.Context;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.log.LPLogger;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventCode;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.model.VideoMenuControlModel;
import com.baijiayun.groupclassui.util.FECompatUtil;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.context.LPError;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

/**
 * 视频菜单更多弹窗
 * Created by Dujuncan on 2018/11/8
 */
public class VideoMenuMoreWindow extends InteractiveBaseWindow {
    private final String TAG = VideoMenuMoreWindow.class.getCanonicalName();
    private WeakReference<VideoMenuControlModel> weakReference;
    int menuWidthDp = 80;//宽度
    int rightMarginDp = 0;//右边距离菜单边距
    private final Random random = new Random();

    public VideoMenuMoreWindow(Context context, int menuWidthDp, VideoMenuControlModel videoMenuControlModel) {
        super(context);
        this.menuWidthDp = menuWidthDp;
        weakReference = new WeakReference<>(videoMenuControlModel);

        FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(UtilsKt.getDp(menuWidthDp), ViewGroup.LayoutParams.WRAP_CONTENT);
        params.leftMargin = videoMenuControlModel.getCurrX() - UtilsKt.getDp(menuWidthDp) - UtilsKt.getDp(rightMarginDp);
        params.topMargin = videoMenuControlModel.getCurrY();
        view.setLayoutParams(params);

        VideoWindow videoWindow = videoMenuControlModel.getVideoWindow();
        String uid = videoWindow.getUserId();
        $.id(R.id.window_video_menu_more_bg_container).view().setBackground(new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_brand_container_color))
                .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .build());
        if (uid.equals(iRouter.getLiveRoom().getCurrentUser().getUserId())) {
            if (iRouter.getLiveRoom().getSpeakQueueVM().enableAttachPhoneCamera()) {
                $.id(R.id.window_video_menu_more_scanner_camera).visible();
            }
            //自己的菜单,学生和老师看到的都是一样
            if (iRouter.getLiveRoom().isClassStarted()) {
                if (!iRouter.getLiveRoom().getSpeakQueueVM().isReplacedUser()) {
                    $.id(R.id.window_video_menu_more_switch_camera).visible();
                    $.id(R.id.window_video_menu_more_close_camera).visible();
                    if (iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout()) {
                        $.id(R.id.window_video_menu_more_seat_down).visible();
                    } else {
                        $.id(R.id.window_video_menu_more_seat_down).gone();
                    }
                    if (iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode() == LPConstants.StudyRoomMode.Discuss) {
                        $.id(R.id.window_video_menu_more_close_mic).visible();
                    } else {
                        $.id(R.id.window_video_menu_more_close_mic).gone();
                    }
                    $.id(R.id.window_video_menu_more).visible();
                }
            }
            if (isAdmin()) {
                $.id(R.id.window_video_menu_more_toggle_horizontal_mirror_mode).gone();
                $.id(R.id.window_video_menu_more_toggle_vertical_mirror_mode).gone();
            }
        } else {
            //不是自己的菜单
            //助教或老师 看学生的菜单(非Android和iOS)
            boolean hideOnlyAudio = (iRouter.getLiveRoom().getCurrentRoomLayout() == BOARD
                    && iRouter.getLiveRoom().getDisableOtherStudentVideoData().isDisable()) || iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode() == LPConstants.StudyRoomMode.SelfStudy;
            // 坐席区 & 教室内开启只看老师 两者满足时 不允许学生主动开启视频
            boolean isMainVideo = videoWindow.getMediaSourceType() == LPConstants.MediaSourceType.MainCamera;
            $.id(R.id.window_video_menu_more_only_audio).visibility((hideOnlyAudio || !isMainVideo || videoWindow.disablePullStream()) ? View.GONE : View.VISIBLE);
            if (videoWindow.getUserModel().getType() == LPConstants.LPUserType.Student &&
                    iRouter.getLiveRoom().getCurrentUser().getType() != LPConstants.LPUserType.Student) {
                if (iRouter.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE
                        && videoWindow.getUserModel().getEndType() != LPConstants.LPEndType.Android
                        && videoWindow.getUserModel().getEndType() != LPConstants.LPEndType.iOS) {
                    $.id(R.id.window_video_menu_more_assist_camera).visible();
                }
                $.id(R.id.window_video_menu_more_screen_share).visible();
            }
            switch (iRouter.getLiveRoom().getCurrentUser().getType()) {
                case Assistant:
                    if (videoWindow.getUserModel().getType() == LPConstants.LPUserType.Student) {
                        if (iRouter.getLiveRoom().isClassStarted()) {
                            if (!iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout()) {
                                $.id(R.id.window_video_menu_more_draw_auth).visible();
                                $.id(R.id.window_video_menu_more_ppt_auth).visible();
                                $.id(R.id.window_video_menu_more_close_camera).visible();
                                $.id(R.id.window_video_menu_more_close_mic).visible();
                                $.id(R.id.window_video_menu_more_seat_down).visible();
                                $.id(R.id.window_video_menu_more_kickout).visible();
                            } else {
                                $.id(R.id.window_video_menu_more_draw_auth).gone();
                                $.id(R.id.window_video_menu_more_ppt_auth).gone();
                                if (iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode() == LPConstants.StudyRoomMode.SelfStudy) {
                                    $.id(R.id.window_video_menu_more_close_mic).gone();
                                } else {
                                    $.id(R.id.window_video_menu_more_close_mic).visible();
                                }
                                $.id(R.id.window_video_menu_more_close_camera).visible();
                                $.id(R.id.window_video_menu_more_seat_down).visible();
                                $.id(R.id.window_video_menu_more_kickout).visible();
                            }
                        }
                    } else if (videoWindow.getUserModel().getType() == LPConstants.LPUserType.Assistant) {
                        $.id(R.id.window_video_menu_more_close_camera).visible();
                        $.id(R.id.window_video_menu_more_seat_down).visible();
                    } else if (videoWindow.getUserModel().getType() == LPConstants.LPUserType.Teacher) {
                        //助教不能切换老师的镜像
                        $.id(R.id.window_video_menu_more_toggle_horizontal_mirror_mode).gone();
                        $.id(R.id.window_video_menu_more_toggle_vertical_mirror_mode).gone();
                    }
                    break;
                case Teacher:
                    //老师看别人的菜单,自习室中只有辅导模式才有老师
                    if (videoWindow.getUserModel().getType() != LPConstants.LPUserType.Assistant) {
                        if (iRouter.getLiveRoom().isClassStarted()) {
                            $.id(R.id.window_video_menu_more_draw_auth).visible();
                            $.id(R.id.window_video_menu_more_ppt_auth).visible();
                            $.id(R.id.window_video_menu_more_seat_down).visible();
                            $.id(R.id.window_video_menu_more_kickout).visible();
                        }
                    }
                    if (iRouter.getLiveRoom().getCurrentRoomLayout() == BOARD || !iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout()) {
                        $.id(R.id.window_video_menu_more_move).visible();
                    }
                    if (iRouter.getLiveRoom().isClassStarted()) {
                        $.id(R.id.window_video_menu_more_close_camera).visible()
                                .id(R.id.window_video_menu_more_close_mic).visible();
                    }
                    break;
                default:
                    $.id(R.id.window_video_menu_more_toggle_horizontal_mirror_mode).gone();
                    $.id(R.id.window_video_menu_more_toggle_vertical_mirror_mode).gone();
                    break;
            }
        }
        if(!iRouter.getLiveRoom().getPartnerConfig().enableSwitchMirrorMode) {
            $.id(R.id.window_video_menu_more_toggle_horizontal_mirror_mode).gone();
            $.id(R.id.window_video_menu_more_toggle_vertical_mirror_mode).gone();
        }
        //在辅导中没有上下台
        String tutorId = iRouter.getSubjectValueByKey(EventKey.StudyStatusTutor, String.class, "");
        if (!TextUtils.isEmpty(tutorId)) {
            $.id(R.id.window_video_menu_more_seat_down).gone();
        }
        $.id(R.id.window_video_menu_more_scanner_camera)
                .text(iRouter.getLiveRoom().getSpeakQueueVM().isReplacedUser() ?
                        getString(R.string.base_live_scanner_camera_end) : getString(R.string.base_live_scanner_camera_start))
                .clicked(v -> {
                    //显示二维码
                    iRouter.getSubjectByKey(EventKey.ShowQrCode).onNext(!iRouter.getLiveRoom().getSpeakQueueVM().hasAsCameraUser());
                    hideMenu(videoMenuControlModel);
                })
                .id(R.id.window_video_menu_more_switch_camera)
                .clicked(v -> {
                    iRouter.getLiveRoom().getRecorder().switchCamera();
                    hideMenu(videoMenuControlModel);
                })
                .id(R.id.window_video_menu_more_close_camera)
                .text(videoWindow.isVideoOn() ? R.string.bjysc_video_menu_more_close_camera : R.string.bjysc_video_menu_more_open_camera)
                .clicked(v -> {
                    if (uid.equals(iRouter.getLiveRoom().getCurrentUser().getUserId())) {
                        iRouter.getPublishSubjectByKey(EventCode.AttachVideo).onNext(!videoWindow.isVideoOn());
                    } else {
                        iRouter.getLiveRoom().getSpeakQueueVM().controlRemoteUser(uid, !videoWindow.isVideoOn(), videoWindow.isAudioOn());
                    }
                    hideMenu(videoMenuControlModel);
                })
                .id(R.id.window_video_menu_more_close_mic)
                .text(videoWindow.isAudioOn() ? R.string.bjysc_video_menu_more_close_mic : R.string.bjysc_video_menu_more_open_mic)
                .clicked(v -> {
                    if (uid.equals(iRouter.getLiveRoom().getCurrentUser().getUserId())) {
                        iRouter.getPublishSubjectByKey(EventCode.AttachAudio).onNext(!videoWindow.isAudioOn());
                    } else {
                        iRouter.getLiveRoom().getSpeakQueueVM().controlRemoteUser(uid, videoWindow.isVideoOn(), !videoWindow.isAudioOn());
                    }
                    hideMenu(videoMenuControlModel);
                })
                .id(R.id.window_video_menu_more_ppt_auth)
                .text(videoWindow.isPPTAuthOn() ? R.string.bjysc_video_menu_more_ppt_auth_back : R.string.bjysc_video_menu_more_ppt_auth)
                .clicked(v -> {
                    LPError lpError = iRouter.getLiveRoom().getDocListVM().requestStudentPPTAuthChange(!videoWindow.isPPTAuthOn(), videoWindow.getUserModel().getNumber());
                    if (null == lpError) {
                        videoWindow.setPPTAuthOn(!videoWindow.isPPTAuthOn());
                    }
                    hideMenu(videoMenuControlModel);
                })
                .id(R.id.window_video_menu_more_screen_share)
                .text(videoWindow.isScreenShareOn() ? R.string.bjysc_video_menu_more_srceen_share_back : R.string.bjysc_video_menu_more_srceen_share)
                .clicked(v -> {
                    iRouter.getLiveRoom().getMediaVM().requestStudentScreenShareChange(!videoWindow.isScreenShareOn(), videoWindow.getUserModel());
                    hideMenu(videoMenuControlModel);
                })
                .id(R.id.window_video_menu_more_assist_camera)
                .text(videoWindow.isAssistCameraOn() ? R.string.bjysc_video_menu_more_assist_camera_back : R.string.bjysc_video_menu_more_assist_camera)
                .clicked(v -> {
                    iRouter.getLiveRoom().getMediaVM().requestStudentExtCameraChange(!videoWindow.isAssistCameraOn(), videoWindow.getUserModel());
                    hideMenu(videoMenuControlModel);
                    if (videoWindow.isAssistCameraOn()) {
                        //userId拼一个ext_camera的mediaId
                        iRouter.getSubjectByKey(EventKey.CloseVideoWindow).onNext(FECompatUtil.getIOSMediaIdFromMediaId(FECompatUtil.getMediaId(videoWindow.getUserId(), LPConstants.MediaSourceType.ExtCamera.getValue())));
                    }
                })
                .id(R.id.window_video_menu_more_only_audio)
                .text(videoWindow.isRemoteVideoClose() ? R.string.bjysc_video_menu_more_open_video : R.string.bjysc_video_menu_more_only_audio)
                .clicked(v -> {
                    if (videoWindow.isDestroyed()) {
                        return;
                    }
                    if (videoWindow.isRemoteVideoClose()) {
                        LPLogger.d("打开画面");
                    } else {
                        LPLogger.d("只听声音");
                    }
                    videoWindow.setRemoteVideoStatus(videoWindow.isRemoteVideoClose() ? VideoWindow.REMOTE_VIDEO_HAND_OPEN : VideoWindow.REMOTE_VIDEO_HAND_CLOSE);
                    videoWindow.muteRemoteVideo(videoWindow.isRemoteVideoClose());
                    hideMenu(videoMenuControlModel);
                })
                .id(R.id.window_video_menu_more_draw_auth)
                .text(videoWindow.isDrawAuthOn() ? R.string.bjysc_video_menu_more_auth_draw_back : R.string.bjysc_video_menu_more_draw_auth)
                .clicked(v -> {
                    hideMenu(videoMenuControlModel);
                    LPError lpError = iRouter.getLiveRoom().getSpeakQueueVM().requestStudentDrawingAuthChange(!videoWindow.isDrawAuthOn(), videoWindow.getUserModel().getNumber());
                    if (lpError != null) {
                        return;
                    }
                    Map<String, String> authPaintColors = iRouter.getLiveRoom().getAuthPaintColor();
                    Map<String, String> cachePaintColors = iRouter.getMapValueByKey(
                            EventKey.CachePaintColorMap, String.class, String.class);
                    if (authPaintColors == null) {
                        authPaintColors = new HashMap<>();
                    }
                    if (cachePaintColors == null) {
                        cachePaintColors = new HashMap<>();
                    }
                    if (videoWindow.isDrawAuthOn()) {
                        //收回权限时，保留信令中的颜色
                        for (String color : authPaintColors.keySet()) {
                            if (authPaintColors.get(color).equals(videoWindow.getUserModel().getNumber())) {
                                cachePaintColors.put(color, videoWindow.getUserModel().getNumber());
                                iRouter.setObjectByKey(EventKey.CachePaintColorMap, cachePaintColors);
                            }
                        }
                    } else {
                        //授权，从调色板分配未用的颜色给学生。
                        String[] colors = context.getResources().getStringArray(R.array.toolbar_paint_color);
                        if (!authPaintColors.containsValue(videoWindow.getUserModel().getNumber())) {
                            String allocateColor = allocateColor(new ArrayList<>(Arrays.asList(colors)), cachePaintColors);
                            authPaintColors.put(allocateColor, videoWindow.getUserModel().getNumber());
                        }
                        iRouter.getLiveRoom().requestAuthPaintColor(authPaintColors);
                    }
                    videoWindow.setDrawAuthOn(!videoWindow.isDrawAuthOn());
                })
                .id(R.id.window_video_menu_more_move)
                .text(videoWindow.isJoinBlackboard() ? R.string.bjysc_video_menu_more_move_to_seat : R.string.bjysc_video_menu_more_move_to_blackboard)
                .clicked(v -> {
                    if (!isAdmin()) {
                        return;
                    }
                    if (videoWindow.isJoinBlackboard()) {
                        //返回座位
                        iRouter.getSubjectByKey(EventKey.CloseVideoWindow).onNext(videoWindow.getMediaId());
                    } else {
                        //进入黑板
                        iRouter.getSubjectByKey(EventKey.VideoWindowToBlackboard).onNext(videoWindow);
                    }
                    hideMenu(videoMenuControlModel);
                })
                .id(R.id.window_video_menu_more_seat_down)
                .clicked(v -> {
                    iRouter.getLiveRoom().getOnlineUserVM().requestRemoveActiveUser(videoWindow.getUserModel());
                    hideMenu(videoMenuControlModel);
                })
                .id(R.id.window_video_menu_more_kickout)
                .clicked(v -> iRouter.getSubjectByKey(EventKey.KickOutConfirm).onNext(videoWindow.getUserModel()))
                .id(R.id.window_video_menu_more)
                .clicked(v -> {
                    iRouter.getSubjectByKey(EventKey.SettingWindow).onNext(true);
                    hideMenu(videoMenuControlModel);
                })
                .id(R.id.window_video_menu_more_toggle_horizontal_mirror_mode)
                .clicked(v -> {
                    boolean isHorizontalMirror = iRouter.getLiveRoom().getSpeakQueueVM().getHorizontalMirrorModeSet().contains(videoWindow.getUserModel().getNumber());
                    boolean isVerticalMirror = iRouter.getLiveRoom().getSpeakQueueVM().getVerticalMirrorModeSet().contains(videoWindow.getUserModel().getNumber());
                    iRouter.getLiveRoom().requestMirrorMode(videoWindow.getUserModel().getNumber(), videoWindow.getUserId(), !isHorizontalMirror, isVerticalMirror);
                    hideMenu(videoMenuControlModel);
                })
                .id(R.id.window_video_menu_more_toggle_vertical_mirror_mode)
                .clicked(v -> {
                    boolean isHorizontalMirror = iRouter.getLiveRoom().getSpeakQueueVM().getHorizontalMirrorModeSet().contains(videoWindow.getUserModel().getNumber());
                    boolean isVerticalMirror = iRouter.getLiveRoom().getSpeakQueueVM().getVerticalMirrorModeSet().contains(videoWindow.getUserModel().getNumber());
                    iRouter.getLiveRoom().requestMirrorMode(videoWindow.getUserModel().getNumber(), videoWindow.getUserId(), isHorizontalMirror, !isVerticalMirror);
                    hideMenu(videoMenuControlModel);
                });
    }

    @Override
    protected View onCreateView(Context context) {
        setNeedShowShadow(true);
        return LayoutInflater.from(context).inflate(R.layout.window_video_menu_more, null);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        weakReference.clear();
        weakReference = null;
    }

    public void repositionMenu() {
        VideoMenuControlModel videoMenuControlModel = weakReference.get();
        VideoWindow videoWindow = videoMenuControlModel.getVideoWindow();
        FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(UtilsKt.getDp(menuWidthDp), ViewGroup.LayoutParams.WRAP_CONTENT);
        params.leftMargin = (videoWindow.getMenuPosition()[0] - UtilsKt.getDp(menuWidthDp) - UtilsKt.getDp(rightMarginDp));
        params.topMargin = videoWindow.getMenuPosition()[1];
        view.setLayoutParams(params);
    }

    private void hideMenu(VideoMenuControlModel videoMenuControlModel) {
        videoMenuControlModel.getVideoWindow().setMenuState(false);
        iRouter.getSubjectByKey(EventKey.ShowVideoMenu).onNext(new VideoMenuControlModel());
    }

    private String allocateColor(List<String> colors, Map<String, String> cachePaintColors) {
        String result;
        Map<String, String> authPaintColors = iRouter.getLiveRoom().getAuthPaintColor();

        if (authPaintColors.size() == 0) {
            //没有分配过画笔颜色，直接从调色板中随机取颜色
            int index = random.nextInt(colors.size());
            result = colors.get(index);
        } else {
            //分配过画笔颜色,过滤已分配的颜色
            for (String color : authPaintColors.keySet()) {
                colors.remove(color);
            }
            for (String color : cachePaintColors.keySet()) {
                colors.remove(color);
            }
            if (colors.size() == 0) {
                //没有颜色分配时，随机移除缓存颜色中的一个，分配给其他学生。
                ArrayList<String> list = new ArrayList<>(cachePaintColors.keySet());
                int randomIndex = random.nextInt(list.size());
                result = list.get(randomIndex);
                cachePaintColors.remove(result);
                iRouter.setObjectByKey(EventKey.CachePaintColorMap, cachePaintColors);
            } else {
                int index = random.nextInt(colors.size());
                result = colors.get(index);
            }
        }
        return result;
    }
}